@@ -111,7 +111,7 @@ static struct my_option my_long_options[]=
    &opt_default_auth, &opt_default_auth, 0,
    GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
   {"force", 'f', "Force execution of mysqlcheck even if mysql_upgrade "
-   "has already been executed for the current version of MySQL.",
+   "has already been executed for the current version of MariaDB.",
    &opt_force, &opt_force, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
   {"host", 'h', "Connect to host.", 0,
    0, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
@@ -135,13 +135,8 @@ static struct my_option my_long_options[]=
    "built-in default (" STRINGIFY_ARG(MYSQL_PORT) ").",
    0, 0, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
   {"protocol", OPT_MYSQL_PROTOCOL,
-   "The protocol to use for connection (tcp, socket, pipe, memory).",
+   "The protocol to use for connection (tcp, socket, pipe).",
    0, 0, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
-#ifdef HAVE_SMEM
-  {"shared-memory-base-name", OPT_SHARED_MEMORY_BASE_NAME,
-   "Base name of shared memory.", 0,
-   0, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
-#endif
   {"silent", OPT_SILENT, "Print less information", &opt_silent,
    &opt_silent, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
   {"socket", 'S', "The socket file to use for connection.",
@@ -153,7 +148,7 @@ static struct my_option my_long_options[]=
    &opt_systables_only, &opt_systables_only, 0,
    GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
 #define USER_OPT (array_elements(my_long_options) - 6)
-  {"user", 'u', "User for login.", &opt_user,
+  {"user", 'u', "User for login if not current user.", &opt_user,
    &opt_user, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
   {"verbose", 'v', "Display more output about the process; Using it twice will print connection argument; Using it 3 times will print out all CHECK, RENAME and ALTER TABLE during the check phase.",
    &opt_not_used, &opt_not_used, 0, GET_BOOL, NO_ARG, 1, 0, 0, 0, 0, 0},
@@ -178,6 +173,7 @@ static const char *load_default_groups[]=
 {
   "client",          /* Read settings how to connect to server */
   "mysql_upgrade",   /* Read special settings for mysql_upgrade */
+  "mariadb-upgrade", /* Read special settings for mysql_upgrade */
   "client-server",   /* Reads settings common between client & server */
   "client-mariadb",  /* Read mariadb unique client settings */
   0
@@ -272,12 +268,12 @@ static void add_one_option_cnf_file(DYNAMIC_STRING *ds,
 }
 
 static my_bool
-get_one_option(int optid, const struct my_option *opt,
-               char *argument)
+get_one_option(const struct my_option *opt, char *argument,
+               const char *filename __attribute__((unused)))
 {
   my_bool add_option= TRUE;
 
-  switch (optid) {
+  switch (opt->id) {
 
   case '?':
     printf("%s  Ver %s Distrib %s, for %s (%s)\n",
@@ -321,7 +317,7 @@ get_one_option(int optid, const struct my_option *opt,
   case 'b': /* --basedir   */
   case 'd': /* --datadir   */
     fprintf(stderr, "%s: the '--%s' option is always ignored\n",
-            my_progname, optid == 'b' ? "basedir" : "datadir");
+            my_progname, opt->id == 'b' ? "basedir" : "datadir");
     /* FALLTHROUGH */
 
   case 'k':                                     /* --version-check */
@@ -354,7 +350,6 @@ get_one_option(int optid, const struct my_option *opt,
   case 'P': /* --port */
   case 'S': /* --socket */
   case OPT_MYSQL_PROTOCOL: /* --protocol */
-  case OPT_SHARED_MEMORY_BASE_NAME: /* --shared-memory-base-name */
   case OPT_PLUGIN_DIR:                          /* --plugin-dir */
   case OPT_DEFAULT_AUTH:                        /* --default-auth */
     add_one_option_cmd_line(&conn_args, opt, argument);
@@ -388,11 +383,20 @@ static int run_command(char* cmd,
   if (opt_verbose >= 4)
     puts(cmd);
 
-  if (!(res_file= popen(cmd, "r")))
+  if (!(res_file= my_popen(cmd, "r")))
     die("popen(\"%s\", \"r\") failed", cmd);
 
   while (fgets(buf, sizeof(buf), res_file))
   {
+#ifdef _WIN32
+    /* Strip '\r' off newlines. */
+    size_t len = strlen(buf);
+    if (len > 1 && buf[len - 2] == '\r' && buf[len - 1] == '\n')
+    {
+      buf[len - 2] = '\n';
+      buf[len - 1] = 0;
+    }
+#endif
     DBUG_PRINT("info", ("buf: %s", buf));
     if(ds_res)
     {
@@ -406,7 +410,7 @@ static int run_command(char* cmd,
     }
   }
 
-  error= pclose(res_file);
+  error= my_pclose(res_file);
   return WEXITSTATUS(error);
 }
 
@@ -503,7 +507,7 @@ static void find_tool(char *tool_executable_name, const char *tool_name,
 
     len= (int)(last_fn_libchar - self_name);
 
-    my_snprintf(tool_executable_name, FN_REFLEN, "%.*b%c%s",
+    my_snprintf(tool_executable_name, FN_REFLEN, "%.*s%c%s",
                 len, self_name, FN_LIBCHAR, tool_name);
   }
 
@@ -558,8 +562,7 @@ static int run_query(const char *query, DYNAMIC_STRING *ds_res,
   DBUG_PRINT("enter", ("query: %s", query));
   if ((fd= create_temp_file(query_file_path, 
                             opt_tmpdir[0] ? opt_tmpdir : NULL,
-                            "sql", O_CREAT | O_SHARE | O_RDWR,
-                            MYF(MY_WME))) < 0)
+                            "sql", O_SHARE, MYF(MY_WME))) < 0)
     die("Failed to create temporary file for defaults");
 
   /*
@@ -673,7 +676,7 @@ static int get_upgrade_info_file_name(char* name)
 
 */
 
-static int upgrade_already_done(void)
+static int upgrade_already_done(myf flags)
 {
   FILE *in;
   char upgrade_info_file[FN_REFLEN]= {0};
@@ -681,17 +684,23 @@ static int upgrade_already_done(void)
   if (get_upgrade_info_file_name(upgrade_info_file))
     return 0; /* Could not get filename => not sure */
 
-  if (!(in= my_fopen(upgrade_info_file, O_RDONLY, MYF(0))))
+  if (!(in= my_fopen(upgrade_info_file, O_RDONLY, flags)))
     return 0; /* Could not open file => not sure */
 
   bzero(upgrade_from_version, sizeof(upgrade_from_version));
   if (!fgets(upgrade_from_version, sizeof(upgrade_from_version), in))
   {
-    /* Ignore, will be detected by strncmp() below */
+    /* Preserve errno for caller */
+    int save_errno= errno;
+    (void) my_fclose(in, flags);
+    errno= save_errno;
+    return 0;
   }
 
-  my_fclose(in, MYF(0));
+  if (my_fclose(in, flags))
+    return 0;
 
+  errno= 0;
   return (strncmp(upgrade_from_version, MYSQL_SERVER_VERSION,
                   sizeof(MYSQL_SERVER_VERSION)-1)==0);
 }
@@ -720,23 +729,24 @@ static void create_mysql_upgrade_info_file(void)
   {
     fprintf(stderr,
             "Could not create the upgrade info file '%s' in "
-            "the MySQL Servers datadir, errno: %d\n",
+            "the MariaDB Servers datadir, errno: %d\n",
             upgrade_info_file, errno);
     return;
   }
 
   /* Write new version to file */
-  fputs(MYSQL_SERVER_VERSION, out);
-  my_fclose(out, MYF(0));
+  my_fwrite(out, (uchar*) MYSQL_SERVER_VERSION,
+            sizeof(MYSQL_SERVER_VERSION), MY_WME);
+  my_fclose(out, MYF(MY_WME));
 
   /*
     Check if the upgrad_info_file was properly created/updated
     It's not a fatal error -> just print a message if it fails
   */
-  if (!upgrade_already_done())
+  if (!upgrade_already_done(MY_WME))
     fprintf(stderr,
-            "Could not write to the upgrade info file '%s' in "
-            "the MySQL Servers datadir, errno: %d\n",
+            "Upgrade file '%s' was not properly created. "
+            "Got error errno while checking file content: %d\n",
             upgrade_info_file, errno);
   return;
 }
@@ -808,7 +818,7 @@ static my_bool is_mysql()
       strstr(ds_events_struct.str, "IGNORE_BAD_TABLE_OPTIONS") != NULL)
     ret= FALSE;
   else
-    verbose("MySQL upgrade detected");
+    verbose("MariaDB upgrade detected");
 
   dynstr_free(&ds_events_struct);
   return(ret);
@@ -878,10 +888,14 @@ static int run_mysqlcheck_fixnames(void)
 
 static const char *expected_errors[]=
 {
+  "ERROR 1051", /* Unknown table */
   "ERROR 1060", /* Duplicate column name */
   "ERROR 1061", /* Duplicate key name */
   "ERROR 1054", /* Unknown column */
+  "ERROR 1146", /* Table does not exist */
   "ERROR 1290", /* RR_OPTION_PREVENTS_STATEMENT */
+  "ERROR 1347", /* 'mysql.user' is not of type 'BASE TABLE' */
+  "ERROR 1348", /* Column 'Show_db_priv' is not updatable */
   0
 };
 
@@ -958,8 +972,9 @@ static int install_used_engines(void)
 {
   char buf[512];
   DYNAMIC_STRING ds_result;
-  const char *query = "SELECT DISTINCT LOWER(engine) FROM information_schema.tables"
-                      " WHERE table_comment LIKE 'Unknown storage engine%'";
+  const char *query = "SELECT DISTINCT LOWER(engine) AS c1 FROM information_schema.tables"
+                      " WHERE table_comment LIKE 'Unknown storage engine%'"
+                      " ORDER BY c1";
 
   if (opt_systables_only || !from_before_10_1())
   {
@@ -999,64 +1014,6 @@ static int install_used_engines(void)
   return 0;
 }
 
-static int check_slave_repositories(void)
-{
-  DYNAMIC_STRING ds_result;
-  int row_count= 0;
-  int error= 0;
-  const char *query = "SELECT COUNT(*) AS c1 FROM mysql.slave_master_info";
-
-  if (init_dynamic_string(&ds_result, "", 512, 512))
-    die("Out of memory");
-
-  run_query(query, &ds_result, TRUE);
-
-  if (ds_result.length)
-  {
-    row_count= atoi((char *)ds_result.str);
-    if (row_count)
-    {
-      fprintf(stderr,"Slave info repository compatibility check:"
-              " Found data in `mysql`.`slave_master_info` table.\n");
-      fprintf(stderr,"Warning: Content of `mysql`.`slave_master_info` table"
-              " will be ignored as MariaDB supports file based info "
-              "repository.\n");
-      error= 1;
-    }
-  }
-  dynstr_free(&ds_result);
-
-  query = "SELECT COUNT(*) AS c1 FROM mysql.slave_relay_log_info";
-
-  if (init_dynamic_string(&ds_result, "", 512, 512))
-    die("Out of memory");
-
-  run_query(query, &ds_result, TRUE);
-
-  if (ds_result.length)
-  {
-    row_count= atoi((char *)ds_result.str);
-    if (row_count)
-    {
-      fprintf(stderr, "Slave info repository compatibility check:"
-              " Found data in `mysql`.`slave_relay_log_info` table.\n");
-      fprintf(stderr, "Warning: Content of `mysql`.`slave_relay_log_info` "
-              "table will be ignored as MariaDB supports file based "
-              "repository.\n");
-      error= 1;
-    }
-  }
-  dynstr_free(&ds_result);
-  if (error)
-  {
-    fprintf(stderr,"Slave server may not possess the correct replication "
-            "metadata.\n");
-    fprintf(stderr, "Execution of CHANGE MASTER as per "
-            "`mysql`.`slave_master_info` and  `mysql`.`slave_relay_log_info` "
-            "table content is recommended.\n");
-  }
-  return 0;
-}
 
 /*
   Update all system tables in MySQL Server to current
@@ -1194,7 +1151,7 @@ int main(int argc, char **argv)
   load_defaults_or_exit("my", load_default_groups, &argc, &argv);
   defaults_argv= argv; /* Must be freed by 'free_defaults' */
 
-#if __WIN__
+#if defined(__WIN__)
   if (GetModuleFileName(NULL, self_name, FN_REFLEN) == 0)
 #endif
   {
@@ -1226,7 +1183,7 @@ int main(int argc, char **argv)
   cnf_file_path= strmov(defaults_file, "--defaults-file=");
   {
     int fd= create_temp_file(cnf_file_path, opt_tmpdir[0] ? opt_tmpdir : NULL,
-                             "mysql_upgrade-", O_CREAT | O_WRONLY, MYF(MY_FAE));
+                             "mysql_upgrade-", 0, MYF(MY_FAE));
     if (fd < 0)
       die(NULL);
     my_write(fd, USTRING_WITH_LEN( "[client]\n"), MYF(MY_FAE));
@@ -1247,9 +1204,9 @@ int main(int argc, char **argv)
     Read the mysql_upgrade_info file to check if mysql_upgrade
     already has been run for this installation of MySQL
   */
-  if (!opt_force && upgrade_already_done())
+  if (!opt_force && upgrade_already_done(0))
   {
-    printf("This installation of MySQL is already upgraded to %s, "
+    printf("This installation of MariaDB is already upgraded to %s, "
            "use --force if you still need to run mysql_upgrade\n",
            MYSQL_SERVER_VERSION);
     goto end;
@@ -1268,8 +1225,7 @@ int main(int argc, char **argv)
       run_mysqlcheck_views() ||
       run_sql_fix_privilege_tables() ||
       run_mysqlcheck_fixnames() ||
-      run_mysqlcheck_upgrade(FALSE) ||
-      check_slave_repositories())
+      run_mysqlcheck_upgrade(FALSE))
     die("Upgrade failed" );
 
   verbose("Phase %d/%d: Running 'FLUSH PRIVILEGES'", ++phase, phases_total);
