@@ -55,7 +55,6 @@ static int first_error = 0;
 static char *opt_skip_database;
 DYNAMIC_ARRAY tables4repair, tables4rebuild, alter_table_cmds;
 DYNAMIC_ARRAY views4repair;
-static char *shared_memory_base_name=0;
 static uint opt_protocol=0;
 
 enum operations { DO_CHECK=1, DO_REPAIR, DO_ANALYZE, DO_OPTIMIZE, DO_FIX_NAMES };
@@ -179,7 +178,7 @@ static struct my_option my_long_options[] =
    "built-in default (" STRINGIFY_ARG(MYSQL_PORT) ").",
    &opt_mysql_port, &opt_mysql_port, 0, GET_UINT, REQUIRED_ARG, 0, 0, 0, 0, 0,
    0},
-  {"protocol", OPT_MYSQL_PROTOCOL, "The protocol to use for connection (tcp, socket, pipe, memory).",
+  {"protocol", OPT_MYSQL_PROTOCOL, "The protocol to use for connection (tcp, socket, pipe).",
    0, 0, 0, GET_STR,  REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
   {"quick", 'q',
    "If you are using this option with CHECK TABLE, it prevents the check from scanning the rows to check for wrong links. This is the fastest check. If you are using this option with REPAIR TABLE, it will try to repair only the index tree. This is the fastest repair method for a table.",
@@ -188,11 +187,6 @@ static struct my_option my_long_options[] =
   {"repair", 'r',
    "Can fix almost anything except unique keys that aren't unique.",
    0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0},
-#ifdef HAVE_SMEM
-  {"shared-memory-base-name", OPT_SHARED_MEMORY_BASE_NAME,
-   "Base name of shared memory.", &shared_memory_base_name, &shared_memory_base_name,
-   0, GET_STR_ALLOC, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
-#endif
   {"silent", 's', "Print only error messages.", &opt_silent,
    &opt_silent, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
   {"skip_database", 0, "Don't process the database specified as argument", 
@@ -228,7 +222,8 @@ static struct my_option my_long_options[] =
 };
 
 static const char *load_default_groups[]=
-{ "mysqlcheck", "client", "client-server", "client-mariadb", 0 };
+{ "mysqlcheck", "mariadb-check", "client", "client-server", "client-mariadb",
+  0 };
 
 
 static void print_version(void);
@@ -289,13 +284,13 @@ static void usage(void)
 
 
 static my_bool
-get_one_option(int optid, const struct my_option *opt __attribute__((unused)),
-	       char *argument)
+get_one_option(const struct my_option *opt,
+	       char *argument, const char *filename __attribute__((unused)))
 {
   int orig_what_to_do= what_to_do;
   DBUG_ENTER("get_one_option");
 
-  switch(optid) {
+  switch(opt->id) {
   case 'a':
     what_to_do = DO_ANALYZE;
     break;
@@ -543,7 +538,7 @@ static int process_selected_tables(char *db, char **table_names, int tables)
 {
   int view;
   char *table;
-  uint table_len;
+  size_t table_len;
   DBUG_ENTER("process_selected_tables");
 
   if (use_db(db))
@@ -587,6 +582,7 @@ static int process_selected_tables(char *db, char **table_names, int tables)
     my_free(table_names_comma_sep);
   }
   else
+  {
     for (; tables > 0; tables--, table_names++)
     {
       table= *table_names;
@@ -596,6 +592,7 @@ static int process_selected_tables(char *db, char **table_names, int tables)
         continue;
       handle_request_for_tables(table, table_len, view == 1, opt_all_in_1);
     }
+  }
   DBUG_RETURN(0);
 } /* process_selected_tables */
 
@@ -669,7 +666,7 @@ static int process_all_tables_in_db(char *database)
     size_t tot_length = 0;
 
     char *views, *views_end;
-    uint tot_views_length = 0;
+    size_t tot_views_length = 0;
 
     while ((row = mysql_fetch_row(res)))
     {
@@ -915,16 +912,29 @@ static int handle_request_for_tables(char *tables, size_t length,
     }
     break;
   case DO_ANALYZE:
+    if (view)
+    {
+      printf("%-50s %s\n", tables, "Can't run anaylyze on a view");
+      DBUG_RETURN(1);
+    }
     DBUG_ASSERT(!view);
     op= (opt_write_binlog) ? "ANALYZE" : "ANALYZE NO_WRITE_TO_BINLOG";
     if (opt_persistent_all) end = strmov(end, " PERSISTENT FOR ALL");
     break;
   case DO_OPTIMIZE:
-    DBUG_ASSERT(!view);
+    if (view)
+    {
+      printf("%-50s %s\n", tables, "Can't run optimize on a view");
+      DBUG_RETURN(1);
+    }
     op= (opt_write_binlog) ? "OPTIMIZE" : "OPTIMIZE NO_WRITE_TO_BINLOG";
     break;
   case DO_FIX_NAMES:
-    DBUG_ASSERT(!view);
+    if (view)
+    {
+      printf("%-50s %s\n", tables, "Can't run fix names on a view");
+      DBUG_RETURN(1);
+    }
     DBUG_RETURN(fix_table_storage_name(tables));
   }
 
@@ -953,7 +963,7 @@ static int handle_request_for_tables(char *tables, size_t length,
   }
   if (verbose >= 3)
     puts(query);
-  if (mysql_real_query(sock, query, query_length))
+  if (mysql_real_query(sock, query, (ulong)query_length))
   {
     my_snprintf(message, sizeof(message), "when executing '%s%s... %s'",
                 op, tab_view, options);
@@ -965,7 +975,7 @@ static int handle_request_for_tables(char *tables, size_t length,
   if (opt_flush_tables)
   {
     query_length= sprintf(query, "FLUSH TABLES %s", table_name);
-    if (mysql_real_query(sock, query, query_length))
+    if (mysql_real_query(sock, query, (ulong)query_length))
     {
       DBerror(sock, query);
       my_free(query);
@@ -1100,8 +1110,6 @@ static int dbConnect(char *host, char *user, char *passwd)
 #endif
   if (opt_protocol)
     mysql_options(&mysql_connection,MYSQL_OPT_PROTOCOL,(char*)&opt_protocol);
-  if (shared_memory_base_name)
-    mysql_options(&mysql_connection,MYSQL_SHARED_MEMORY_BASE_NAME,shared_memory_base_name);
 
   if (opt_plugin_dir && *opt_plugin_dir)
     mysql_options(&mysql_connection, MYSQL_PLUGIN_DIR, opt_plugin_dir);
@@ -1241,8 +1249,7 @@ int main(int argc, char **argv)
     delete_dynamic(&alter_table_cmds);
   }
  end1:
-  my_free(opt_password);
-  my_free(shared_memory_base_name);
+  my_free(opt_password);;
   mysql_library_end();
   free_defaults(defaults_argv);
   my_end(my_end_arg);
