@@ -67,9 +67,9 @@ then
   sed 's/ --with systemd//' -i debian/rules
   sed '/systemd/d' -i debian/rules
   sed '/\.service/d' -i debian/rules
-  sed '/galera_new_cluster/d' -i debian/mariadb-server-10.4.install
-  sed '/galera_recovery/d' -i debian/mariadb-server-10.4.install
-  sed '/mariadb-service-convert/d' -i debian/mariadb-server-10.4.install
+  sed '/galera_new_cluster/d' -i debian/mariadb-server-10.5.install
+  sed '/galera_recovery/d' -i debian/mariadb-server-10.5.install
+  sed '/mariadb-service-convert/d' -i debian/mariadb-server-10.5.install
 fi
 
 # If libzstd-dev is not available (before Debian Stretch and Ubuntu Xenial)
@@ -77,6 +77,7 @@ fi
 if ! apt-cache madison libzstd-dev | grep 'libzstd-dev' >/dev/null 2>&1
 then
   sed '/libzstd-dev/d' -i debian/control
+  sed '/libzstd1/d' -i debian/control
 fi
 
 # The binaries should be fully hardened by default. However TokuDB compilation seems to fail on
@@ -105,6 +106,8 @@ then
   sed '/Package: mariadb-plugin-cassandra/,/^$/d' -i debian/control
 fi
 
+sed -i -e "/Package: mariadb-plugin-tokudb/,/^$/d" debian/control
+
 # Mroonga, TokuDB never built on Travis CI anyway, see build flags above
 if [[ $TRAVIS ]]
 then
@@ -112,7 +115,7 @@ then
   sed -i -e "/Package: mariadb-plugin-mroonga/,/^$/d" debian/control
   sed -i -e "/Package: mariadb-plugin-spider/,/^$/d" debian/control
   sed -i -e "/Package: mariadb-plugin-oqgraph/,/^$/d" debian/control
-  sed -i -e "/usr\/lib\/mysql\/plugin\/ha_sphinx.so/d" debian/mariadb-server-10.4.install
+  sed -i -e "/usr\/lib\/mysql\/plugin\/ha_sphinx.so/d" debian/mariadb-server-10.5.install
   sed -i -e "/Package: libmariadbd-dev/,/^$/d" debian/control
 fi
 
