@@ -151,7 +151,7 @@ static char *remove_end_comment(char *ptr);
     --defaults_group_suffix
 */
 
-static int my_search_option_files(const char *conf_file, int *argc, char ***argv,
+static int my_search_option_files(const char *conf_file,
                                   struct handle_option_ctx *ctx,
                                   const char **default_directories)
 {
@@ -322,17 +322,13 @@ int get_defaults_options(char **argv)
 
   if (my_defaults_extra_file && my_defaults_extra_file != extra_file_buffer)
   {
-    int error= my_realpath(extra_file_buffer, my_defaults_extra_file, MYF(0));
-    if (error)
-      return error;
+    my_realpath(extra_file_buffer, my_defaults_extra_file, MYF(0));
     my_defaults_extra_file= extra_file_buffer;
   }
 
   if (my_defaults_file && my_defaults_file != file_buffer)
   {
-    int error= my_realpath(file_buffer, my_defaults_file, MYF(0));
-    if (error)
-      return error;
+    my_realpath(file_buffer, my_defaults_file, MYF(0));
     my_defaults_file= file_buffer;
   }
 
@@ -444,8 +440,7 @@ int my_load_defaults(const char *conf_file, const char **groups, int *argc,
     ctx.args= &args;
     ctx.group= &group;
 
-    if ((error= my_search_option_files(conf_file, argc - args_used,
-                                       argv + args_used, &ctx, dirs)))
+    if ((error= my_search_option_files(conf_file, &ctx, dirs)))
     {
       delete_dynamic(&args);
       free_root(&alloc,MYF(0));
