@@ -1,4 +1,4 @@
-/* Copyright (C) 2010, 2017, MariaDB Corporation Ab
+/* Copyright (C) 2010, 2020, MariaDB Corporation.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
   Engine defined options of tables/fields/keys in CREATE/ALTER TABLE.
 */
 
+#include "mariadb.h"
 #include "create_options.h"
 #include <my_getopt.h>
 #include "set_var.h"
@@ -43,9 +44,8 @@ void engine_option_value::link(engine_option_value **start,
   /* check duplicates to avoid writing them to frm*/
   for(opt= *start;
       opt && ((opt->parsed && !opt->value.str) ||
-              my_strnncoll(system_charset_info,
-                           (uchar *)name.str, name.length,
-                           (uchar*)opt->name.str, opt->name.length));
+              system_charset_info->strnncoll(name.str, name.length,
+                                             opt->name.str, opt->name.length));
       opt= opt->next) /* no-op */;
   if (opt)
   {
@@ -119,7 +119,7 @@ static bool report_unknown_option(THD *thd, engine_option_value *val,
 #define value_ptr(STRUCT,OPT)    ((char*)(STRUCT) + (OPT)->offset)
 
 static bool set_one_value(ha_create_table_option *opt,
-                          THD *thd, const LEX_STRING *value, void *base,
+                          THD *thd, const LEX_CSTRING *value, void *base,
                           bool suppress_warning,
                           MEM_ROOT *root)
 {
@@ -131,7 +131,8 @@ static bool set_one_value(ha_create_table_option *opt,
   switch (opt->type)
   {
   case HA_OPTION_TYPE_SYSVAR:
-    DBUG_ASSERT(0); // HA_OPTION_TYPE_SYSVAR's are replaced in resolve_sysvars()
+    // HA_OPTION_TYPE_SYSVAR's are replaced in resolve_sysvars()
+    break; // to DBUG_ASSERT(0)
   case HA_OPTION_TYPE_ULL:
     {
       ulonglong *val= (ulonglong*)value_ptr(base, opt);
@@ -185,9 +186,8 @@ static bool set_one_value(ha_create_table_option *opt,
         for (end=start;
              *end && *end != ',';
              end++) /* no-op */;
-        if (!my_strnncoll(system_charset_info,
-                          (uchar*)start, end-start,
-                          (uchar*)value->str, value->length))
+        if (!system_charset_info->strnncoll(start, end-start,
+                                            value->str, value->length))
         {
           *val= num;
           DBUG_RETURN(0);
@@ -209,29 +209,17 @@ static bool set_one_value(ha_create_table_option *opt,
       if (!value->str)
         DBUG_RETURN(0);
 
-      if (!my_strnncoll(system_charset_info,
-                        (const uchar*)"NO", 2,
-                        (uchar *)value->str, value->length) ||
-          !my_strnncoll(system_charset_info,
-                        (const uchar*)"OFF", 3,
-                        (uchar *)value->str, value->length) ||
-          !my_strnncoll(system_charset_info,
-                        (const uchar*)"0", 1,
-                        (uchar *)value->str, value->length))
+      if (!system_charset_info->strnncoll("NO", 2, value->str, value->length) ||
+          !system_charset_info->strnncoll("OFF", 3, value->str, value->length) ||
+          !system_charset_info->strnncoll("0", 1, value->str, value->length))
       {
         *val= FALSE;
         DBUG_RETURN(FALSE);
       }
 
-      if (!my_strnncoll(system_charset_info,
-                        (const uchar*)"YES", 3,
-                        (uchar *)value->str, value->length) ||
-          !my_strnncoll(system_charset_info,
-                        (const uchar*)"ON", 2,
-                        (uchar *)value->str, value->length) ||
-          !my_strnncoll(system_charset_info,
-                        (const uchar*)"1", 1,
-                        (uchar *)value->str, value->length))
+      if (!system_charset_info->strnncoll("YES", 3, value->str, value->length) ||
+          !system_charset_info->strnncoll("ON", 2, value->str, value->length) ||
+          !system_charset_info->strnncoll("1", 1, value->str, value->length))
       {
         *val= TRUE;
         DBUG_RETURN(FALSE);
@@ -293,9 +281,8 @@ bool parse_option_list(THD* thd, handlerton *hton, void *option_struct_arg,
     for (val= *option_list; val; val= val->next)
     {
       last= val;
-      if (my_strnncoll(system_charset_info,
-                       (uchar*)opt->name, opt->name_length,
-                       (uchar*)val->name.str, val->name.length))
+      if (system_charset_info->strnncoll(opt->name, opt->name_length,
+                                         val->name.str, val->name.length))
         continue;
 
       /* skip duplicates (see engine_option_value constructor above) */
@@ -311,7 +298,7 @@ bool parse_option_list(THD* thd, handlerton *hton, void *option_struct_arg,
     }
     if (!seen || (opt->var && !last->value.str))
     {
-      LEX_STRING default_val= null_lex_str;
+      LEX_CSTRING default_val= null_clex_str;
 
       /*
         Okay, here's the logic for sysvar options:
@@ -320,8 +307,8 @@ bool parse_option_list(THD* thd, handlerton *hton, void *option_struct_arg,
            *current* value of the underlying sysvar.
         2. But only if the underlying sysvar value is different from the
            sysvar's default.
-        3. If it's ALTER TABLE and the sysvar option was not explicitly
-           mentioned - do nothing, do not add it to the list.
+        3. If it's ALTER TABLE or CREATE_SEQUENCE and the sysvar option was
+           not explicitly mentioned - do nothing, do not add it to the list.
         4. But if it was ALTER TABLE with sysvar option = DEFAULT, we
            add it to the list (under the same condition #2).
         5. If we're here parsing the option list from the .frm file
@@ -329,7 +316,6 @@ bool parse_option_list(THD* thd, handlerton *hton, void *option_struct_arg,
            do not add it to the list (makes no sense anyway) and
            use the *default* value of the underlying sysvar. Because
            sysvar value can change, but it should not affect existing tables.
-
         This is how it's implemented: the current sysvar value is added
         to the list if suppress_warning is FALSE (meaning a table is created,
         that is CREATE TABLE or ALTER TABLE) and it's actually a CREATE TABLE
@@ -349,9 +335,9 @@ bool parse_option_list(THD* thd, handlerton *hton, void *option_struct_arg,
         {
           char buf[256];
           String sbuf(buf, sizeof(buf), system_charset_info), *str;
-          if ((str= sysvar->val_str(&sbuf, thd, OPT_SESSION, &null_lex_str)))
+          if ((str= sysvar->val_str(&sbuf, thd, OPT_SESSION, &null_clex_str)))
           {
-            LEX_STRING name= { const_cast<char*>(opt->name), opt->name_length };
+            LEX_CSTRING name= { opt->name, opt->name_length };
             default_val.str= strmake_root(root, str->ptr(), str->length());
             default_val.length= str->length();
             val= new (root) engine_option_value(name, default_val,
@@ -545,7 +531,7 @@ uint engine_option_value::frm_length()
 
     if value.str is NULL, this option is not written to frm (=DEFAULT)
   */
-  return value.str ? 1 + name.length + 2 + value.length : 0;
+  return value.str ? (uint)(1 + name.length + 2 + value.length) : 0;
 }
 
 
@@ -690,7 +676,7 @@ uchar *engine_option_value::frm_read(const uchar *buff, const uchar *buff_end,
                                      engine_option_value **start,
                                      engine_option_value **end, MEM_ROOT *root)
 {
-  LEX_STRING name, value;
+  LEX_CSTRING name, value;
   uint len;
 #define need_buff(N)  if (buff + (N) >= buff_end) return NULL
 
@@ -730,7 +716,7 @@ uchar *engine_option_value::frm_read(const uchar *buff, const uchar *buff_end,
   @retval FALSE OK
 */
 
-bool engine_table_options_frm_read(const uchar *buff, uint length,
+bool engine_table_options_frm_read(const uchar *buff, size_t length,
                                    TABLE_SHARE *share)
 {
   const uchar *buff_end= buff + length;
@@ -808,9 +794,8 @@ bool is_engine_option_known(engine_option_value *opt,
 
   for (; rules->name; rules++)
   {
-      if (!my_strnncoll(system_charset_info,
-                        (uchar*)rules->name, rules->name_length,
-                        (uchar*)opt->name.str, opt->name.length))
+      if (!system_charset_info->strnncoll(rules->name, rules->name_length,
+                                          opt->name.str, opt->name.length))
         return true;
   }
   return false;
