@@ -425,7 +425,7 @@ rpl_slave_state::truncate_state_table(THD *thd)
   if (!(err= open_and_lock_tables(thd, &tlist, FALSE, 0)))
   {
     tdc_remove_table(thd, TDC_RT_REMOVE_UNUSED, "mysql",
-                     rpl_gtid_slave_state_table_name.str, false);
+                     rpl_gtid_slave_state_table_name.str);
     err= tlist.table->file->ha_truncate();
 
     if (err)
