@@ -1,5 +1,5 @@
 /* Copyright (c) 2006, 2017, Oracle and/or its affiliates.
-   Copyright (c) 2010, 2020, MariaDB Corporation.
+   Copyright (c) 2010, 2017, MariaDB Corporation
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -62,7 +62,6 @@ Relay_log_info::Relay_log_info(bool is_slave_recovery)
    slave_running(MYSQL_SLAVE_NOT_RUN), until_condition(UNTIL_NONE),
    until_log_pos(0), retried_trans(0), executed_entries(0),
    sql_delay(0), sql_delay_end(0),
-   until_relay_log_names_defer(false),
    m_flags(0)
 {
   DBUG_ENTER("Relay_log_info::Relay_log_info");
@@ -238,7 +237,7 @@ a file name for --relay-log-index option", opt_relaylog_index_name);
     */
     mysql_mutex_lock(log_lock);
     if (relay_log.open_index_file(buf_relaylog_index_name, ln, TRUE) ||
-        relay_log.open(ln, LOG_BIN, 0, 0, SEQ_READ_APPEND,
+        relay_log.open(ln, 0, 0, SEQ_READ_APPEND,
                        (ulong)max_relay_log_size, 1, TRUE))
     {
       mysql_mutex_unlock(log_lock);
@@ -504,8 +503,6 @@ void Relay_log_info::clear_until_condition()
   until_condition= Relay_log_info::UNTIL_NONE;
   until_log_name[0]= 0;
   until_log_pos= 0;
-  until_relay_log_names_defer= false;
-
   DBUG_VOID_RETURN;
 }
 
@@ -996,6 +993,7 @@ void Relay_log_info::inc_group_relay_log_pos(ulonglong log_pos,
     {
       group_relay_log_pos= rgi->future_event_relay_log_pos;
       strmake_buf(group_relay_log_name, rgi->event_relay_log_name);
+      notify_group_relay_log_name_update();
     } else if (cmp == 0 && group_relay_log_pos < rgi->future_event_relay_log_pos)
       group_relay_log_pos= rgi->future_event_relay_log_pos;
 
@@ -1180,7 +1178,7 @@ int purge_relay_logs(Relay_log_info* rli, THD *thd, bool just_reset,
         DBUG_RETURN(1);
       }
       mysql_mutex_lock(rli->relay_log.get_log_lock());
-      if (rli->relay_log.open(ln, LOG_BIN, 0, 0, SEQ_READ_APPEND,
+      if (rli->relay_log.open(ln, 0, 0, SEQ_READ_APPEND,
                              (ulong)(rli->max_relay_log_size ? rli->max_relay_log_size :
                               max_binlog_size), 1, TRUE))
       {
@@ -1285,78 +1283,29 @@ err:
      autoincrement or if we have transactions).
 
      Should be called ONLY if until_condition != UNTIL_NONE !
-
-     In the parallel execution mode and UNTIL_MASTER_POS the file name is
-     presented by future_event_master_log_name which may be ahead of
-     group_master_log_name. Log_event::log_pos does relate to it nevertheless
-     so the pair comprises a correct binlog coordinate.
-     Internal group events and events that have zero log_pos also
-     produce the zero for the local log_pos which may not lead to the
-     function falsely return true.
-     In UNTIL_RELAY_POS the original caching and notification are simplified
-     to straightforward files comparison when the current event can't be
-     a part of an event group.
-
    RETURN VALUE
      true - condition met or error happened (condition seems to have
             bad log file name)
      false - condition not met
 */
 
-bool Relay_log_info::is_until_satisfied(Log_event *ev)
+bool Relay_log_info::is_until_satisfied(my_off_t master_beg_pos)
 {
   const char *log_name;
   ulonglong log_pos;
-  /* Prevents stopping within transaction; needed solely for Relay UNTIL. */
-  bool in_trans= false;
-
   DBUG_ENTER("Relay_log_info::is_until_satisfied");
 
   if (until_condition == UNTIL_MASTER_POS)
   {
     log_name= (mi->using_parallel() ? future_event_master_log_name
                                     : group_master_log_name);
-    log_pos= (get_flag(Relay_log_info::IN_TRANSACTION) || !ev || !ev->log_pos) ?
-      (mi->using_parallel() ? 0 : group_master_log_pos) :
-      ev->log_pos - ev->data_written;
+    log_pos= master_beg_pos;
   }
   else
   {
     DBUG_ASSERT(until_condition == UNTIL_RELAY_POS);
-    if (!mi->using_parallel())
-    {
-      log_name= group_relay_log_name;
-      log_pos= group_relay_log_pos;
-    }
-    else
-    {
-      log_name= event_relay_log_name;
-      log_pos=  event_relay_log_pos;
-      in_trans= get_flag(Relay_log_info::IN_TRANSACTION);
-      /*
-        until_log_names_cmp_result is set to UNKNOWN either
-        -  by a non-group event *and* only when it is in the middle of a group
-        -  or by a group event when the preceding group made the above
-           non-group event to defer the resetting.
-      */
-      if ((ev && !Log_event::is_group_event(ev->get_type_code())))
-      {
-        if (in_trans)
-        {
-          until_relay_log_names_defer= true;
-        }
-        else
-        {
-          until_log_names_cmp_result= UNTIL_LOG_NAMES_CMP_UNKNOWN;
-          until_relay_log_names_defer= false;
-        }
-      }
-      else if (!in_trans && until_relay_log_names_defer)
-      {
-        until_log_names_cmp_result= UNTIL_LOG_NAMES_CMP_UNKNOWN;
-        until_relay_log_names_defer= false;
-      }
-    }
+    log_name= group_relay_log_name;
+    log_pos= group_relay_log_pos;
   }
 
   DBUG_PRINT("info", ("group_master_log_name='%s', group_master_log_pos=%llu",
@@ -1410,8 +1359,8 @@ bool Relay_log_info::is_until_satisfied(Log_event *ev)
   }
 
   DBUG_RETURN(((until_log_names_cmp_result == UNTIL_LOG_NAMES_CMP_EQUAL &&
-                (log_pos >= until_log_pos && !in_trans)) ||
-               until_log_names_cmp_result == UNTIL_LOG_NAMES_CMP_GREATER));
+           log_pos >= until_log_pos) ||
+          until_log_names_cmp_result == UNTIL_LOG_NAMES_CMP_GREATER));
 }
 
 
@@ -1471,14 +1420,8 @@ bool Relay_log_info::stmt_done(my_off_t event_master_log_pos, THD *thd,
     }
     DBUG_EXECUTE_IF("inject_crash_before_flush_rli", DBUG_SUICIDE(););
     if (mi->using_gtid == Master_info::USE_GTID_NO)
-    {
-      if (rgi->is_parallel_exec)
-        mysql_mutex_lock(&data_lock);
       if (flush())
         error= 1;
-      if (rgi->is_parallel_exec)
-        mysql_mutex_unlock(&data_lock);
-    }
     DBUG_EXECUTE_IF("inject_crash_after_flush_rli", DBUG_SUICIDE(););
   }
   DBUG_RETURN(error);
@@ -1492,27 +1435,31 @@ Relay_log_info::alloc_inuse_relaylog(const char *name)
   uint32 gtid_count;
   rpl_gtid *gtid_list;
 
+  if (!(ir= (inuse_relaylog *)my_malloc(sizeof(*ir), MYF(MY_WME|MY_ZEROFILL))))
+  {
+    my_error(ER_OUTOFMEMORY, MYF(0), (int)sizeof(*ir));
+    return 1;
+  }
   gtid_count= relay_log_state.count();
   if (!(gtid_list= (rpl_gtid *)my_malloc(sizeof(*gtid_list)*gtid_count,
                                          MYF(MY_WME))))
   {
+    my_free(ir);
     my_error(ER_OUTOFMEMORY, MYF(0), (int)sizeof(*gtid_list)*gtid_count);
     return 1;
   }
-  if (!(ir= new inuse_relaylog(this, gtid_list, gtid_count, name)))
-  {
-    my_free(gtid_list);
-    my_error(ER_OUTOFMEMORY, MYF(0), (int) sizeof(*ir));
-    return 1;
-  }
   if (relay_log_state.get_gtid_list(gtid_list, gtid_count))
   {
     my_free(gtid_list);
-    delete ir;
+    my_free(ir);
     DBUG_ASSERT(0 /* Should not be possible as we allocated correct length */);
     my_error(ER_OUT_OF_RESOURCES, MYF(0));
     return 1;
   }
+  ir->rli= this;
+  strmake_buf(ir->name, name);
+  ir->relay_log_state= gtid_list;
+  ir->relay_log_state_count= gtid_count;
 
   if (!inuse_relaylog_list)
     inuse_relaylog_list= ir;
@@ -1531,7 +1478,7 @@ void
 Relay_log_info::free_inuse_relaylog(inuse_relaylog *ir)
 {
   my_free(ir->relay_log_state);
-  delete ir;
+  my_free(ir);
 }
 
 
@@ -2072,9 +2019,10 @@ find_gtid_slave_pos_tables(THD *thd)
       However we can add new entries, and warn about any tables that
       disappeared, but may still be visible to running SQL threads.
     */
-    rpl_slave_state::gtid_pos_table *new_entry, **next_ptr_ptr;
-    auto old_entry= rpl_global_gtid_slave_state->
-                    gtid_pos_tables.load(std::memory_order_relaxed);
+    rpl_slave_state::gtid_pos_table *old_entry, *new_entry, **next_ptr_ptr;
+
+    old_entry= (rpl_slave_state::gtid_pos_table *)
+      rpl_global_gtid_slave_state->gtid_pos_tables;
     while (old_entry)
     {
       new_entry= cb_data.table_list;
@@ -2096,8 +2044,8 @@ find_gtid_slave_pos_tables(THD *thd)
     while (new_entry)
     {
       /* Check if we already have a table with this storage engine. */
-      old_entry= rpl_global_gtid_slave_state->
-                 gtid_pos_tables.load(std::memory_order_relaxed);
+      old_entry= (rpl_slave_state::gtid_pos_table *)
+        rpl_global_gtid_slave_state->gtid_pos_tables;
       while (old_entry)
       {
         if (new_entry->table_hton == old_entry->table_hton)
