@@ -35,7 +35,7 @@
 #include "tztime.h"     // my_tz_find, my_tz_SYSTEM, struct Time_zone
 #include "sql_acl.h"    // SUPER_ACL
 #include "sql_select.h" // free_underlaid_joins
-#include "sql_show.h"
+#include "sql_i_s.h"
 #include "sql_view.h"   // updatable_views_with_limit_typelib
 #include "lock.h"                               // lock_global_read_lock,
                                                 // make_global_read_lock_block_commit,
@@ -220,13 +220,12 @@ bool sys_var::update(THD *thd, set_var *var)
     */
     if ((var->type == OPT_SESSION) && (!ret))
     {
-      SESSION_TRACKER_CHANGED(thd, SESSION_SYSVARS_TRACKER,
-                              (LEX_CSTRING*)var->var);
+      thd->session_tracker.sysvars.mark_as_changed(thd, var->var);
       /*
         Here MySQL sends variable name to avoid reporting change of
         the tracker itself, but we decided that it is not needed
       */
-      SESSION_TRACKER_CHANGED(thd, SESSION_STATE_CHANGE_TRACKER, NULL);
+      thd->session_tracker.state_change.mark_as_changed(thd);
     }
 
     return ret;
@@ -742,7 +741,7 @@ int sql_set_variables(THD *thd, List<set_var_base> *var_list, bool free)
 
 err:
   if (free)
-    free_underlaid_joins(thd, &thd->lex->select_lex);
+    free_underlaid_joins(thd, thd->lex->first_select_lex());
   DBUG_RETURN(error);
 }
 
@@ -907,7 +906,7 @@ int set_var_user::update(THD *thd)
     return -1;
   }
 
-  SESSION_TRACKER_CHANGED(thd, SESSION_STATE_CHANGE_TRACKER, NULL);
+  thd->session_tracker.state_change.mark_as_changed(thd);
   return 0;
 }
 
@@ -957,8 +956,7 @@ int set_var_role::update(THD *thd)
 #ifndef NO_EMBEDDED_ACCESS_CHECKS
   int res= acl_setrole(thd, role.str, access);
   if (!res)
-    thd->session_tracker.mark_as_changed(thd, SESSION_STATE_CHANGE_TRACKER,
-                                         NULL);
+    thd->session_tracker.state_change.mark_as_changed(thd);
   return res;
 #else
   return 0;
@@ -973,17 +971,8 @@ int set_var_default_role::check(THD *thd)
 {
 #ifndef NO_EMBEDDED_ACCESS_CHECKS
   real_user= get_current_user(thd, user);
-  real_role= role.str;
-  if (role.str == current_role.str)
-  {
-    if (!thd->security_ctx->priv_role[0])
-      real_role= "NONE";
-    else
-      real_role= thd->security_ctx->priv_role;
-  }
-
-  return acl_check_set_default_role(thd, real_user->host.str,
-                                    real_user->user.str, real_role);
+  int status= acl_check_set_default_role(thd, real_user->host.str, real_user->user.str);
+  return status;
 #else
   return 0;
 #endif
@@ -994,8 +983,7 @@ int set_var_default_role::update(THD *thd)
 #ifndef NO_EMBEDDED_ACCESS_CHECKS
   Reprepare_observer *save_reprepare_observer= thd->m_reprepare_observer;
   thd->m_reprepare_observer= 0;
-  int res= acl_set_default_role(thd, real_user->host.str, real_user->user.str,
-                                real_role);
+  int res= acl_set_default_role(thd, real_user->host.str, real_user->user.str, role.str);
   thd->m_reprepare_observer= save_reprepare_observer;
   return res;
 #else
@@ -1025,18 +1013,13 @@ int set_var_collation_client::update(THD *thd)
                       character_set_results);
 
   /* Mark client collation variables as changed */
-#ifndef EMBEDDED_LIBRARY
-  if (thd->session_tracker.sysvars.is_enabled())
-  {
-    thd->session_tracker.sysvars.
-      mark_as_changed(thd, (LEX_CSTRING*)Sys_character_set_client_ptr);
-    thd->session_tracker.sysvars.
-      mark_as_changed(thd, (LEX_CSTRING*)Sys_character_set_results_ptr);
-    thd->session_tracker.sysvars.
-      mark_as_changed(thd, (LEX_CSTRING*)Sys_character_set_connection_ptr);
-  }
-  thd->session_tracker.mark_as_changed(thd, SESSION_STATE_CHANGE_TRACKER, NULL);
-#endif //EMBEDDED_LIBRARY
+  thd->session_tracker.sysvars.mark_as_changed(thd,
+                                               Sys_character_set_client_ptr);
+  thd->session_tracker.sysvars.mark_as_changed(thd,
+                                               Sys_character_set_results_ptr);
+  thd->session_tracker.sysvars.mark_as_changed(thd,
+                                               Sys_character_set_connection_ptr);
+  thd->session_tracker.state_change.mark_as_changed(thd);
 
   thd->protocol_text.init(thd);
   thd->protocol_binary.init(thd);
@@ -1068,15 +1051,18 @@ static void store_var(Field *field, sys_var *var, enum_var_type scope,
 int fill_sysvars(THD *thd, TABLE_LIST *tables, COND *cond)
 {
   char name_buffer[NAME_CHAR_LEN];
+  enum_check_fields save_count_cuted_fields= thd->count_cuted_fields;
   bool res= 1;
   CHARSET_INFO *scs= system_charset_info;
   StringBuffer<STRING_BUFFER_USUAL_SIZE> strbuf(scs);
   const char *wild= thd->lex->wild ? thd->lex->wild->ptr() : 0;
   Field **fields=tables->table->field;
+  bool has_file_acl= !check_access(thd, FILE_ACL, any_db, NULL, NULL, 0, 1);
 
   DBUG_ASSERT(tables->table->in_use == thd);
 
   cond= make_cond_for_info_schema(thd, cond, tables);
+  thd->count_cuted_fields= CHECK_FIELD_WARN;
   mysql_prlock_rdlock(&LOCK_system_variables_hash);
 
   for (uint i= 0; i < system_variable_hash.records; i++)
@@ -1106,6 +1092,7 @@ int fill_sysvars(THD *thd, TABLE_LIST *tables, COND *cond)
     static const LEX_CSTRING origins[]=
     {
       { STRING_WITH_LEN("CONFIG") },
+      { STRING_WITH_LEN("COMMAND-LINE") },
       { STRING_WITH_LEN("AUTO") },
       { STRING_WITH_LEN("SQL") },
       { STRING_WITH_LEN("COMPILE-TIME") },
@@ -1234,6 +1221,14 @@ int fill_sysvars(THD *thd, TABLE_LIST *tables, COND *cond)
       fields[13]->store(arg->str, arg->length, scs);
     }
 
+    // GLOBAL_VALUE_PATH
+    if (var->value_origin == sys_var::CONFIG && has_file_acl)
+    {
+      fields[14]->set_notnull();
+      fields[14]->store(var->origin_filename, strlen(var->origin_filename),
+                        files_charset_info);
+    }
+
     if (schema_table_store_record(thd, tables->table))
       goto end;
     thd->get_stmt_da()->inc_current_row_for_warning();
@@ -1241,6 +1236,7 @@ int fill_sysvars(THD *thd, TABLE_LIST *tables, COND *cond)
   res= 0;
 end:
   mysql_prlock_unlock(&LOCK_system_variables_hash);
+  thd->count_cuted_fields= save_count_cuted_fields;
   return res;
 }
 
