@@ -1,5 +1,5 @@
 /* Copyright (c) 2000, 2013, Oracle and/or its affiliates.
-   Copyright (c) 2016, MariaDB
+   Copyright (c) 2016, 2020, MariaDB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -125,7 +125,7 @@ bool String::set_int(longlong num, bool unsigned_flag, CHARSET_INFO *cs)
 
   if (alloc(l))
     return TRUE;
-  str_length=(uint32) (cs->cset->longlong10_to_str)(cs,Ptr,l,base,num);
+  str_length=(uint32) (cs->longlong10_to_str)(Ptr,l,base,num);
   set_charset(cs);
   return FALSE;
 }
@@ -183,17 +183,6 @@ bool Binary_string::set_hex(const char *str, uint32 len)
 }
 
 
-bool Binary_string::set_fcvt(double num, uint decimals)
-{
-  // Assert that `decimals` is small enough to fit into FLOATING_POINT_BUFFER
-  DBUG_ASSERT(decimals < DECIMAL_NOT_SPECIFIED);
-  if (alloc(FLOATING_POINT_BUFFER))
-    return true;
-  length(my_fcvt(num, decimals, Ptr, NULL));
-  return false;
-}
-
-
 bool String::set_real(double num,uint decimals, CHARSET_INFO *cs)
 {
   char buff[FLOATING_POINT_BUFFER];
@@ -788,6 +777,29 @@ void Static_binary_string::qs_append(ulonglong i)
   str_length+= (int) (end-buff);
 }
 
+
+bool Binary_string::copy_printable_hhhh(CHARSET_INFO *to_cs,
+                                        CHARSET_INFO *from_cs,
+                                        const char *from,
+                                        size_t from_length)
+{
+  DBUG_ASSERT(from_length < UINT_MAX32);
+  uint errors;
+  uint one_escaped_char_length= MY_CS_PRINTABLE_CHAR_LENGTH * to_cs->mbminlen;
+  uint one_char_length= MY_MAX(one_escaped_char_length, to_cs->mbmaxlen);
+  ulonglong bytes_needed= from_length * one_char_length;
+  if (bytes_needed >= UINT_MAX32 || alloc((size_t) bytes_needed))
+    return true;
+  str_length= my_convert_using_func(Ptr, Alloced_length, to_cs,
+                                    my_wc_to_printable_generic,
+                                    from, from_length,
+                                    from_cs,
+                                    from_cs->cset->mb_wc,
+                                    &errors);
+  return false;
+}
+
+
 /*
   Compare strings according to collation, without end space.
 
@@ -809,9 +821,8 @@ void Static_binary_string::qs_append(ulonglong i)
 
 int sortcmp(const String *s,const String *t, CHARSET_INFO *cs)
 {
- return cs->coll->strnncollsp(cs,
-                              (uchar *) s->ptr(),s->length(),
-                              (uchar *) t->ptr(),t->length());
+ return cs->strnncollsp(s->ptr(), s->length(),
+                        t->ptr(), t->length());
 }
 
 
@@ -836,7 +847,7 @@ int sortcmp(const String *s,const String *t, CHARSET_INFO *cs)
 int stringcmp(const String *s,const String *t)
 {
   uint32 s_len=s->length(),t_len=t->length(),len=MY_MIN(s_len,t_len);
-  int cmp= len ? memcmp(s->ptr(), t->ptr(), len) : 0;
+  int cmp= memcmp(s->ptr(), t->ptr(), len);
   return (cmp) ? cmp : (int) (s_len - t_len);
 }
 
@@ -1087,8 +1098,8 @@ String_copier::well_formed_copy(CHARSET_INFO *to_cs,
       my_charset_same(from_cs, to_cs))
   {
     m_cannot_convert_error_pos= NULL;
-    return (uint) to_cs->cset->copy_fix(to_cs, to, to_length, from, from_length,
-                                 nchars, this);
+    return (uint) to_cs->copy_fix(to, to_length, from, from_length,
+                                  nchars, this);
   }
   return (uint) my_convert_fix(to_cs, to, to_length, from_cs, from, from_length,
                         nchars, this, this);
