@@ -16,7 +16,7 @@
 
 /* Write some debug info */
 
-#include <my_global.h>
+#include "mariadb.h"
 #include "sql_priv.h"
 #include "unireg.h"
 #include "sql_test.h"
@@ -24,6 +24,7 @@
 #include "sql_show.h" // calc_sum_of_all_status
 #include "sql_select.h"
 #include "keycaches.h"
+#include "my_json_writer.h"
 #include <hash.h>
 #include <thr_alarm.h>
 #if defined(HAVE_MALLINFO) && defined(HAVE_MALLOC_H)
@@ -36,6 +37,8 @@
 #include "events.h"
 #endif
 
+#define FT_KEYPART   (MAX_FIELDS+10)
+
 static const char *lock_descriptions[] =
 {
   /* TL_UNLOCK                  */  "No lock",
@@ -85,9 +88,8 @@ static my_bool print_cached_tables_callback(TDC_element *element,
   while ((entry= it++))
   {
     THD *in_use= entry->in_use;
-    printf("%-14.14s %-32s%6lu%8ld%6d  %s\n",
+    printf("%-14.14s %-32s%8ld%6d  %s\n",
            entry->s->db.str, entry->s->table_name.str,
-           (ulong) element->version,
            in_use ? (long) in_use->thread_id : (long) 0,
            entry->db_stat ? 1 : 0,
            in_use ? lock_descriptions[(int)entry->reginfo.lock_type] :
@@ -107,8 +109,6 @@ static void print_cached_tables(void)
 
   tdc_iterate(0, (my_hash_walk_action) print_cached_tables_callback, NULL, true);
 
-  printf("\nCurrent refresh version: %ld\n",
-         (long) tdc_refresh_version());
   fflush(stdout);
   /* purecov: end */
   return;
@@ -137,7 +137,8 @@ void TEST_filesort(SORT_FIELD *sortorder,uint s_length)
 	out.append(*sortorder->field->table_name);
 	out.append('.');
       }
-      out.append(sortorder->field->field_name ? sortorder->field->field_name:
+      out.append(sortorder->field->field_name.str ?
+                 sortorder->field->field_name.str :
 		 "tmp_table_column");
     }
     else
@@ -224,8 +225,6 @@ TEST_join(JOIN *join)
 }
 
 
-#define FT_KEYPART   (MAX_FIELDS+10)
-
 static void print_keyuse(KEYUSE *keyuse)
 {
   char buff[256];
@@ -238,11 +237,11 @@ static void print_keyuse(KEYUSE *keyuse)
   keyuse->val->print(&str, QT_ORDINARY);
   str.append('\0');
   if (keyuse->is_for_hash_join())
-    fieldname= keyuse->table->field[keyuse->keypart]->field_name;
+    fieldname= keyuse->table->field[keyuse->keypart]->field_name.str;
   else if (keyuse->keypart == FT_KEYPART)
     fieldname= "FT_KEYPART";
   else
-    fieldname= key_info->key_part[keyuse->keypart].field->field_name;
+    fieldname= key_info->key_part[keyuse->keypart].field->field_name.str;
   ll2str(keyuse->used_tables, buf2, 16, 0); 
   fprintf(DBUG_FILE, "KEYUSE: %s.%s=%s  optimize: %u  used_tables: %s "
           "ref_table_rows: %lu  keypart_map: %0lx\n",
@@ -262,7 +261,6 @@ void print_keyuse_array(DYNAMIC_ARRAY *keyuse_array)
   DBUG_UNLOCK_FILE;
 }
 
-
 /* 
   Print the current state during query optimization.
 
@@ -387,10 +385,10 @@ void print_sjm(SJ_MATERIALIZATION_INFO *sjm)
 /*
   Debugging help: force List<...>::elem function not be removed as unused.
 */
-Item* (List<Item>::*dbug_list_item_elem_ptr)(int)= &List<Item>::elem;
-Item_equal* (List<Item_equal>::*dbug_list_item_equal_elem_ptr)(int)=
+Item* (List<Item>:: *dbug_list_item_elem_ptr)(uint)= &List<Item>::elem;
+Item_equal* (List<Item_equal>:: *dbug_list_item_equal_elem_ptr)(uint)=
   &List<Item_equal>::elem;
-TABLE_LIST* (List<TABLE_LIST>::*dbug_list_table_list_elem_ptr)(int) =
+TABLE_LIST* (List<TABLE_LIST>:: *dbug_list_table_list_elem_ptr)(uint) =
   &List<TABLE_LIST>::elem;
 
 #endif
@@ -595,13 +593,13 @@ update:     %10lu\n",
 	 tmp.ha_update_count);
   printf("\nTable status:\n\
 Opened tables: %10lu\n\
-Open tables:   %10lu\n\
-Open files:    %10lu\n\
+Open tables:   %10u\n\
+Open files:    %10u\n\
 Open streams:  %10lu\n",
 	 tmp.opened_tables,
-	 (ulong) tc_records(),
-	 (ulong) my_file_opened,
-	 (ulong) my_stream_opened);
+	 tc_records(),
+	 my_file_opened,
+	 my_stream_opened);
 
 #ifndef DONT_USE_THR_ALARM
   ALARM_INFO alarm_info;
@@ -654,3 +652,24 @@ Memory allocated by threads:             %s\n",
   puts("");
   fflush(stdout);
 }
+
+void print_keyuse_array_for_trace(THD *thd, DYNAMIC_ARRAY *keyuse_array)
+{
+  Json_writer_object wrapper(thd);
+  Json_writer_array trace_key_uses(thd, "ref_optimizer_key_uses");
+  for(uint i=0; i < keyuse_array->elements; i++)
+  {
+    KEYUSE *keyuse= (KEYUSE*)dynamic_array_ptr(keyuse_array, i);
+    Json_writer_object keyuse_elem(thd);
+    keyuse_elem.add_table_name(keyuse->table->reginfo.join_tab);
+    keyuse_elem.add("field", (keyuse->keypart == FT_KEYPART) ? "<fulltext>":
+                                        (keyuse->is_for_hash_join() ?
+                                        keyuse->table->field[keyuse->keypart]
+                                                     ->field_name.str :
+                                        keyuse->table->key_info[keyuse->key]
+                                          .key_part[keyuse->keypart]
+                                          .field->field_name.str));
+    keyuse_elem.add("equals",keyuse->val);
+    keyuse_elem.add("null_rejecting",keyuse->null_rejecting);
+  }
+}
