@@ -26,8 +26,12 @@
 #include "ha_heap.h"
 #include "sql_base.h"                    // enum_tdc_remove_table_type
 
-static handler *heap_create_handler(handlerton *, TABLE_SHARE *, MEM_ROOT *);
-static int heap_prepare_hp_create_info(TABLE *, bool, HP_CREATE_INFO *);
+static handler *heap_create_handler(handlerton *hton,
+                                    TABLE_SHARE *table, 
+                                    MEM_ROOT *mem_root);
+static int
+heap_prepare_hp_create_info(TABLE *table_arg, bool internal_table,
+                            HP_CREATE_INFO *hp_create_info);
 
 
 int heap_panic(handlerton *hton, ha_panic_function flag)
@@ -45,7 +49,6 @@ int heap_init(void *p)
 #endif
 
   heap_hton= (handlerton *)p;
-  heap_hton->state=      SHOW_OPTION_YES;
   heap_hton->db_type=    DB_TYPE_HEAP;
   heap_hton->create=     heap_create_handler;
   heap_hton->panic=      heap_panic;
@@ -224,7 +227,7 @@ void ha_heap::update_key_stats()
 }
 
 
-int ha_heap::write_row(uchar * buf)
+int ha_heap::write_row(const uchar * buf)
 {
   int res;
   if (table->next_number_field && buf == table->record[0])
@@ -364,7 +367,7 @@ int ha_heap::info(uint flag)
   HEAPINFO hp_info;
 
   if (!table)
-    return 0;
+    return 1;
 
   (void) heap_info(file,&hp_info,flag);
 
@@ -599,15 +602,16 @@ ha_rows ha_heap::records_in_range(uint inx, key_range *min_key,
 }
 
 
-static int heap_prepare_hp_create_info(TABLE *table_arg, bool internal_table,
-                                       HP_CREATE_INFO *hp_create_info)
+static int
+heap_prepare_hp_create_info(TABLE *table_arg, bool internal_table,
+                            HP_CREATE_INFO *hp_create_info)
 {
-  TABLE_SHARE *share= table_arg->s;
-  uint key, parts, mem_per_row= 0, keys= share->keys;
+  uint key, parts, mem_per_row= 0, keys= table_arg->s->keys;
   uint auto_key= 0, auto_key_type= 0;
   ha_rows max_rows;
   HP_KEYDEF *keydef;
   HA_KEYSEG *seg;
+  TABLE_SHARE *share= table_arg->s;
   bool found_real_auto_increment= 0;
 
   bzero(hp_create_info, sizeof(*hp_create_info));
@@ -615,11 +619,11 @@ static int heap_prepare_hp_create_info(TABLE *table_arg, bool internal_table,
   for (key= parts= 0; key < keys; key++)
     parts+= table_arg->key_info[key].user_defined_key_parts;
 
-  if (!my_multi_malloc(MYF(MY_WME | MY_THREAD_SPECIFIC),
-                       &keydef, keys * sizeof(HP_KEYDEF),
-                       &seg, parts * sizeof(HA_KEYSEG),
-                       NULL))
+  if (!(keydef= (HP_KEYDEF*) my_malloc(keys * sizeof(HP_KEYDEF) +
+				       parts * sizeof(HA_KEYSEG),
+				       MYF(MY_WME | MY_THREAD_SPECIFIC))))
     return my_errno;
+  seg= reinterpret_cast<HA_KEYSEG*>(keydef + keys);
   for (key= 0; key < keys; key++)
   {
     KEY *pos= table_arg->key_info+key;
