@@ -39,7 +39,7 @@ safe to look at BUF_BUDDY_STAMP_OFFSET.
 The answer lies in following invariants:
 * All blocks allocated by buddy allocator are used for compressed
 page frame.
-* A compressed table always have space_id < SRV_LOG_SPACE_FIRST_ID
+* A compressed table always have space_id < SRV_SPACE_ID_UPPER_BOUND
 * BUF_BUDDY_STAMP_OFFSET always points to the space_id field in
 a frame.
   -- The above is true because we look at these fields when the
@@ -67,16 +67,12 @@ are written.*/
 
 /** Value that we stamp on all buffers that are currently on the zip_free
 list. This value is stamped at BUF_BUDDY_STAMP_OFFSET offset */
-#define BUF_BUDDY_STAMP_FREE	 SRV_LOG_SPACE_FIRST_ID
+#define BUF_BUDDY_STAMP_FREE	 SRV_SPACE_ID_UPPER_BOUND
 
 /** Stamp value for non-free buffers. Will be overwritten by a non-zero
 value by the consumer of the block */
 #define BUF_BUDDY_STAMP_NONFREE	0XFFFFFFFFUL
 
-#if BUF_BUDDY_STAMP_FREE >= BUF_BUDDY_STAMP_NONFREE
-# error "BUF_BUDDY_STAMP_FREE >= BUF_BUDDY_STAMP_NONFREE"
-#endif
-
 /** Return type of buf_buddy_is_free() */
 enum buf_buddy_state_t {
 	BUF_BUDDY_STATE_FREE,	/*!< If the buddy to completely free */
@@ -85,6 +81,7 @@ enum buf_buddy_state_t {
 				are in use */
 };
 
+#ifdef UNIV_DEBUG_VALGRIND
 /**********************************************************************//**
 Invalidate memory area that we won't access while page is free */
 UNIV_INLINE
@@ -94,11 +91,15 @@ buf_buddy_mem_invalid(
 	buf_buddy_free_t*	buf,	/*!< in: block to check */
 	ulint			i)	/*!< in: index of zip_free[] */
 {
-  ut_ad(i <= BUF_BUDDY_SIZES);
+	const size_t	size	= BUF_BUDDY_LOW << i;
+	ut_ad(i <= BUF_BUDDY_SIZES);
 
-  MEM_CHECK_ADDRESSABLE(buf, BUF_BUDDY_LOW << i);
-  MEM_UNDEFINED(buf, BUF_BUDDY_LOW << i);
+	UNIV_MEM_ASSERT_W(buf, size);
+	UNIV_MEM_INVALID(buf, size);
 }
+#else /* UNIV_DEBUG_VALGRIND */
+# define buf_buddy_mem_invalid(buf, i) ut_ad((i) <= BUF_BUDDY_SIZES)
+#endif /* UNIV_DEBUG_VALGRIND */
 
 /**********************************************************************//**
 Check if a buddy is stamped free.
@@ -109,6 +110,7 @@ buf_buddy_stamp_is_free(
 /*====================*/
 	const buf_buddy_free_t*	buf)	/*!< in: block to check */
 {
+	compile_time_assert(BUF_BUDDY_STAMP_FREE < BUF_BUDDY_STAMP_NONFREE);
 	return(mach_read_from_4(buf->stamp.bytes + BUF_BUDDY_STAMP_OFFSET)
 	       == BUF_BUDDY_STAMP_FREE);
 }
@@ -133,13 +135,12 @@ buf_buddy_stamp_free(
 Stamps a buddy nonfree.
 @param[in,out]	buf	block to stamp
 @param[in]	i	block size */
-#define buf_buddy_stamp_nonfree(buf, i) do {				\
-	buf_buddy_mem_invalid(buf, i);					\
-	memset(buf->stamp.bytes + BUF_BUDDY_STAMP_OFFSET, 0xff, 4);	\
-} while (0)
-#if BUF_BUDDY_STAMP_NONFREE != 0xffffffff
-# error "BUF_BUDDY_STAMP_NONFREE != 0xffffffff"
-#endif
+static inline void buf_buddy_stamp_nonfree(buf_buddy_free_t* buf, ulint i)
+{
+	buf_buddy_mem_invalid(buf, i);
+	compile_time_assert(BUF_BUDDY_STAMP_NONFREE == 0xffffffffU);
+	memset(buf->stamp.bytes + BUF_BUDDY_STAMP_OFFSET, 0xff, 4);
+}
 
 /**********************************************************************//**
 Get the offset of the buddy of a compressed page frame.
@@ -155,7 +156,7 @@ buf_buddy_get(
 	ut_ad(size >= BUF_BUDDY_LOW);
 	ut_ad(BUF_BUDDY_LOW <= UNIV_ZIP_SIZE_MIN);
 	ut_ad(size < BUF_BUDDY_HIGH);
-	ut_ad(BUF_BUDDY_HIGH == UNIV_PAGE_SIZE);
+	ut_ad(BUF_BUDDY_HIGH == srv_page_size);
 	ut_ad(!ut_align_offset(page, size));
 
 	if (((ulint) page) & size) {
@@ -180,13 +181,8 @@ struct	CheckZipFree {
 };
 
 /** Validate a buddy list.
-@param[in]	buf_pool	buffer pool instance
 @param[in]	i		buddy size to validate */
-static
-void
-buf_buddy_list_validate(
-	const buf_pool_t*	buf_pool,
-	ulint			i)
+static void buf_buddy_list_validate(ulint i)
 {
 	ut_list_validate(buf_pool->zip_free[i], CheckZipFree(i));
 }
@@ -194,18 +190,14 @@ buf_buddy_list_validate(
 /**********************************************************************//**
 Debug function to validate that a buffer is indeed free i.e.: in the
 zip_free[].
+@param[in]	buf		block to check
+@param[in]	i		index of buf_pool->zip_free[]
 @return true if free */
-UNIV_INLINE
-bool
-buf_buddy_check_free(
-/*=================*/
-	buf_pool_t*		buf_pool,/*!< in: buffer pool instance */
-	const buf_buddy_free_t*	buf,	/*!< in: block to check */
-	ulint			i)	/*!< in: index of buf_pool->zip_free[] */
+static bool buf_buddy_check_free(const buf_buddy_free_t* buf, ulint i)
 {
 	const ulint	size	= BUF_BUDDY_LOW << i;
 
-	ut_ad(buf_pool_mutex_own(buf_pool));
+	ut_ad(mutex_own(&buf_pool->mutex));
 	ut_ad(!ut_align_offset(buf, size));
 	ut_ad(i >= buf_buddy_get_slot(UNIV_ZIP_SIZE_MIN));
 
@@ -267,99 +259,85 @@ buf_buddy_is_free(
 	       : BUF_BUDDY_STATE_PARTIALLY_USED);
 }
 
-/**********************************************************************//**
-Add a block to the head of the appropriate buddy free list. */
+/** Add a block to the head of the appropriate buddy free list.
+@param[in,out]	buf		block to be freed
+@param[in]	i		index of buf_pool->zip_free[] */
 UNIV_INLINE
 void
-buf_buddy_add_to_free(
-/*==================*/
-	buf_pool_t*		buf_pool,	/*!< in: buffer pool instance */
-	buf_buddy_free_t*	buf,		/*!< in,own: block to be freed */
-	ulint			i)		/*!< in: index of
-						buf_pool->zip_free[] */
+buf_buddy_add_to_free(buf_buddy_free_t* buf, ulint i)
 {
-	ut_ad(buf_pool_mutex_own(buf_pool));
+	ut_ad(mutex_own(&buf_pool->mutex));
 	ut_ad(buf_pool->zip_free[i].start != buf);
 
 	buf_buddy_stamp_free(buf, i);
 	UT_LIST_ADD_FIRST(buf_pool->zip_free[i], buf);
-	ut_d(buf_buddy_list_validate(buf_pool, i));
+	ut_d(buf_buddy_list_validate(i));
 }
 
-/**********************************************************************//**
-Remove a block from the appropriate buddy free list. */
+/** Remove a block from the appropriate buddy free list.
+@param[in,out]	buf		block to be freed
+@param[in]	i		index of buf_pool->zip_free[] */
 UNIV_INLINE
 void
-buf_buddy_remove_from_free(
-/*=======================*/
-	buf_pool_t*		buf_pool,	/*!< in: buffer pool instance */
-	buf_buddy_free_t*	buf,		/*!< in,own: block to be
-						freed */
-	ulint			i)		/*!< in: index of
-						buf_pool->zip_free[] */
+buf_buddy_remove_from_free(buf_buddy_free_t* buf, ulint i)
 {
-	ut_ad(buf_pool_mutex_own(buf_pool));
-	ut_ad(buf_buddy_check_free(buf_pool, buf, i));
+	ut_ad(mutex_own(&buf_pool->mutex));
+	ut_ad(buf_buddy_check_free(buf, i));
 
 	UT_LIST_REMOVE(buf_pool->zip_free[i], buf);
 	buf_buddy_stamp_nonfree(buf, i);
 }
 
-/**********************************************************************//**
-Try to allocate a block from buf_pool->zip_free[].
+/** Try to allocate a block from buf_pool->zip_free[].
+@param[in]	i		index of buf_pool->zip_free[]
 @return allocated block, or NULL if buf_pool->zip_free[] was empty */
-static
-buf_buddy_free_t*
-buf_buddy_alloc_zip(
-/*================*/
-	buf_pool_t*	buf_pool,	/*!< in: buffer pool instance */
-	ulint		i)		/*!< in: index of buf_pool->zip_free[] */
+static buf_buddy_free_t* buf_buddy_alloc_zip(ulint i)
 {
 	buf_buddy_free_t*	buf;
 
-	ut_ad(buf_pool_mutex_own(buf_pool));
+	ut_ad(mutex_own(&buf_pool->mutex));
 	ut_a(i < BUF_BUDDY_SIZES);
 	ut_a(i >= buf_buddy_get_slot(UNIV_ZIP_SIZE_MIN));
 
-	ut_d(buf_buddy_list_validate(buf_pool, i));
+	ut_d(buf_buddy_list_validate(i));
 
 	buf = UT_LIST_GET_FIRST(buf_pool->zip_free[i]);
 
 	if (buf_pool->curr_size < buf_pool->old_size
 	    && UT_LIST_GET_LEN(buf_pool->withdraw)
-		< buf_pool->withdraw_target) {
+	    < buf_pool->withdraw_target) {
 
 		while (buf != NULL
-		       && buf_frame_will_withdrawn(
-			       buf_pool, reinterpret_cast<byte*>(buf))) {
+		       && buf_frame_will_be_withdrawn(
+			       reinterpret_cast<byte*>(buf))) {
 			/* This should be withdrawn, not to be allocated */
 			buf = UT_LIST_GET_NEXT(list, buf);
 		}
 	}
 
 	if (buf) {
-		buf_buddy_remove_from_free(buf_pool, buf, i);
+		buf_buddy_remove_from_free(buf, i);
 	} else if (i + 1 < BUF_BUDDY_SIZES) {
 		/* Attempt to split. */
-		buf = buf_buddy_alloc_zip(buf_pool, i + 1);
+		buf = buf_buddy_alloc_zip(i + 1);
 
 		if (buf) {
 			buf_buddy_free_t* buddy =
 				reinterpret_cast<buf_buddy_free_t*>(
 					buf->stamp.bytes
 					+ (BUF_BUDDY_LOW << i));
-
-			ut_ad(!buf_pool_contains_zip(buf_pool, buddy));
-			buf_buddy_add_to_free(buf_pool, buddy, i);
+			ut_ad(!buf_pool_contains_zip(buddy));
+			buf_buddy_add_to_free(buddy, i);
 		}
 	}
 
 	if (buf) {
 		/* Trash the page other than the BUF_BUDDY_STAMP_NONFREE. */
-		MEM_UNDEFINED(buf, BUF_BUDDY_STAMP_OFFSET);
-		MEM_UNDEFINED(BUF_BUDDY_STAMP_OFFSET + 4 + buf->stamp.bytes,
-			      (BUF_BUDDY_LOW << i)
-			      - (BUF_BUDDY_STAMP_OFFSET + 4));
+		UNIV_MEM_TRASH((void*) buf, ~i, BUF_BUDDY_STAMP_OFFSET);
+		UNIV_MEM_TRASH(BUF_BUDDY_STAMP_OFFSET + 4
+			       + buf->stamp.bytes, ~i,
+			       (BUF_BUDDY_LOW << i)
+			       - (BUF_BUDDY_STAMP_OFFSET + 4));
 		ut_ad(mach_read_from_4(buf->stamp.bytes
 				       + BUF_BUDDY_STAMP_OFFSET)
 		      == BUF_BUDDY_STAMP_NONFREE);
@@ -368,22 +346,19 @@ buf_buddy_alloc_zip(
 	return(buf);
 }
 
-/**********************************************************************//**
-Deallocate a buffer frame of UNIV_PAGE_SIZE. */
+/** Deallocate a buffer frame of srv_page_size.
+@param[in]	buf		buffer frame to deallocate */
 static
 void
-buf_buddy_block_free(
-/*=================*/
-	buf_pool_t*	buf_pool,	/*!< in: buffer pool instance */
-	void*		buf)		/*!< in: buffer frame to deallocate */
+buf_buddy_block_free(void* buf)
 {
 	const ulint	fold	= BUF_POOL_ZIP_FOLD_PTR(buf);
 	buf_page_t*	bpage;
 	buf_block_t*	block;
 
-	ut_ad(buf_pool_mutex_own(buf_pool));
+	ut_ad(mutex_own(&buf_pool->mutex));
 	ut_ad(!mutex_own(&buf_pool->zip_mutex));
-	ut_a(!ut_align_offset(buf, UNIV_PAGE_SIZE));
+	ut_a(!ut_align_offset(buf, srv_page_size));
 
 	HASH_SEARCH(hash, buf_pool->zip_hash, fold, buf_page_t*, bpage,
 		    ut_ad(buf_page_get_state(bpage) == BUF_BLOCK_MEMORY
@@ -397,9 +372,7 @@ buf_buddy_block_free(
 	HASH_DELETE(buf_page_t, hash, buf_pool->zip_hash, fold, bpage);
 
 	ut_d(memset(buf, 0, srv_page_size));
-#ifdef HAVE_valgrind_or_MSAN
-	MEM_UNDEFINED(buf, srv_page_size);
-#endif /* HAVE_valgrind_or_MSAN */
+	UNIV_MEM_INVALID(buf, srv_page_size);
 
 	block = (buf_block_t*) bpage;
 	buf_page_mutex_enter(block);
@@ -418,16 +391,15 @@ buf_buddy_block_register(
 /*=====================*/
 	buf_block_t*	block)	/*!< in: buffer frame to allocate */
 {
-	buf_pool_t*	buf_pool = buf_pool_from_block(block);
 	const ulint	fold = BUF_POOL_ZIP_FOLD(block);
-	ut_ad(buf_pool_mutex_own(buf_pool));
+	ut_ad(mutex_own(&buf_pool->mutex));
 	ut_ad(!mutex_own(&buf_pool->zip_mutex));
 	ut_ad(buf_block_get_state(block) == BUF_BLOCK_READY_FOR_USE);
 
 	buf_block_set_state(block, BUF_BLOCK_MEMORY);
 
 	ut_a(block->frame);
-	ut_a(!ut_align_offset(block->frame, UNIV_PAGE_SIZE));
+	ut_a(!ut_align_offset(block->frame, srv_page_size));
 
 	ut_ad(!block->page.in_page_hash);
 	ut_ad(!block->page.in_zip_hash);
@@ -437,19 +409,14 @@ buf_buddy_block_register(
 	ut_d(buf_pool->buddy_n_frames++);
 }
 
-/**********************************************************************//**
-Allocate a block from a bigger object.
+/** Allocate a block from a bigger object.
+@param[in]	buf		a block that is free to use
+@param[in]	i		index of buf_pool->zip_free[]
+@param[in]	j		size of buf as an index of buf_pool->zip_free[]
 @return allocated block */
 static
 void*
-buf_buddy_alloc_from(
-/*=================*/
-	buf_pool_t*	buf_pool,	/*!< in: buffer pool instance */
-	void*		buf,		/*!< in: a block that is free to use */
-	ulint		i,		/*!< in: index of
-					buf_pool->zip_free[] */
-	ulint		j)		/*!< in: size of buf as an index
-					of buf_pool->zip_free[] */
+buf_buddy_alloc_from(void* buf, ulint i, ulint j)
 {
 	ulint	offs	= BUF_BUDDY_LOW << j;
 	ut_ad(j <= BUF_BUDDY_SIZES);
@@ -466,39 +433,28 @@ buf_buddy_alloc_from(
 
 		zip_buf = reinterpret_cast<buf_buddy_free_t*>(
 			reinterpret_cast<byte*>(buf) + offs);
-		buf_buddy_add_to_free(buf_pool, zip_buf, j);
+		buf_buddy_add_to_free(zip_buf, j);
 	}
 
 	buf_buddy_stamp_nonfree(reinterpret_cast<buf_buddy_free_t*>(buf), i);
 	return(buf);
 }
 
-/**********************************************************************//**
-Allocate a block.  The thread calling this function must hold
-buf_pool->mutex and must not hold buf_pool->zip_mutex or any block->mutex.
-The buf_pool_mutex may be released and reacquired.
+/** Allocate a block.
+@param[in]	i	index of buf_pool->zip_free[] or BUF_BUDDY_SIZES
+@param[out]	lru	whether buf_pool->mutex was temporarily released
 @return allocated block, never NULL */
-void*
-buf_buddy_alloc_low(
-/*================*/
-	buf_pool_t*	buf_pool,	/*!< in/out: buffer pool instance */
-	ulint		i,		/*!< in: index of buf_pool->zip_free[],
-					or BUF_BUDDY_SIZES */
-	ibool*		lru)		/*!< in: pointer to a variable that
-					will be assigned TRUE if storage was
-					allocated from the LRU list and
-					buf_pool->mutex was temporarily
-					released */
+byte *buf_buddy_alloc_low(ulint i, bool *lru)
 {
 	buf_block_t*	block;
 
-	ut_ad(buf_pool_mutex_own(buf_pool));
+	ut_ad(mutex_own(&buf_pool->mutex));
 	ut_ad(!mutex_own(&buf_pool->zip_mutex));
 	ut_ad(i >= buf_buddy_get_slot(UNIV_ZIP_SIZE_MIN));
 
 	if (i < BUF_BUDDY_SIZES) {
 		/* Try to allocate from the buddy system. */
-		block = (buf_block_t*) buf_buddy_alloc_zip(buf_pool, i);
+		block = (buf_block_t*) buf_buddy_alloc_zip(i);
 
 		if (block) {
 			goto func_exit;
@@ -506,81 +462,71 @@ buf_buddy_alloc_low(
 	}
 
 	/* Try allocating from the buf_pool->free list. */
-	block = buf_LRU_get_free_only(buf_pool);
+	block = buf_LRU_get_free_only();
 
 	if (block) {
-
 		goto alloc_big;
 	}
 
 	/* Try replacing an uncompressed page in the buffer pool. */
-	buf_pool_mutex_exit(buf_pool);
-	block = buf_LRU_get_free_block(buf_pool);
-	*lru = TRUE;
-	buf_pool_mutex_enter(buf_pool);
+	mutex_exit(&buf_pool->mutex);
+	block = buf_LRU_get_free_block();
+	mutex_enter(&buf_pool->mutex);
+	if (lru) {
+		*lru = true;
+	}
 
 alloc_big:
 	buf_buddy_block_register(block);
 
 	block = (buf_block_t*) buf_buddy_alloc_from(
-		buf_pool, block->frame, i, BUF_BUDDY_SIZES);
+		block->frame, i, BUF_BUDDY_SIZES);
 
 func_exit:
 	buf_pool->buddy_stat[i].used++;
-	return(block);
+	return reinterpret_cast<byte*>(block);
 }
 
-/**********************************************************************//**
-Try to relocate a block.
+/** Try to relocate a block. The caller must hold zip_free_mutex, and this
+function will release and lock it again.
+@param[in]	src		block to relocate
+@param[in]	dst		free block to relocated to
+@param[in]	i		index of buf_pool->zip_free[]
+@param[in]	force		true if we must relocated always
 @return true if relocated */
-static
-bool
-buf_buddy_relocate(
-/*===============*/
-	buf_pool_t*	buf_pool,	/*!< in: buffer pool instance */
-	void*		src,		/*!< in: block to relocate */
-	void*		dst,		/*!< in: free block to relocate to */
-	ulint		i,		/*!< in: index of
-					buf_pool->zip_free[] */
-	bool		force)		/*!< in: true if we must relocate
-					always */
+static bool buf_buddy_relocate(void* src, void* dst, ulint i, bool force)
 {
 	buf_page_t*	bpage;
 	const ulint	size = BUF_BUDDY_LOW << i;
 	ulint		space;
 	ulint		offset;
 
-	ut_ad(buf_pool_mutex_own(buf_pool));
+	ut_ad(mutex_own(&buf_pool->mutex));
 	ut_ad(!mutex_own(&buf_pool->zip_mutex));
 	ut_ad(!ut_align_offset(src, size));
 	ut_ad(!ut_align_offset(dst, size));
 	ut_ad(i >= buf_buddy_get_slot(UNIV_ZIP_SIZE_MIN));
-	MEM_CHECK_ADDRESSABLE(dst, size);
+	UNIV_MEM_ASSERT_W(dst, size);
 
 	space	= mach_read_from_4((const byte*) src
 				   + FIL_PAGE_ARCH_LOG_NO_OR_SPACE_ID);
 	offset	= mach_read_from_4((const byte*) src
 				   + FIL_PAGE_OFFSET);
 
-	/* Suppress Valgrind or MSAN warnings. */
-	MEM_MAKE_DEFINED(&space, sizeof space);
-	MEM_MAKE_DEFINED(&offset, sizeof offset);
+	/* Suppress Valgrind warnings about conditional jump
+	on uninitialized value. */
+	UNIV_MEM_VALID(&space, sizeof space);
+	UNIV_MEM_VALID(&offset, sizeof offset);
 
 	ut_ad(space != BUF_BUDDY_STAMP_FREE);
 
 	const page_id_t	page_id(space, offset);
 
-	/* If space,offset is bogus, then we know that the
-	buf_page_hash_get_low() call below will return NULL. */
-	if (!force && buf_pool != buf_pool_get(page_id)) {
-		return(false);
-	}
-
-	rw_lock_t*	hash_lock = buf_page_hash_lock_get(buf_pool, page_id);
+	rw_lock_t*	hash_lock = buf_page_hash_lock_get(page_id);
 
 	rw_lock_x_lock(hash_lock);
 
-	bpage = buf_page_hash_get_low(buf_pool, page_id);
+	bpage = buf_page_hash_get_low(page_id);
 
 	if (!bpage || bpage->zip.data != src) {
 		/* The block has probably been freshly
@@ -600,8 +546,7 @@ buf_buddy_relocate(
 		bpage = UT_LIST_GET_FIRST(buf_pool->LRU);
 		while (bpage != NULL) {
 			if (bpage->zip.data == src) {
-				hash_lock = buf_page_hash_lock_get(
-					buf_pool, bpage->id);
+				hash_lock = buf_page_hash_lock_get(bpage->id);
 				rw_lock_x_lock(hash_lock);
 				break;
 			}
@@ -626,7 +571,7 @@ buf_buddy_relocate(
 
 	/* The block must have been allocated, but it may
 	contain uninitialized data. */
-	MEM_CHECK_ADDRESSABLE(src, size);
+	UNIV_MEM_ASSERT_W(src, size);
 
 	BPageMutex*	block_mutex = buf_page_get_mutex(bpage);
 
@@ -660,20 +605,15 @@ buf_buddy_relocate(
 	return(false);
 }
 
-/**********************************************************************//**
-Deallocate a block. */
-void
-buf_buddy_free_low(
-/*===============*/
-	buf_pool_t*	buf_pool,	/*!< in: buffer pool instance */
-	void*		buf,		/*!< in: block to be freed, must not be
-					pointed to by the buffer pool */
-	ulint		i)		/*!< in: index of buf_pool->zip_free[],
-					or BUF_BUDDY_SIZES */
+/** Deallocate a block.
+@param[in]	buf	block to be freed, must not be pointed to
+			by the buffer pool
+@param[in]	i	index of buf_pool->zip_free[], or BUF_BUDDY_SIZES */
+void buf_buddy_free_low(void* buf, ulint i)
 {
 	buf_buddy_free_t*	buddy;
 
-	ut_ad(buf_pool_mutex_own(buf_pool));
+	ut_ad(mutex_own(&buf_pool->mutex));
 	ut_ad(!mutex_own(&buf_pool->zip_mutex));
 	ut_ad(i <= BUF_BUDDY_SIZES);
 	ut_ad(i >= buf_buddy_get_slot(UNIV_ZIP_SIZE_MIN));
@@ -681,16 +621,16 @@ buf_buddy_free_low(
 
 	buf_pool->buddy_stat[i].used--;
 recombine:
-	MEM_UNDEFINED(buf, BUF_BUDDY_LOW << i);
+	UNIV_MEM_ALLOC(buf, BUF_BUDDY_LOW << i);
 
 	if (i == BUF_BUDDY_SIZES) {
-		buf_buddy_block_free(buf_pool, buf);
+		buf_buddy_block_free(buf);
 		return;
 	}
 
 	ut_ad(i < BUF_BUDDY_SIZES);
 	ut_ad(buf == ut_align_down(buf, BUF_BUDDY_LOW << i));
-	ut_ad(!buf_pool_contains_zip(buf_pool, buf));
+	ut_ad(!buf_pool_contains_zip(buf));
 
 	/* Do not recombine blocks if there are few free blocks.
 	We may waste up to 15360*max_len bytes to free blocks
@@ -708,16 +648,16 @@ recombine:
 	switch (buf_buddy_is_free(buddy, i)) {
 	case BUF_BUDDY_STATE_FREE:
 		/* The buddy is free: recombine */
-		buf_buddy_remove_from_free(buf_pool, buddy, i);
+		buf_buddy_remove_from_free(buddy, i);
 buddy_is_free:
-		ut_ad(!buf_pool_contains_zip(buf_pool, buddy));
+		ut_ad(!buf_pool_contains_zip(buddy));
 		i++;
 		buf = ut_align_down(buf, BUF_BUDDY_LOW << i);
 
 		goto recombine;
 
 	case BUF_BUDDY_STATE_USED:
-		ut_d(buf_buddy_list_validate(buf_pool, i));
+		ut_d(buf_buddy_list_validate(i));
 
 		/* The buddy is not free. Is there a free block of
 		this size? */
@@ -727,17 +667,15 @@ buddy_is_free:
 			/* Remove the block from the free list, because
 			a successful buf_buddy_relocate() will overwrite
 			zip_free->list. */
-			buf_buddy_remove_from_free(buf_pool, zip_buf, i);
+			buf_buddy_remove_from_free(zip_buf, i);
 
 			/* Try to relocate the buddy of buf to the free
 			block. */
-			if (buf_buddy_relocate(buf_pool, buddy, zip_buf, i,
-					       false)) {
-
+			if (buf_buddy_relocate(buddy, zip_buf, i, false)) {
 				goto buddy_is_free;
 			}
 
-			buf_buddy_add_to_free(buf_pool, zip_buf, i);
+			buf_buddy_add_to_free(zip_buf, i);
 		}
 
 		break;
@@ -749,40 +687,32 @@ buddy_is_free:
 
 func_exit:
 	/* Free the block to the buddy list. */
-	buf_buddy_add_to_free(buf_pool,
-			      reinterpret_cast<buf_buddy_free_t*>(buf),
-			      i);
+	buf_buddy_add_to_free(reinterpret_cast<buf_buddy_free_t*>(buf), i);
 }
 
-/** Reallocate a block.
-@param[in]	buf_pool	buffer pool instance
-@param[in]	buf		block to be reallocated, must be pointed
-to by the buffer pool
-@param[in]	size		block size, up to UNIV_PAGE_SIZE
-@retval false	if failed because of no free blocks. */
+/** Try to reallocate a block.
+@param[in]	buf	buf_pool block to be reallocated
+@param[in]	size	block size, up to srv_page_size
+@return	whether the reallocation succeeded */
 bool
-buf_buddy_realloc(
-	buf_pool_t*	buf_pool,
-	void*		buf,
-	ulint		size)
+buf_buddy_realloc(void* buf, ulint size)
 {
 	buf_block_t*	block = NULL;
 	ulint		i = buf_buddy_get_slot(size);
 
-	ut_ad(buf_pool_mutex_own(buf_pool));
+	ut_ad(mutex_own(&buf_pool->mutex));
 	ut_ad(!mutex_own(&buf_pool->zip_mutex));
 	ut_ad(i <= BUF_BUDDY_SIZES);
 	ut_ad(i >= buf_buddy_get_slot(UNIV_ZIP_SIZE_MIN));
 
 	if (i < BUF_BUDDY_SIZES) {
 		/* Try to allocate from the buddy system. */
-		block = reinterpret_cast<buf_block_t*>(
-			buf_buddy_alloc_zip(buf_pool, i));
+		block = reinterpret_cast<buf_block_t*>(buf_buddy_alloc_zip(i));
 	}
 
 	if (block == NULL) {
 		/* Try allocating from the buf_pool->free list. */
-		block = buf_LRU_get_free_only(buf_pool);
+		block = buf_LRU_get_free_only();
 
 		if (block == NULL) {
 			return(false); /* free_list was not enough */
@@ -792,30 +722,27 @@ buf_buddy_realloc(
 
 		block = reinterpret_cast<buf_block_t*>(
 			buf_buddy_alloc_from(
-				buf_pool, block->frame, i, BUF_BUDDY_SIZES));
+				block->frame, i, BUF_BUDDY_SIZES));
 	}
 
 	buf_pool->buddy_stat[i].used++;
 
 	/* Try to relocate the buddy of buf to the free block. */
-	if (buf_buddy_relocate(buf_pool, buf, block, i, true)) {
+	if (buf_buddy_relocate(buf, block, i, true)) {
 		/* succeeded */
-		buf_buddy_free_low(buf_pool, buf, i);
+		buf_buddy_free_low(buf, i);
 	} else {
 		/* failed */
-		buf_buddy_free_low(buf_pool, block, i);
+		buf_buddy_free_low(block, i);
 	}
 
 	return(true); /* free_list was enough */
 }
 
-/** Combine all pairs of free buddies.
-@param[in]	buf_pool	buffer pool instance */
-void
-buf_buddy_condense_free(
-	buf_pool_t*	buf_pool)
+/** Combine all pairs of free buddies. */
+void buf_buddy_condense_free()
 {
-	ut_ad(buf_pool_mutex_own(buf_pool));
+	ut_ad(mutex_own(&buf_pool->mutex));
 	ut_ad(buf_pool->curr_size < buf_pool->old_size);
 
 	for (ulint i = 0; i < UT_ARR_SIZE(buf_pool->zip_free); ++i) {
@@ -824,8 +751,8 @@ buf_buddy_condense_free(
 
 		/* seek to withdraw target */
 		while (buf != NULL
-		       && !buf_frame_will_withdrawn(
-				buf_pool, reinterpret_cast<byte*>(buf))) {
+		       && !buf_frame_will_be_withdrawn(
+			       reinterpret_cast<byte*>(buf))) {
 			buf = UT_LIST_GET_NEXT(list, buf);
 		}
 
@@ -842,8 +769,7 @@ buf_buddy_condense_free(
 			/* seek to the next withdraw target */
 			while (true) {
 				while (next != NULL
-				       && !buf_frame_will_withdrawn(
-						buf_pool,
+				       && !buf_frame_will_be_withdrawn(
 						reinterpret_cast<byte*>(next))) {
 					 next = UT_LIST_GET_NEXT(list, next);
 				}
@@ -859,10 +785,10 @@ buf_buddy_condense_free(
 			    == BUF_BUDDY_STATE_FREE) {
 				/* Both buf and buddy are free.
 				Try to combine them. */
-				buf_buddy_remove_from_free(buf_pool, buf, i);
+				buf_buddy_remove_from_free(buf, i);
 				buf_pool->buddy_stat[i].used++;
 
-				buf_buddy_free_low(buf_pool, buf, i);
+				buf_buddy_free_low(buf, i);
 			}
 
 			buf = next;
