@@ -1,7 +1,7 @@
 /*****************************************************************************
 
 Copyright (c) 1995, 2017, Oracle and/or its affiliates. All Rights Reserved.
-Copyright (c) 2013, 2019, MariaDB Corporation.
+Copyright (c) 2013, 2020, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -71,25 +71,13 @@ buf_dblwr_page_inside(
 	return(FALSE);
 }
 
-/****************************************************************//**
-Calls buf_page_get() on the TRX_SYS_PAGE and returns a pointer to the
-doublewrite buffer within it.
-@return pointer to the doublewrite buffer within the filespace header
-page. */
-UNIV_INLINE
-byte*
-buf_dblwr_get(
-/*==========*/
-	mtr_t*	mtr)	/*!< in/out: MTR to hold the page latch */
+/** @return the TRX_SYS page */
+inline buf_block_t *buf_dblwr_trx_sys_get(mtr_t *mtr)
 {
-	buf_block_t*	block;
-
-	block = buf_page_get(page_id_t(TRX_SYS_SPACE, TRX_SYS_PAGE_NO),
-			     0, RW_X_LATCH, mtr);
-
-	buf_block_dbg_add_level(block, SYNC_NO_ORDER_CHECK);
-
-	return(buf_block_get_frame(block) + TRX_SYS_DOUBLEWRITE);
+  buf_block_t *block= buf_page_get(page_id_t(TRX_SYS_SPACE, TRX_SYS_PAGE_NO),
+                                   0, RW_X_LATCH, mtr);
+  buf_block_dbg_add_level(block, SYNC_NO_ORDER_CHECK);
+  return block;
 }
 
 /********************************************************************//**
@@ -106,12 +94,7 @@ buf_dblwr_sync_datafiles()
 
 /****************************************************************//**
 Creates or initialializes the doublewrite buffer at a database start. */
-static
-void
-buf_dblwr_init(
-/*===========*/
-	byte*	doublewrite)	/*!< in: pointer to the doublewrite buf
-				header on trx sys page */
+static void buf_dblwr_init(const byte *doublewrite)
 {
 	ulint	buf_size;
 
@@ -143,12 +126,9 @@ buf_dblwr_init(
 	buf_dblwr->in_use = static_cast<bool*>(
 		ut_zalloc_nokey(buf_size * sizeof(bool)));
 
-	buf_dblwr->write_buf_unaligned = static_cast<byte*>(
-		ut_malloc_nokey((1 + buf_size) << srv_page_size_shift));
-
 	buf_dblwr->write_buf = static_cast<byte*>(
-		ut_align(buf_dblwr->write_buf_unaligned,
-			 srv_page_size));
+		aligned_malloc(buf_size << srv_page_size_shift,
+			       srv_page_size));
 
 	buf_dblwr->buf_block_arr = static_cast<buf_page_t**>(
 		ut_zalloc_nokey(buf_size * sizeof(void*)));
@@ -164,7 +144,6 @@ buf_dblwr_create()
 {
 	buf_block_t*	block2;
 	buf_block_t*	new_block;
-	byte*	doublewrite;
 	byte*	fseg_header;
 	ulint	page_no;
 	ulint	prev_page_no;
@@ -180,14 +159,15 @@ start_again:
 	mtr.start();
 	buf_dblwr_being_created = TRUE;
 
-	doublewrite = buf_dblwr_get(&mtr);
+	buf_block_t *trx_sys_block = buf_dblwr_trx_sys_get(&mtr);
 
-	if (mach_read_from_4(doublewrite + TRX_SYS_DOUBLEWRITE_MAGIC)
+	if (mach_read_from_4(TRX_SYS_DOUBLEWRITE + TRX_SYS_DOUBLEWRITE_MAGIC
+			     + trx_sys_block->frame)
 	    == TRX_SYS_DOUBLEWRITE_MAGIC_N) {
 		/* The doublewrite buffer has already been created:
 		just read in some numbers */
 
-		buf_dblwr_init(doublewrite);
+		buf_dblwr_init(TRX_SYS_DOUBLEWRITE + trx_sys_block->frame);
 
 		mtr.commit();
 		buf_dblwr_being_created = FALSE;
@@ -229,7 +209,8 @@ too_small:
 
 	buf_block_dbg_add_level(block2, SYNC_NO_ORDER_CHECK);
 
-	fseg_header = doublewrite + TRX_SYS_DOUBLEWRITE_FSEG;
+	fseg_header = TRX_SYS_DOUBLEWRITE + TRX_SYS_DOUBLEWRITE_FSEG
+		+ trx_sys_block->frame;
 	prev_page_no = 0;
 
 	for (i = 0; i < TRX_SYS_DOUBLEWRITE_BLOCKS * TRX_SYS_DOUBLEWRITE_BLOCK_SIZE
@@ -265,30 +246,38 @@ too_small:
 		recv_parse_or_apply_log_rec_body() will see a valid
 		page type. The flushes of new_block are actually
 		unnecessary here.  */
-		ut_d(mlog_write_ulint(FIL_PAGE_TYPE + new_block->frame,
-				      FIL_PAGE_TYPE_SYS, MLOG_2BYTES, &mtr));
+		ut_d(mtr.write<2>(*new_block,
+				  FIL_PAGE_TYPE + new_block->frame,
+				  FIL_PAGE_TYPE_SYS));
 
 		if (i == FSP_EXTENT_SIZE / 2) {
 			ut_a(page_no == FSP_EXTENT_SIZE);
-			mlog_write_ulint(doublewrite
-					 + TRX_SYS_DOUBLEWRITE_BLOCK1,
-					 page_no, MLOG_4BYTES, &mtr);
-			mlog_write_ulint(doublewrite
-					 + TRX_SYS_DOUBLEWRITE_REPEAT
-					 + TRX_SYS_DOUBLEWRITE_BLOCK1,
-					 page_no, MLOG_4BYTES, &mtr);
+			mtr.write<4>(*trx_sys_block,
+				     TRX_SYS_DOUBLEWRITE
+				     + TRX_SYS_DOUBLEWRITE_BLOCK1
+				     + trx_sys_block->frame,
+				     page_no);
+			mtr.write<4>(*trx_sys_block,
+				     TRX_SYS_DOUBLEWRITE
+				     + TRX_SYS_DOUBLEWRITE_REPEAT
+				     + TRX_SYS_DOUBLEWRITE_BLOCK1
+				     + trx_sys_block->frame,
+				     page_no);
 
 		} else if (i == FSP_EXTENT_SIZE / 2
 			   + TRX_SYS_DOUBLEWRITE_BLOCK_SIZE) {
 			ut_a(page_no == 2 * FSP_EXTENT_SIZE);
-			mlog_write_ulint(doublewrite
-					 + TRX_SYS_DOUBLEWRITE_BLOCK2,
-					 page_no, MLOG_4BYTES, &mtr);
-			mlog_write_ulint(doublewrite
-					 + TRX_SYS_DOUBLEWRITE_REPEAT
-					 + TRX_SYS_DOUBLEWRITE_BLOCK2,
-					 page_no, MLOG_4BYTES, &mtr);
-
+			mtr.write<4>(*trx_sys_block,
+				     TRX_SYS_DOUBLEWRITE
+				     + TRX_SYS_DOUBLEWRITE_BLOCK2
+				     + trx_sys_block->frame,
+				     page_no);
+			mtr.write<4>(*trx_sys_block,
+				     TRX_SYS_DOUBLEWRITE
+				     + TRX_SYS_DOUBLEWRITE_REPEAT
+				     + TRX_SYS_DOUBLEWRITE_BLOCK2
+				     + trx_sys_block->frame,
+				     page_no);
 		} else if (i > FSP_EXTENT_SIZE / 2) {
 			ut_a(page_no == prev_page_no + 1);
 		}
@@ -303,29 +292,32 @@ too_small:
 			lock the fseg header too many times. Since
 			this code is not done while any other threads
 			are active, restart the MTR occasionally. */
-			mtr_commit(&mtr);
-			mtr_start(&mtr);
-			doublewrite = buf_dblwr_get(&mtr);
-			fseg_header = doublewrite
-				      + TRX_SYS_DOUBLEWRITE_FSEG;
+			mtr.commit();
+			mtr.start();
+			trx_sys_block = buf_dblwr_trx_sys_get(&mtr);
+			fseg_header = TRX_SYS_DOUBLEWRITE
+				+ TRX_SYS_DOUBLEWRITE_FSEG
+				+ trx_sys_block->frame;
 		}
 
 		prev_page_no = page_no;
 	}
 
-	mlog_write_ulint(doublewrite + TRX_SYS_DOUBLEWRITE_MAGIC,
-			 TRX_SYS_DOUBLEWRITE_MAGIC_N,
-			 MLOG_4BYTES, &mtr);
-	mlog_write_ulint(doublewrite + TRX_SYS_DOUBLEWRITE_MAGIC
-			 + TRX_SYS_DOUBLEWRITE_REPEAT,
-			 TRX_SYS_DOUBLEWRITE_MAGIC_N,
-			 MLOG_4BYTES, &mtr);
+	mtr.write<4>(*trx_sys_block,
+		     TRX_SYS_DOUBLEWRITE + TRX_SYS_DOUBLEWRITE_MAGIC
+		     + trx_sys_block->frame,
+		     TRX_SYS_DOUBLEWRITE_MAGIC_N);
+	mtr.write<4>(*trx_sys_block,
+		     TRX_SYS_DOUBLEWRITE + TRX_SYS_DOUBLEWRITE_MAGIC
+		     + TRX_SYS_DOUBLEWRITE_REPEAT
+		     + trx_sys_block->frame,
+		     TRX_SYS_DOUBLEWRITE_MAGIC_N);
 
-	mlog_write_ulint(doublewrite
-			 + TRX_SYS_DOUBLEWRITE_SPACE_ID_STORED,
-			 TRX_SYS_DOUBLEWRITE_SPACE_ID_STORED_N,
-			 MLOG_4BYTES, &mtr);
-	mtr_commit(&mtr);
+	mtr.write<4>(*trx_sys_block,
+		     TRX_SYS_DOUBLEWRITE + TRX_SYS_DOUBLEWRITE_SPACE_ID_STORED
+		     + trx_sys_block->frame,
+		     TRX_SYS_DOUBLEWRITE_SPACE_ID_STORED_N);
+	mtr.commit();
 
 	/* Flush the modified pages to disk and make a checkpoint */
 	log_make_checkpoint();
@@ -360,23 +352,18 @@ buf_dblwr_init_or_load_pages(
 	ulint		space_id;
 	byte*		read_buf;
 	byte*		doublewrite;
-	byte*		unaligned_read_buf;
 	ibool		reset_space_ids = FALSE;
 	recv_dblwr_t&	recv_dblwr = recv_sys.dblwr;
 
 	/* We do the file i/o past the buffer pool */
-
-	unaligned_read_buf = static_cast<byte*>(
-		ut_malloc_nokey(3U << srv_page_size_shift));
-
 	read_buf = static_cast<byte*>(
-		ut_align(unaligned_read_buf, srv_page_size));
+		aligned_malloc(2 * srv_page_size, srv_page_size));
 
 	/* Read the trx sys header to check if we are using the doublewrite
 	buffer */
 	dberr_t		err;
 
-	IORequest	read_request(IORequest::READ);
+	IORequest       read_request(IORequest::READ);
 
 	err = os_file_read(
 		read_request,
@@ -387,9 +374,8 @@ buf_dblwr_init_or_load_pages(
 
 		ib::error()
 			<< "Failed to read the system tablespace header page";
-
-		ut_free(unaligned_read_buf);
-
+func_exit:
+		aligned_free(read_buf);
 		return(err);
 	}
 
@@ -408,8 +394,8 @@ buf_dblwr_init_or_load_pages(
 
 		buf = buf_dblwr->write_buf;
 	} else {
-		ut_free(unaligned_read_buf);
-		return(DB_SUCCESS);
+		err = DB_SUCCESS;
+		goto func_exit;
 	}
 
 	if (mach_read_from_4(doublewrite + TRX_SYS_DOUBLEWRITE_SPACE_ID_STORED)
@@ -437,10 +423,7 @@ buf_dblwr_init_or_load_pages(
 		ib::error()
 			<< "Failed to read the first double write buffer "
 			"extent";
-
-		ut_free(unaligned_read_buf);
-
-		return(err);
+		goto func_exit;
 	}
 
 	err = os_file_read(
@@ -455,10 +438,7 @@ buf_dblwr_init_or_load_pages(
 		ib::error()
 			<< "Failed to read the second double write buffer "
 			"extent";
-
-		ut_free(unaligned_read_buf);
-
-		return(err);
+		goto func_exit;
 	}
 
 	/* Check if any of these pages is half-written in data files, in the
@@ -485,10 +465,8 @@ buf_dblwr_init_or_load_pages(
 					+ i - TRX_SYS_DOUBLEWRITE_BLOCK_SIZE;
 			}
 
-			IORequest	write_request(IORequest::WRITE);
-
 			err = os_file_write(
-				write_request, path, file, page,
+				IORequestWrite, path, file, page,
 				source_page_no << srv_page_size_shift,
 				srv_page_size);
 			if (err != DB_SUCCESS) {
@@ -496,13 +474,9 @@ buf_dblwr_init_or_load_pages(
 				ib::error()
 					<< "Failed to write to the double write"
 					" buffer";
-
-				ut_free(unaligned_read_buf);
-
-				return(err);
+				goto func_exit;
 			}
-
-		} else if (memcmp(field_ref_zero, page + FIL_PAGE_LSN, 8)) {
+		} else if (mach_read_from_8(page + FIL_PAGE_LSN)) {
 			/* Each valid page header must contain
 			a nonzero FIL_PAGE_LSN field. */
 			recv_dblwr.add(page);
@@ -515,9 +489,8 @@ buf_dblwr_init_or_load_pages(
 		os_file_flush(file);
 	}
 
-	ut_free(unaligned_read_buf);
-
-	return(DB_SUCCESS);
+	err = DB_SUCCESS;
+	goto func_exit;
 }
 
 /** Process and remove the double write buffer pages for all tablespaces. */
@@ -526,18 +499,14 @@ buf_dblwr_process()
 {
 	ulint		page_no_dblwr	= 0;
 	byte*		read_buf;
-	byte*		unaligned_read_buf;
 	recv_dblwr_t&	recv_dblwr	= recv_sys.dblwr;
 
 	if (!buf_dblwr) {
 		return;
 	}
 
-	unaligned_read_buf = static_cast<byte*>(
-		ut_malloc_nokey(3U << srv_page_size_shift));
-
 	read_buf = static_cast<byte*>(
-		ut_align(unaligned_read_buf, srv_page_size));
+		aligned_malloc(2 * srv_page_size, srv_page_size));
 	byte* const buf = read_buf + srv_page_size;
 
 	for (recv_dblwr_t::list::iterator i = recv_dblwr.pages.begin();
@@ -693,7 +662,7 @@ bad:
 	recv_dblwr.pages.clear();
 
 	fil_flush_file_spaces(FIL_TYPE_TABLESPACE);
-	ut_free(unaligned_read_buf);
+	aligned_free(read_buf);
 }
 
 /****************************************************************//**
@@ -708,15 +677,9 @@ buf_dblwr_free()
 
 	os_event_destroy(buf_dblwr->b_event);
 	os_event_destroy(buf_dblwr->s_event);
-	ut_free(buf_dblwr->write_buf_unaligned);
-	buf_dblwr->write_buf_unaligned = NULL;
-
+	aligned_free(buf_dblwr->write_buf);
 	ut_free(buf_dblwr->buf_block_arr);
-	buf_dblwr->buf_block_arr = NULL;
-
 	ut_free(buf_dblwr->in_use);
-	buf_dblwr->in_use = NULL;
-
 	mutex_free(&buf_dblwr->mutex);
 	ut_free(buf_dblwr);
 	buf_dblwr = NULL;
@@ -789,29 +752,21 @@ buf_dblwr_update(
 
 #ifdef UNIV_DEBUG
 /** Check the LSN values on the page.
-@param[in]	page	page to check
-@param[in]	s	tablespace */
+@param[in] page  page to check
+@param[in] s     tablespace */
 static void buf_dblwr_check_page_lsn(const page_t* page, const fil_space_t& s)
 {
-	/* Ignore page compressed or encrypted pages */
-	if (s.is_compressed()
-	    || buf_page_get_key_version(page, s.flags)) {
-		return;
-	}
-
-	const unsigned lsn1 = mach_read_from_4(page + FIL_PAGE_LSN + 4),
-		lsn2 = mach_read_from_4(page + srv_page_size
-					- (s.full_crc32()
-					   ? FIL_PAGE_FCRC32_END_LSN
-					   : FIL_PAGE_END_LSN_OLD_CHKSUM - 4));
-	if (UNIV_UNLIKELY(lsn1 != lsn2)) {
-		ib::error() << "The page to be written to "
-			    << s.chain.start->name <<
-			" seems corrupt!"
-			" The low 4 bytes of LSN fields do not match"
-			" (" << lsn1 << " != " << lsn2 << ")!"
-			" Noticed in the buffer pool.";
-	}
+  /* Ignore page compressed or encrypted pages */
+  if (s.is_compressed() || buf_page_get_key_version(page, s.flags))
+    return;
+  const byte* lsn_start= FIL_PAGE_LSN + 4 + page;
+  const byte* lsn_end= page +
+    srv_page_size - (s.full_crc32()
+    ? FIL_PAGE_FCRC32_END_LSN
+    : FIL_PAGE_END_LSN_OLD_CHKSUM - 4);
+  static_assert(FIL_PAGE_FCRC32_END_LSN % 4 == 0, "alignment");
+  static_assert(FIL_PAGE_LSN % 4 == 0, "alignment");
+  ut_ad(!memcmp_aligned<4>(lsn_start, lsn_end, 4));
 }
 
 static void buf_dblwr_check_page_lsn(const buf_page_t& b, const byte* page)
@@ -1130,7 +1085,8 @@ try_again:
 		ut_a(buf_page_get_state(bpage) == BUF_BLOCK_FILE_PAGE);
 
 		UNIV_MEM_ASSERT_RW(frame, srv_page_size);
-		memcpy(p, frame, srv_page_size);
+		memcpy_aligned<OS_FILE_LOG_BLOCK_SIZE>(p, frame,
+						       srv_page_size);
 	}
 
 	buf_dblwr->buf_block_arr[buf_dblwr->first_free] = bpage;
