@@ -1,7 +1,7 @@
 /*****************************************************************************
 
 Copyright (c) 2011, 2018, Oracle and/or its affiliates. All Rights Reserved.
-Copyright (c) 2016, 2020, MariaDB Corporation.
+Copyright (c) 2016, 2019, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -317,11 +317,19 @@ public:
 	/** fts_t destructor. */
 	~fts_t();
 
-	/** Whether the ADDED table record sync-ed after crash recovery */
+	/** Mutex protecting bg_threads* and fts_add_wq. */
+	ib_mutex_t	bg_threads_mutex;
+
+	/** Whether the ADDED table record sync-ed after
+	crash recovery; protected by bg_threads_mutex */
 	unsigned	added_synced:1;
-	/** Whether the table holds dict_sys.mutex */
+	/** Whether the table holds dict_sys.mutex;
+	protected by bg_threads_mutex */
 	unsigned	dict_locked:1;
 
+	/** Number of background threads accessing this table. */
+	ulint		bg_threads;
+
 	/** Work queue for scheduling jobs for the FTS 'Add' thread, or NULL
 	if the thread has not yet been created. Each work item is a
 	fts_trx_doc_ids_t*. */
@@ -341,6 +349,10 @@ public:
 	protected by fts_optimize_wq mutex */
 	bool		in_queue;
 
+	/** Whether the sync message exists in fts_optimize_wq;
+	protected by fts_optimize_wq mutex */
+	bool		sync_message;
+
 	/** Heap for fts_t allocation. */
 	mem_heap_t*	fts_heap;
 };
@@ -584,7 +596,7 @@ doc_id_t
 fts_get_doc_id_from_rec(
 	const rec_t*		rec,
 	const dict_index_t*	index,
-	const rec_offs*		offsets);
+	const offset_t*		offsets);
 
 /** Add new fts doc id to the update vector.
 @param[in]	table		the table that contains the FTS index.
@@ -726,9 +738,12 @@ fts_savepoint_rollback_last_stmt(
 /*=============================*/
 	trx_t*		trx);			/*!< in: transaction */
 
-/** Drop all orphaned FTS auxiliary tables, those that don't have a parent
+/***********************************************************************//**
+Drop all orphaned FTS auxiliary tables, those that don't have a parent
 table or FTS index defined on them. */
-void fts_drop_orphaned_tables();
+void
+fts_drop_orphaned_tables(void);
+/*==========================*/
 
 /** Run SYNC on the table, i.e., write out data from the cache to the
 FTS auxiliary INDEX table and clear the cache at the end.
@@ -762,6 +777,15 @@ fts_init_doc_id(
 /*============*/
 	const dict_table_t*		table);	/*!< in: table */
 
+/* Get parent table name if it's a fts aux table
+@param[in]	aux_table_name	aux table name
+@param[in]	aux_table_len	aux table length
+@return parent table name, or NULL */
+char*
+fts_get_parent_table_name(
+	const char*	aux_table_name,
+	ulint		aux_table_len);
+
 /******************************************************************//**
 compare two character string according to their charset. */
 extern
@@ -856,18 +880,20 @@ fts_valid_stopword_table(
 						name */
 /****************************************************************//**
 This function loads specified stopword into FTS cache
-@return true if success */
-bool
+@return TRUE if success */
+ibool
 fts_load_stopword(
 /*==============*/
 	const dict_table_t*
 			table,			/*!< in: Table with FTS */
 	trx_t*		trx,			/*!< in: Transaction */
+	const char*	global_stopword_table,	/*!< in: Global stopword table
+						name */
 	const char*	session_stopword_table,	/*!< in: Session stopword table
 						name */
-	bool		stopword_is_on,		/*!< in: Whether stopword
+	ibool		stopword_is_on,		/*!< in: Whether stopword
 						option is turned on/off */
-	bool		reload);		/*!< in: Whether it is during
+	ibool		reload);		/*!< in: Whether it is during
 						reload of FTS table */
 
 /****************************************************************//**
@@ -962,20 +988,8 @@ fts_trx_t*
 fts_trx_create(
 	trx_t*  trx);
 
-/** Clear all fts resources when there is no internal DOC_ID
-and there are no new fts index to add.
-@param[in,out]  table   table  where fts is to be freed
-@param[in]      trx     transaction to drop all fts tables */
-void fts_clear_all(dict_table_t *table, trx_t *trx);
-
-/** Check whether the given name is fts auxiliary table
-and fetch the parent table id and index id
-@param[in]	name		table name
-@param[in,out]	table_id	parent table id
-@param[in,out]	index_id	index id
-@return true if it is auxilary table */
-bool fts_check_aux_table(const char *name,
-                         table_id_t *table_id,
-                         index_id_t *index_id);
+/** Sync the table during commit phase
+@param[in]	table	table to be synced */
+void fts_sync_during_ddl(dict_table_t* table);
 
 #endif /*!< fts0fts.h */
