@@ -38,10 +38,10 @@ Created 12/9/1995 Heikki Tuuri
 #include "log0types.h"
 #include "os0event.h"
 #include "os0file.h"
+#include "span.h"
+#include <atomic>
 
-#ifndef UINT32_MAX
-#define UINT32_MAX             (4294967295U)
-#endif
+using st_::span;
 
 /** Maximum number of srv_n_log_files, or innodb_log_files_in_group */
 #define SRV_N_LOG_FILES_MAX 100
@@ -445,15 +445,41 @@ or the MySQL version that created the redo log file. */
 					header */
 #define LOG_FILE_HDR_SIZE	(4 * OS_FILE_LOG_BLOCK_SIZE)
 
-/* As long as fil_io() is used to handle log io, log group max size is limited
-by (maximum page number) * (minimum page size). Page number type is uint32_t.
-Remove this limitation if page number is no longer used for log file io. */
-static const ulonglong log_group_max_size =
-	((ulonglong(UINT32_MAX) + 1) * UNIV_PAGE_SIZE_MIN - 1);
-
 typedef ib_mutex_t	LogSysMutex;
 typedef ib_mutex_t	FlushOrderMutex;
 
+/** RAII wrapper over path and file descriptor. Supposed to be used for log
+files only */
+class log_file_t
+{
+public:
+  log_file_t()= default;
+  log_file_t(std::string path) : m_path{std::move(path)} {}
+
+  log_file_t(const log_file_t &)= delete;
+  log_file_t &operator=(const log_file_t &)= delete;
+
+  log_file_t(log_file_t &&rhs);
+  log_file_t &operator=(log_file_t &&rhs);
+
+  ~log_file_t();
+
+  bool open();
+
+  bool is_opened() const { return m_fd != OS_FILE_CLOSED; }
+  const std::string get_path() const { return m_path; }
+
+  dberr_t rename(std::string new_path);
+  bool close();
+  dberr_t read(os_offset_t offset, span<byte> buf);
+  dberr_t write(os_offset_t offset, span<const byte> buf);
+  bool flush_data_only();
+
+private:
+  pfs_os_file_t m_fd;
+  std::string m_path;
+};
+
 /** Redo log buffer */
 struct log_t{
   /** The original (not version-tagged) InnoDB redo log format */
@@ -534,11 +560,30 @@ struct log_t{
     lsn_t				lsn;
     /** the byte offset of the above lsn */
     lsn_t				lsn_offset;
+
   public:
     /** used only in recovery: recovery scan succeeded up to this
     lsn in this log group */
     lsn_t				scanned_lsn;
 
+    /** file descriptors for all log files */
+    std::vector<log_file_t> files;
+
+    /** opens log files which must be closed prior this call */
+    void open_files(std::vector<std::string> paths);
+    /** reads buffer from log files
+    @param[in]	total_offset	offset in log files treated as a single file
+    @param[in]	buf		buffer where to read */
+    void read(os_offset_t total_offset, span<byte> buf);
+    /** writes buffer to log files
+    @param[in]	total_offset	offset in log files treated as a single file
+    @param[in]	buf		buffer from which to write */
+    void write(os_offset_t total_offset, span<byte> buf);
+    /** flushes OS page cache (excluding metadata!) for all log files */
+    void flush_data_only();
+    /** closes all log files */
+    void close_files();
+
     /** @return whether the redo log is encrypted */
     bool is_encrypted() const { return format & FORMAT_ENCRYPTED; }
     /** @return capacity in bytes */
@@ -571,6 +616,7 @@ struct log_t{
     void close()
     {
       n_files = 0;
+      close_files();
     }
     void set_lsn(lsn_t a_lsn);
     lsn_t get_lsn() const { return lsn; }
@@ -593,6 +639,8 @@ struct log_t{
 	lsn_t		flushed_to_disk_lsn;
 					/*!< how far we have written the log
 					AND flushed to disk */
+	std::atomic<size_t> pending_flushes; /*!< system calls in progress */
+	std::atomic<size_t> flushes;	/*!< system calls counter */
 	ulint		n_pending_flushes;/*!< number of currently
 					pending flushes; protected by
 					log_sys.mutex */
@@ -696,6 +744,16 @@ public:
       : OS_FILE_LOG_BLOCK_SIZE - LOG_BLOCK_CHECKSUM;
   }
 
+  size_t get_pending_flushes() const
+  {
+    return pending_flushes.load(std::memory_order_relaxed);
+  }
+
+  size_t get_flushes() const
+  {
+    return flushes.load(std::memory_order_relaxed);
+  }
+
   /** Initialise the redo log subsystem. */
   void create();
 
