@@ -2,7 +2,7 @@
 
 Copyright (c) 1995, 2017, Oracle and/or its affiliates. All Rights Reserved.
 Copyright (c) 2012, Facebook Inc.
-Copyright (c) 2013, 2019, MariaDB Corporation.
+Copyright (c) 2013, 2020, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -125,29 +125,22 @@ struct mtr_memo_slot_t {
 
 /** Mini-transaction handle and buffer */
 struct mtr_t {
-	mtr_t() : m_state(MTR_STATE_INIT) {}
+  /** Start a mini-transaction. */
+  void start();
 
-	/** Start a mini-transaction. */
-	void start();
+  /** Commit the mini-transaction. */
+  void commit();
 
-	/** Commit the mini-transaction. */
-	void commit();
+  /** Commit a mini-transaction that did not modify any pages,
+  but generated some redo log on a higher level, such as
+  MLOG_FILE_NAME records and an optional MLOG_CHECKPOINT marker.
+  The caller must invoke log_mutex_enter() and log_mutex_exit().
+  This is to be used at log_checkpoint().
+  @param checkpoint_lsn   the log sequence number of a checkpoint, or 0 */
+  void commit_files(lsn_t checkpoint_lsn= 0);
 
-	/** Commit a mini-transaction that did not modify any pages,
-	but generated some redo log on a higher level, such as
-	MLOG_FILE_NAME records and a MLOG_CHECKPOINT marker.
-	The caller must invoke log_mutex_enter() and log_mutex_exit().
-	This is to be used at log_checkpoint().
-	@param[in]	checkpoint_lsn		the LSN of the log checkpoint
-	@param[in]	write_mlog_checkpoint	Write MLOG_CHECKPOINT marker
-						if it is enabled. */
-	void commit_checkpoint(
-		lsn_t	checkpoint_lsn,
-		bool	write_mlog_checkpoint);
-
-	/** Return current size of the buffer.
-	@return	savepoint */
-	ulint get_savepoint() const {ut_ad(is_active()); return m_memo.size();}
+  /** @return mini-transaction savepoint (current size of m_memo) */
+  ulint get_savepoint() const { ut_ad(is_active()); return m_memo.size(); }
 
 	/** Release the (index tree) s-latch stored in an mtr memo after a
 	savepoint.
@@ -168,10 +161,12 @@ struct mtr_t {
 	/** X-latch a not yet latched block after a savepoint. */
 	inline void x_latch_at_savepoint(ulint savepoint, buf_block_t*	block);
 
-	/** Get the logging mode.
-	@return	logging mode */
-	inline mtr_log_t get_log_mode() const
-		MY_ATTRIBUTE((warn_unused_result));
+  /** @return the logging mode */
+  mtr_log_t get_log_mode() const
+  {
+    static_assert(MTR_LOG_ALL == 0, "efficiency");
+    return static_cast<mtr_log_t>(m_log_mode);
+  }
 
 	/** Change the logging mode.
 	@param mode	 logging mode
@@ -198,7 +193,7 @@ struct mtr_t {
 	fil_space_t* set_named_space_id(ulint space_id)
 	{
 		ut_ad(!m_user_space_id);
-		ut_d(m_user_space_id = space_id);
+		ut_d(m_user_space_id = static_cast<uint32_t>(space_id));
 		if (!space_id) {
 			return fil_system.sys_space;
 		} else {
@@ -216,7 +211,7 @@ struct mtr_t {
 	void set_named_space(fil_space_t* space)
 	{
 		ut_ad(!m_user_space_id);
-		ut_d(m_user_space_id = space->id);
+		ut_d(m_user_space_id = static_cast<uint32_t>(space->id));
 		if (space->id) {
 			m_user_space = space;
 		}
@@ -309,47 +304,26 @@ struct mtr_t {
 	@param[in]	type	object type: MTR_MEMO_PAGE_X_FIX, ... */
 	void release_page(const void* ptr, mtr_memo_type_t type);
 
-	/** Note that the mini-transaction has modified data. */
-	void set_modified() { m_modifications = true; }
+  /** Note that the mini-transaction has modified data. */
+  void set_modified() { m_modifications = true; }
 
-	/** Set the state to not-modified. This will not log the
-	changes.  This is only used during redo log apply, to avoid
-	logging the changes. */
-	void discard_modifications() { m_modifications = false; }
+  /** Set the state to not-modified. This will not log the changes.
+  This is only used during redo log apply, to avoid logging the changes. */
+  void discard_modifications() { m_modifications = false; }
 
-	/** Get the LSN of commit().
-	@return the commit LSN
-	@retval 0 if the transaction only modified temporary tablespaces */
-	lsn_t commit_lsn() const
-	{
-		ut_ad(has_committed());
-		return(m_commit_lsn);
-	}
+  /** Get the LSN of commit().
+  @return the commit LSN
+  @retval 0 if the transaction only modified temporary tablespaces */
+  lsn_t commit_lsn() const { ut_ad(has_committed()); return m_commit_lsn; }
 
-	/** Note that we are inside the change buffer code. */
-	void enter_ibuf() { m_inside_ibuf = true; }
+  /** Note that we are inside the change buffer code. */
+  void enter_ibuf() { m_inside_ibuf= true; }
 
-	/** Note that we have exited from the change buffer code. */
-	void exit_ibuf() { m_inside_ibuf = false; }
+  /** Note that we have exited from the change buffer code. */
+  void exit_ibuf() { m_inside_ibuf= false; }
 
-	/** @return true if we are inside the change buffer code */
-	bool is_inside_ibuf() const { return m_inside_ibuf; }
-
-	/*
-	@return true if the mini-transaction is active */
-	bool is_active() const { return m_state == MTR_STATE_ACTIVE; }
-
-	/** Get flush observer
-	@return flush observer */
-	FlushObserver* get_flush_observer() const { return m_flush_observer; }
-
-	/** Set flush observer
-	@param[in]	observer	flush observer */
-	void set_flush_observer(FlushObserver*	observer)
-	{
-		ut_ad(observer == NULL || m_log_mode == MTR_LOG_NO_REDO);
-		m_flush_observer = observer;
-	}
+  /** @return true if we are inside the change buffer code */
+  bool is_inside_ibuf() const { return m_inside_ibuf; }
 
 #ifdef UNIV_DEBUG
 	/** Check if memo contains the given item.
@@ -387,9 +361,6 @@ struct mtr_t {
 	/** Print info of an mtr handle. */
 	void print() const;
 
-	/** @return true if the mini-transaction has committed */
-	bool has_committed() const { return m_state == MTR_STATE_COMMITTED; }
-
 	/** @return true if mini-transaction contains modifications. */
 	bool has_modifications() const { return m_modifications; }
 
@@ -425,57 +396,127 @@ struct mtr_t {
 	static inline bool is_block_dirtied(const buf_block_t* block)
 		MY_ATTRIBUTE((warn_unused_result));
 
+  /** Write request types */
+  enum write_type
+  {
+    /** the page is guaranteed to always change */
+    NORMAL= 0,
+    /** optional: the page contents might not change */
+    OPT,
+    /** force a write, even if the page contents is not changing */
+    FORCED
+  };
+
+  /** Write 1, 2, 4, or 8 bytes to a file page.
+  @param[in]      block   file page
+  @param[in,out]  ptr     pointer in file page
+  @param[in]      val     value to write
+  @tparam l       number of bytes to write
+  @tparam w       write request type
+  @tparam V       type of val */
+  template<unsigned l,write_type w= NORMAL,typename V>
+  inline void write(const buf_block_t &block, byte *ptr, V val)
+    MY_ATTRIBUTE((nonnull));
+
+  /** Log a write of a byte string to a page.
+  @param[in]      b       buffer page
+  @param[in]      ofs     byte offset from b->frame
+  @param[in]      len     length of the data to write */
+  void memcpy(const buf_block_t &b, ulint ofs, ulint len);
+
+  /** Write a byte string to a page.
+  @param[in,out]  b       buffer page
+  @param[in]      offset  byte offset from b->frame
+  @param[in]      str     the data to write
+  @param[in]      len     length of the data to write */
+  inline void memcpy(buf_block_t *b, ulint offset, const void *str, ulint len);
+
+  /** Initialize a string of bytes.
+  @param[in,out]        b       buffer page
+  @param[in]            ofs     byte offset from b->frame
+  @param[in]            len     length of the data to write
+  @param[in]            val     the data byte to write */
+  void memset(const buf_block_t* b, ulint ofs, ulint len, byte val);
+
 private:
-	/** Prepare to write the mini-transaction log to the redo log buffer.
-	@return number of bytes to write in finish_write() */
-	inline ulint prepare_write();
+  /**
+  Write a log record for writing 1, 2, or 4 bytes.
+  @param[in]      block   file page
+  @param[in,out]  ptr     pointer in file page
+  @param[in]      l       number of bytes to write
+  @param[in,out]  log_ptr log record buffer
+  @param[in]      val     value to write */
+  void log_write(const buf_block_t &block, byte *ptr, mlog_id_t l,
+                 byte *log_ptr, uint32_t val)
+    MY_ATTRIBUTE((nonnull));
+  /**
+  Write a log record for writing 8 bytes.
+  @param[in]      block   file page
+  @param[in,out]  ptr     pointer in file page
+  @param[in]      l       number of bytes to write (8)
+  @param[in,out]  log_ptr log record buffer
+  @param[in]      val     value to write */
+  void log_write(const buf_block_t &block, byte *ptr, mlog_id_t l,
+                 byte *log_ptr, uint64_t val)
+    MY_ATTRIBUTE((nonnull));
 
-	/** Append the redo log records to the redo log buffer.
-	@param[in]	len	number of bytes to write
-	@return start_lsn */
-	inline lsn_t finish_write(ulint len);
+  /** Prepare to write the mini-transaction log to the redo log buffer.
+  @return number of bytes to write in finish_write() */
+  inline ulint prepare_write();
 
-	/** Release the resources */
-	inline void release_resources();
+  /** Append the redo log records to the redo log buffer.
+  @param len   number of bytes to write
+  @return start_lsn */
+  inline lsn_t finish_write(ulint len);
 
-	/** memo stack for locks etc. */
-	mtr_buf_t	m_memo;
+  /** Release the resources */
+  inline void release_resources();
 
-	/** mini-transaction log */
-	mtr_buf_t	m_log;
-
-	/** true if mtr has made at least one buffer pool page dirty */
-	bool		m_made_dirty;
-
-	/** true if inside ibuf changes */
-	bool		m_inside_ibuf;
-
-	/** true if the mini-transaction modified buffer pool pages */
-	bool		m_modifications;
-
-	/** Count of how many page initial log records have been
-	written to the mtr log */
-	ib_uint32_t	m_n_log_recs;
-
-	/** specifies which operations should be logged; default
-	value MTR_LOG_ALL */
-	mtr_log_t	m_log_mode;
 #ifdef UNIV_DEBUG
-	/** Persistent user tablespace associated with the
-	mini-transaction, or 0 (TRX_SYS_SPACE) if none yet */
-	ulint		m_user_space_id;
+public:
+  /** @return whether the mini-transaction is active */
+  bool is_active() const { ut_ad(!m_commit || m_start); return m_start; }
+  /** @return whether the mini-transaction has been committed */
+  bool has_committed() const { ut_ad(!m_commit || m_start); return m_commit; }
+private:
+  /** whether start() has been called */
+  bool m_start= false;
+  /** whether commit() has been called */
+  bool m_commit= false;
+#endif
+
+  /** specifies which operations should be logged; default MTR_LOG_ALL */
+  uint16_t m_log_mode:2;
+
+  /** whether at least one buffer pool page was written to */
+  uint16_t m_modifications:1;
+
+  /** whether at least one previously clean buffer pool page was written to */
+  uint16_t m_made_dirty:1;
+
+  /** whether change buffer is latched; only needed in non-debug builds
+  to suppress some read-ahead operations, @see ibuf_inside() */
+  uint16_t m_inside_ibuf:1;
+
+  /** number of m_log records */
+  uint16_t m_n_log_recs:11;
+#ifdef UNIV_DEBUG
+  /** Persistent user tablespace associated with the
+  mini-transaction, or 0 (TRX_SYS_SPACE) if none yet */
+  uint32_t m_user_space_id;
 #endif /* UNIV_DEBUG */
-	/** User tablespace that is being modified by the mini-transaction */
-	fil_space_t*	m_user_space;
 
-	/** State of the transaction */
-	mtr_state_t	m_state;
+  /** acquired dict_index_t::lock, fil_space_t::latch, buf_block_t */
+  mtr_buf_t m_memo;
 
-	/** Flush Observer */
-	FlushObserver*	m_flush_observer;
+  /** mini-transaction log */
+  mtr_buf_t m_log;
 
-	/** LSN at commit time */
-	lsn_t		m_commit_lsn;
+  /** user tablespace that is being modified by the mini-transaction */
+  fil_space_t* m_user_space;
+
+  /** LSN at commit time */
+  lsn_t m_commit_lsn;
 };
 
 #include "mtr0mtr.ic"
