@@ -2,7 +2,7 @@
 
 Copyright (c) 1995, 2017, Oracle and/or its affiliates. All Rights Reserved.
 Copyright (c) 2009, Percona Inc.
-Copyright (c) 2013, 2019, MariaDB Corporation.
+Copyright (c) 2013, 2020, MariaDB Corporation.
 
 Portions of this file contain modifications contributed and copyrighted
 by Percona Inc.. Those modifications are
@@ -187,10 +187,6 @@ static const ulint OS_FILE_ERROR_MAX = 200;
 /** No transformations during read/write, write as is. */
 #define IORequestRead		IORequest(IORequest::READ)
 #define IORequestWrite		IORequest(IORequest::WRITE)
-#define IORequestLogRead	IORequest(IORequest::LOG | IORequest::READ)
-#define IORequestLogWrite	IORequest(IORequest::LOG | IORequest::WRITE)
-
-
 
 /**
 The IO Context that is passed down to the low level IO code */
@@ -209,9 +205,6 @@ public:
 		/** Data file */
 		DATA_FILE = 8,
 
-		/** Log file request*/
-		LOG = 16,
-
 		/** Disable partial read warnings */
 		DISABLE_PARTIAL_IO_WARNINGS = 32,
 
@@ -279,13 +272,6 @@ public:
 		return((m_type & WRITE) == WRITE);
 	}
 
-	/** @return true if it is a redo log write */
-	bool is_log() const
-		MY_ATTRIBUTE((warn_unused_result))
-	{
-		return((m_type & LOG) == LOG);
-	}
-
 	/** Clear the punch hole flag */
 	void clear_punch_hole()
 	{
@@ -417,9 +403,6 @@ static const ulint OS_AIO_NORMAL = 21;
 /**  Asynchronous i/o for ibuf pages or ibuf bitmap pages */
 static const ulint OS_AIO_IBUF = 22;
 
-/** Asynchronous i/o for the log */
-static const ulint OS_AIO_LOG = 23;
-
 /**Calling thread will wait for the i/o to complete,
 and perform IO completion routine itself;
 can be used for any pages, ibuf or non-ibuf.  This is used to save
@@ -787,6 +770,9 @@ The wrapper functions have the prefix of "innodb_". */
 # define os_file_flush(file)					\
 	pfs_os_file_flush_func(file, __FILE__, __LINE__)
 
+#define os_file_flush_data(file)                                              \
+  pfs_os_file_flush_data_func(file, __FILE__, __LINE__)
+
 # define os_file_rename(key, oldpath, newpath)				\
 	pfs_os_file_rename_func(key, oldpath, newpath, __FILE__, __LINE__)
 
@@ -1032,6 +1018,18 @@ pfs_os_file_flush_func(
 	const char*	src_file,
 	uint		src_line);
 
+/** NOTE! Please use the corresponding macro os_file_flush_data(), not directly
+this function!
+This is the performance schema instrumented wrapper function for
+os_file_flush_data() which flushes only(!) data (excluding metadata) from OS
+page cache of a given file to the disk.
+@param[in]	file		Open file handle
+@param[in]	src_file	file name where func invoked
+@param[in]	src_line	line where the func invoked
+@return true if success */
+bool pfs_os_file_flush_data_func(pfs_os_file_t file, const char *src_file,
+                                 uint src_line);
+
 /** NOTE! Please use the corresponding macro os_file_rename(), not directly
 this function!
 This is the performance schema instrumented wrapper function for
@@ -1126,6 +1124,8 @@ to original un-instrumented file I/O APIs */
 
 # define os_file_flush(file)	os_file_flush_func(file)
 
+#define os_file_flush_data(file) os_file_flush_data_func(file)
+
 # define os_file_rename(key, oldpath, newpath)				\
 	os_file_rename_func(oldpath, newpath)
 
@@ -1205,6 +1205,14 @@ bool
 os_file_flush_func(
 	os_file_t	file);
 
+/** NOTE! Use the corresponding macro os_file_flush_data(), not directly this
+function!
+Flushes only(!) data (excluding metadata) from OS page cache of a given file to
+the disk.
+@param[in]	file		handle to a file
+@return true if success */
+bool os_file_flush_data_func(os_file_t file);
+
 /** Retrieves the last error number if an error occurs in a file io function.
 The number should be retrieved before any other OS calls (because they may
 overwrite the error number). If the number is not known to this program,
