@@ -3,7 +3,7 @@
 Copyright (c) 1995, 2017, Oracle and/or its affiliates. All rights reserved.
 Copyright (c) 2008, 2009, Google Inc.
 Copyright (c) 2009, Percona Inc.
-Copyright (c) 2013, 2019, MariaDB Corporation.
+Copyright (c) 2013, 2020, MariaDB Corporation.
 
 Portions of this file contain modifications contributed and copyrighted by
 Google, Inc. Those modifications are gratefully acknowledged and are described
@@ -321,8 +321,9 @@ extern my_bool	srv_undo_log_truncate;
 /* Enables or disables this prefix optimization.  Disabled by default. */
 extern my_bool	srv_prefix_index_cluster_optimization;
 
-/** Default size of UNDO tablespace while it is created new. */
-extern const ulint	SRV_UNDO_TABLESPACE_SIZE_IN_PAGES;
+/** Default size of UNDO tablespace (10MiB for innodb_page_size=16k) */
+constexpr ulint SRV_UNDO_TABLESPACE_SIZE_IN_PAGES= (10U << 20) /
+  UNIV_PAGE_SIZE_DEF;
 
 extern char*	srv_log_group_home_dir;
 
@@ -353,13 +354,8 @@ extern ulint		srv_buf_pool_size;
 extern const ulint	srv_buf_pool_min_size;
 /** Default pool size in bytes */
 extern const ulint	srv_buf_pool_def_size;
-/** Requested buffer pool chunk size. Each buffer pool instance consists
-of one or more chunks. */
+/** Requested buffer pool chunk size */
 extern ulong		srv_buf_pool_chunk_unit;
-/** Requested number of buffer pool instances */
-extern ulong		srv_buf_pool_instances;
-/** Default number of buffer pool instances */
-extern const ulong	srv_buf_pool_instances_default;
 /** Number of locks to protect buf_pool->page_hash */
 extern ulong	srv_n_page_hash_locks;
 /** Scan depth for LRU flush batch i.e.: number of blocks scanned*/
@@ -421,8 +417,6 @@ extern ulong	srv_innodb_stats_method;
 
 extern ulint	srv_max_n_open_files;
 
-extern ulong	srv_n_page_cleaners;
-
 extern double	srv_max_dirty_pages_pct;
 extern double	srv_max_dirty_pages_pct_lwm;
 
@@ -597,10 +591,6 @@ do {								\
 everything after flush log_make_checkpoint(). */
 extern PSI_stage_info	srv_stage_alter_table_end;
 
-/** Performance schema stage event for monitoring ALTER TABLE progress
-log_make_checkpoint(). */
-extern PSI_stage_info	srv_stage_alter_table_flush;
-
 /** Performance schema stage event for monitoring ALTER TABLE progress
 row_merge_insert_index_tuples(). */
 extern PSI_stage_info	srv_stage_alter_table_insert;
@@ -817,9 +807,6 @@ void srv_error_monitor_task(void*);
 ulint srv_get_task_queue_length();
 #endif
 
-/** Wakeup the purge threads. */
-void srv_purge_wakeup();
-
 /** Shut down the purge threads. */
 void srv_purge_shutdown();
 
@@ -888,9 +875,9 @@ struct export_var_t{
 #endif
 	ulint innodb_mem_dictionary;
 	lsn_t innodb_os_log_written;		/*!< srv_os_log_written */
-	ulint innodb_os_log_fsyncs;		/*!< fil_n_log_flushes */
+	ulint innodb_os_log_fsyncs;		/*!< n_log_flushes */
 	ulint innodb_os_log_pending_writes;	/*!< srv_os_log_pending_writes */
-	ulint innodb_os_log_pending_fsyncs;	/*!< fil_n_pending_log_flushes */
+	ulint innodb_os_log_pending_fsyncs;	/*!< n_pending_log_flushes */
 	ulint innodb_pages_created;		/*!< buf_pool->stat.n_pages_created */
 	ulint innodb_pages_read;		/*!< buf_pool->stat.n_pages_read*/
 	ulint innodb_pages_written;		/*!< buf_pool->stat.n_pages_written */
