@@ -2,7 +2,7 @@
 
 Copyright (c) 1995, 2017, Oracle and/or its affiliates. All Rights Reserved.
 Copyright (c) 2009, Google Inc.
-Copyright (c) 2014, 2019, MariaDB Corporation.
+Copyright (c) 2014, 2020, MariaDB Corporation.
 
 Portions of this file contain modifications contributed and copyrighted by
 Google, Inc. Those modifications are gratefully acknowledged and are described
@@ -62,7 +62,7 @@ General philosophy of InnoDB redo-logs:
 through mtr, which in mtr_commit() writes log records
 to the InnoDB redo log.
 
-2) Normally these changes are performed using a mlog_write_ulint()
+2) Normally these changes are performed using a mlog_write()
 or similar function.
 
 3) In some page level operations only a code number of a
@@ -181,7 +181,8 @@ void log_buffer_extend(ulong len)
 	srv_log_buffer_size = new_buf_size;
 	log_sys.buf = new_buf;
 	log_sys.first_in_use = true;
-	memcpy(log_sys.buf, old_buf_begin, log_sys.buf_free);
+	memcpy_aligned<OS_FILE_LOG_BLOCK_SIZE>(log_sys.buf, old_buf_begin,
+					       log_sys.buf_free);
 
 	log_sys.max_buf_free = new_buf_size / LOG_BUF_FLUSH_RATIO
 		- LOG_BUF_FLUSH_MARGIN;
@@ -586,6 +587,127 @@ void log_t::create()
   }
 }
 
+log_file_t::log_file_t(log_file_t &&rhs)
+{
+  m_fd= std::move(rhs.m_fd);
+  rhs.m_fd= OS_FILE_CLOSED;
+  m_path= std::move(rhs.m_path);
+}
+log_file_t &log_file_t::operator=(log_file_t &&rhs)
+{
+  std::swap(m_fd, rhs.m_fd);
+  std::swap(m_path, rhs.m_path);
+  return *this;
+}
+
+log_file_t::~log_file_t()
+{
+  if (is_opened())
+    os_file_close(m_fd);
+}
+
+bool log_file_t::open()
+{
+  ut_a(!is_opened());
+
+  bool success;
+  m_fd= os_file_create(innodb_log_file_key, m_path.c_str(),
+                       OS_FILE_OPEN | OS_FILE_ON_ERROR_NO_EXIT, OS_FILE_NORMAL,
+                       OS_LOG_FILE, srv_read_only_mode, &success);
+  if (!success)
+    m_fd= OS_FILE_CLOSED;
+
+  return success;
+}
+
+dberr_t log_file_t::rename(std::string new_path)
+{
+  if (!os_file_rename(innodb_log_file_key, m_path.c_str(),
+                      new_path.c_str())) {
+    return DB_ERROR;
+  }
+  m_path = std::move(new_path);
+  return DB_SUCCESS;
+}
+
+bool log_file_t::close()
+{
+  ut_a(is_opened());
+  bool result= os_file_close(m_fd);
+  m_fd= OS_FILE_CLOSED;
+  return result;
+}
+
+dberr_t log_file_t::read(os_offset_t offset, span<byte> buf)
+{
+  ut_ad(is_opened());
+  return os_file_read(IORequestRead, m_fd, buf.data(), offset, buf.size());
+}
+
+dberr_t log_file_t::write(os_offset_t offset, span<const byte> buf)
+{
+  ut_ad(is_opened());
+  return os_file_write(IORequestWrite, m_path.c_str(), m_fd, buf.data(),
+                       offset, buf.size());
+}
+
+bool log_file_t::flush_data_only()
+{
+  ut_ad(is_opened());
+  return os_file_flush_data(m_fd);
+}
+
+void log_t::files::open_files(std::vector<std::string> paths)
+{
+  files.clear();
+  files.reserve(paths.size());
+  for (auto &&path : paths)
+  {
+    files.push_back(std::move(path));
+    if (!files.back().open())
+      ib::fatal() << "create(" << files.back().get_path() << ") failed";
+  }
+}
+
+void log_t::files::read(os_offset_t total_offset, span<byte> buf)
+{
+  auto &file= files[static_cast<size_t>(total_offset / file_size)];
+  const os_offset_t offset= total_offset % file_size;
+
+  if (const dberr_t err= file.read(offset, buf))
+    ib::fatal() << "read(" << file.get_path() << ") returned " << err;
+}
+
+void log_t::files::write(os_offset_t total_offset, span<byte> buf)
+{
+  auto &file= files[static_cast<size_t>(total_offset / file_size)];
+  const os_offset_t offset= total_offset % file_size;
+
+  if (const dberr_t err= file.write(offset, buf))
+    ib::fatal() << "write(" << file.get_path() << ") returned " << err;
+}
+
+void log_t::files::flush_data_only()
+{
+  log_sys.pending_flushes.fetch_add(1, std::memory_order_acquire);
+  for (auto &file : files)
+  {
+    if (!file.flush_data_only())
+      ib::fatal() << "flush_data_only(" << file.get_path() << ") failed";
+  }
+  log_sys.pending_flushes.fetch_sub(1, std::memory_order_release);
+  log_sys.flushes.fetch_add(1, std::memory_order_release);
+}
+
+void log_t::files::close_files()
+{
+  for (auto &file : files)
+  {
+    if (file.is_opened() && !file.close())
+      ib::fatal() << "close(" << file.get_path() << ") failed";
+  }
+}
+
 /** Initialize the redo log.
 @param[in]	n_files		number of files */
 void log_t::files::create(ulint n_files)
@@ -649,13 +771,7 @@ log_file_header_flush(
 
 	srv_stats.os_log_pending_writes.inc();
 
-	const ulint	page_no = ulint(dest_offset >> srv_page_size_shift);
-
-	fil_io(IORequestLogWrite, true,
-	       page_id_t(SRV_LOG_SPACE_FIRST_ID, page_no),
-	       0,
-	       ulint(dest_offset & (srv_page_size - 1)),
-	       OS_FILE_LOG_BLOCK_SIZE, buf, NULL);
+	log_sys.log.write(static_cast<size_t>(dest_offset), buf);
 
 	srv_stats.os_log_pending_writes.dec();
 }
@@ -756,12 +872,7 @@ loop:
 
 	ut_a((next_offset >> srv_page_size_shift) <= ULINT_MAX);
 
-	const ulint	page_no = ulint(next_offset >> srv_page_size_shift);
-
-	fil_io(IORequestLogWrite, true,
-	       page_id_t(SRV_LOG_SPACE_FIRST_ID, page_no),
-	       0,
-	       ulint(next_offset & (srv_page_size - 1)), write_len, buf, NULL);
+	log_sys.log.write(static_cast<size_t>(next_offset), {buf, write_len});
 
 	srv_stats.os_log_pending_writes.dec();
 
@@ -789,17 +900,10 @@ log_write_flush_to_disk_low()
 	calling os_event_set()! */
 	ut_a(log_sys.n_pending_flushes == 1); /* No other threads here */
 
-	bool	do_flush = srv_file_flush_method != SRV_O_DSYNC;
-
-	if (do_flush) {
-		fil_flush(SRV_LOG_SPACE_FIRST_ID);
-	}
-
+	log_sys.log.flush_data_only();
 
 	log_mutex_enter();
-	if (do_flush) {
-		log_sys.flushed_to_disk_lsn = log_sys.current_flush_lsn;
-	}
+	log_sys.flushed_to_disk_lsn = log_sys.current_flush_lsn;
 
 	log_sys.n_pending_flushes--;
 
@@ -822,19 +926,20 @@ log_buffer_switch()
 
 	if (log_sys.first_in_use) {
 		log_sys.first_in_use = false;
-		ut_ad(log_sys.buf == ut_align(log_sys.buf,
-					       OS_FILE_LOG_BLOCK_SIZE));
+		ut_ad(log_sys.buf == ut_align_down(log_sys.buf,
+						   OS_FILE_LOG_BLOCK_SIZE));
 		log_sys.buf += srv_log_buffer_size;
 	} else {
 		log_sys.first_in_use = true;
 		log_sys.buf -= srv_log_buffer_size;
-		ut_ad(log_sys.buf == ut_align(log_sys.buf,
-					       OS_FILE_LOG_BLOCK_SIZE));
+		ut_ad(log_sys.buf == ut_align_down(log_sys.buf,
+						   OS_FILE_LOG_BLOCK_SIZE));
 	}
 
 	/* Copy the last block to new buf */
-	memcpy(log_sys.buf, old_buf + area_end - OS_FILE_LOG_BLOCK_SIZE,
-	       OS_FILE_LOG_BLOCK_SIZE);
+	memcpy_aligned<OS_FILE_LOG_BLOCK_SIZE>(
+		log_sys.buf, old_buf + area_end - OS_FILE_LOG_BLOCK_SIZE,
+		OS_FILE_LOG_BLOCK_SIZE);
 
 	log_sys.buf_free %= OS_FILE_LOG_BLOCK_SIZE;
 	log_sys.buf_next_to_write = log_sys.buf_free;
@@ -1018,13 +1123,6 @@ loop:
 	srv_stats.log_padded.add(pad_size);
 	log_sys.write_lsn = write_lsn;
 
-
-	if (srv_file_flush_method == SRV_O_DSYNC) {
-		/* O_SYNC means the OS did not buffer the log file at all:
-		so we have also flushed to disk what we have written */
-		log_sys.flushed_to_disk_lsn = log_sys.write_lsn;
-	}
-
 	log_write_mutex_exit();
 
 	if (flush_to_disk) {
@@ -1133,7 +1231,7 @@ static bool log_preflush_pool_modified_pages(lsn_t new_oldest)
 
 		success = buf_flush_lists(ULINT_MAX, new_oldest, &n_pages);
 
-		buf_flush_wait_batch_end(NULL, BUF_FLUSH_LIST);
+		buf_flush_wait_batch_end(BUF_FLUSH_LIST);
 
 		if (!success) {
 			MONITOR_INC(MONITOR_FLUSH_SYNC_WAITS);
@@ -1170,11 +1268,8 @@ void log_header_read(ulint header)
 
 	MONITOR_INC(MONITOR_LOG_IO);
 
-	fil_io(IORequestLogRead, true,
-	       page_id_t(SRV_LOG_SPACE_FIRST_ID,
-			 header >> srv_page_size_shift),
-	       0, header & (srv_page_size - 1),
-	       OS_FILE_LOG_BLOCK_SIZE, log_sys.checkpoint_buf, NULL);
+	log_sys.log.read(header,
+			 {log_sys.checkpoint_buf, OS_FILE_LOG_BLOCK_SIZE});
 }
 
 /** Write checkpoint info to the log header and invoke log_mutex_exit().
@@ -1228,21 +1323,11 @@ void log_write_checkpoint_info(lsn_t end_lsn)
 	/* Note: We alternate the physical place of the checkpoint info.
 	See the (next_checkpoint_no & 1) below. */
 
-	fil_io(IORequestLogWrite, true,
-	       page_id_t(SRV_LOG_SPACE_FIRST_ID, 0),
-	       0,
-	       (log_sys.next_checkpoint_no & 1)
-	       ? LOG_CHECKPOINT_2 : LOG_CHECKPOINT_1,
-	       OS_FILE_LOG_BLOCK_SIZE,
-	       buf, nullptr);
+	log_sys.log.write((log_sys.next_checkpoint_no & 1) ? LOG_CHECKPOINT_2
+							   : LOG_CHECKPOINT_1,
+			  {buf, OS_FILE_LOG_BLOCK_SIZE});
 
-	switch (srv_flush_t(srv_file_flush_method)) {
-	case SRV_O_DSYNC:
-	case SRV_NOSYNC:
-		break;
-	default:
-		fil_flush(SRV_LOG_SPACE_FIRST_ID);
-	}
+	log_sys.log.flush_data_only();
 
 	log_mutex_enter();
 
@@ -1546,7 +1631,7 @@ loop:
 	if (!srv_read_only_mode) {
 		if (recv_sys.flush_start) {
 			/* This is in case recv_writer_thread was never
-			started, or buf_flush_page_cleaner_coordinator
+			started, or buf_flush_page_cleaner
 			failed to notice its termination. */
 			os_event_set(recv_sys.flush_start);
 		}
@@ -1660,7 +1745,7 @@ wait_suspend_loop:
 
 	ut_ad(!log_scrub_thread_active);
 
-	if (!buf_pool_ptr) {
+	if (!buf_pool) {
 		ut_ad(!srv_was_started);
 	} else if (ulint pending_io = buf_pool_check_no_pending_io()) {
 		if (srv_print_verbose_log && count > 600) {
@@ -1720,7 +1805,7 @@ wait_suspend_loop:
 
 		/* Ensure that all buffered changes are written to the
 		redo log before fil_close_all_files(). */
-		fil_flush_file_spaces(FIL_TYPE_LOG);
+		log_sys.log.flush_data_only();
 	} else {
 		lsn = srv_start_lsn;
 	}
@@ -1732,7 +1817,7 @@ wait_suspend_loop:
 
 	service_manager_extend_timeout(INNODB_EXTEND_TIMEOUT_INTERVAL,
 				       "Free innodb buffer pool");
-	buf_all_freed();
+	ut_d(buf_assert_all_freed());
 
 	ut_a(lsn == log_sys.lsn
 	     || srv_force_recovery == SRV_FORCE_NO_LOG_REDO);
