@@ -1,7 +1,7 @@
 /*****************************************************************************
 
 Copyright (c) 1995, 2016, Oracle and/or its affiliates. All Rights Reserved.
-Copyright (c) 2017, 2019, MariaDB Corporation.
+Copyright (c) 2017, 2020, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -50,38 +50,7 @@ mlog_catenate_string(
 }
 
 /********************************************************//**
-Writes the initial part of a log record consisting of one-byte item
-type and four-byte space and page numbers. Also pushes info
-to the mtr memo that a buffer page has been modified. */
-void
-mlog_write_initial_log_record(
-/*==========================*/
-	const byte*	ptr,	/*!< in: pointer to (inside) a buffer
-				frame holding the file page where
-				modification is made */
-	mlog_id_t	type,	/*!< in: log item type: MLOG_1BYTE, ... */
-	mtr_t*		mtr)	/*!< in: mini-transaction handle */
-{
-	byte*	log_ptr;
-
-	ut_ad(type <= MLOG_BIGGEST_TYPE || EXTRA_CHECK_MLOG_NUMBER(type));
-	ut_ad(type > MLOG_8BYTES);
-
-	log_ptr = mlog_open(mtr, 11);
-
-	/* If no logging is requested, we may return now */
-	if (log_ptr == NULL) {
-
-		return;
-	}
-
-	log_ptr = mlog_write_initial_log_record_fast(ptr, type, log_ptr, mtr);
-
-	mlog_close(mtr, log_ptr);
-}
-
-/********************************************************//**
-Parses an initial log record written by mlog_write_initial_log_record.
+Parses an initial log record written by mlog_write_initial_log_record_low().
 @return parsed record end, NULL if not a complete record */
 const byte*
 mlog_parse_initial_log_record(
@@ -121,7 +90,7 @@ mlog_parse_initial_log_record(
 }
 
 /********************************************************//**
-Parses a log record written by mlog_write_ulint, mlog_write_ull, mlog_memset.
+Parses a log record written by mtr_t::write(), mtr_t::memset().
 @return parsed record end, NULL if not a complete record or a corrupt record */
 const byte*
 mlog_parse_nbytes(
@@ -244,139 +213,91 @@ mlog_parse_nbytes(
 	return const_cast<byte*>(ptr);
 }
 
-/********************************************************//**
-Writes 1, 2 or 4 bytes to a file page. Writes the corresponding log
-record to the mini-transaction log if mtr is not NULL. */
-void
-mlog_write_ulint(
-/*=============*/
-	byte*		ptr,	/*!< in: pointer where to write */
-	ulint		val,	/*!< in: value to write */
-	mlog_id_t	type,	/*!< in: MLOG_1BYTE, MLOG_2BYTES, MLOG_4BYTES */
-	mtr_t*		mtr)	/*!< in: mini-transaction handle */
+/**
+Write a log record for writing 1, 2, 4, or 8 bytes.
+@param[in]      block   file page
+@param[in,out]  ptr     pointer in file page
+@param[in]      l       number of bytes to write
+@param[in,out]  log_ptr log record buffer
+@param[in,out]  mtr     mini-transaction */
+static byte *
+mlog_log_write_low(const buf_block_t &block, byte *ptr, mlog_id_t l,
+                   byte *log_ptr, mtr_t &mtr)
 {
-	switch (type) {
-	case MLOG_1BYTE:
-		mach_write_to_1(ptr, val);
-		break;
-	case MLOG_2BYTES:
-		mach_write_to_2(ptr, val);
-		break;
-	case MLOG_4BYTES:
-		mach_write_to_4(ptr, val);
-		break;
-	default:
-		ut_error;
-	}
+  ut_ad(block.page.state == BUF_BLOCK_FILE_PAGE);
+  ut_ad(ptr >= block.frame + FIL_PAGE_OFFSET);
+  ut_ad(ptr + unsigned(l) <= &block.frame[srv_page_size - FIL_PAGE_DATA_END]);
+  log_ptr= mlog_write_initial_log_record_low(l,
+                                             block.page.id.space(),
+                                             block.page.id.page_no(),
+                                             log_ptr, &mtr);
+  mach_write_to_2(log_ptr, page_offset(ptr));
+  return log_ptr + 2;
+}
 
-	if (mtr != 0) {
-		byte*	log_ptr = mlog_open(mtr, 11 + 2 + 5);
+/**
+Write a log record for writing 1, 2, or 4 bytes.
+@param[in]      block   file page
+@param[in,out]  ptr     pointer in file page
+@param[in]      l       number of bytes to write
+@param[in,out]  log_ptr log record buffer
+@param[in]      val     value to write */
+void mtr_t::log_write(const buf_block_t &block, byte *ptr, mlog_id_t l,
+                      byte *log_ptr, uint32_t val)
+{
+  ut_ad(l == MLOG_1BYTE || l == MLOG_2BYTES || l == MLOG_4BYTES);
+  log_ptr= mlog_log_write_low(block, ptr, l, log_ptr, *this);
+  log_ptr+= mach_write_compressed(log_ptr, val);
+  mlog_close(this, log_ptr);
+}
 
-		/* If no logging is requested, we may return now */
+/**
+Write a log record for writing 8 bytes.
+@param[in]      block   file page
+@param[in,out]  ptr     pointer in file page
+@param[in]      l       number of bytes to write
+@param[in,out]  log_ptr log record buffer
+@param[in]      val     value to write */
+void mtr_t::log_write(const buf_block_t &block, byte *ptr, mlog_id_t l,
+                      byte *log_ptr, uint64_t val)
+{
+  ut_ad(l == MLOG_8BYTES);
+  log_ptr= mlog_log_write_low(block, ptr, l, log_ptr, *this);
+  log_ptr+= mach_u64_write_compressed(log_ptr, val);
+  mlog_close(this, log_ptr);
+}
 
-		if (log_ptr != 0) {
+/** Log a write of a byte string to a page.
+@param[in]      b       buffer page
+@param[in]      ofs     byte offset from b->frame
+@param[in]      len     length of the data to write */
+void mtr_t::memcpy(const buf_block_t &b, ulint ofs, ulint len)
+{
+  ut_ad(len);
+  ut_ad(ofs <= ulint(srv_page_size));
+  ut_ad(ofs + len <= ulint(srv_page_size));
+  ut_ad(ofs + len < PAGE_DATA || !b.page.zip.data ||
+        mach_read_from_2(b.frame + FIL_PAGE_TYPE) <= FIL_PAGE_TYPE_ZBLOB2);
 
-			log_ptr = mlog_write_initial_log_record_fast(
-				ptr, type, log_ptr, mtr);
+  set_modified();
+  if (get_log_mode() != MTR_LOG_ALL)
+  {
+    ut_ad(get_log_mode() == MTR_LOG_NONE ||
+          get_log_mode() == MTR_LOG_NO_REDO);
+    return;
+  }
 
-			mach_write_to_2(log_ptr, page_offset(ptr));
-			log_ptr += 2;
-
-			log_ptr += mach_write_compressed(log_ptr, val);
-
-			mlog_close(mtr, log_ptr);
-		}
-	}
+  byte *l= get_log()->open(11 + 2 + 2);
+  l= mlog_write_initial_log_record_low(MLOG_WRITE_STRING, b.page.id.space(),
+                                       b.page.id.page_no(), l, this);
+  mach_write_to_2(l, ofs);
+  mach_write_to_2(l + 2, len);
+  mlog_close(this, l + 4);
+  mlog_catenate_string(this, b.frame + ofs, len);
 }
 
 /********************************************************//**
-Writes 8 bytes to a file page. Writes the corresponding log
-record to the mini-transaction log, only if mtr is not NULL */
-void
-mlog_write_ull(
-/*===========*/
-	byte*		ptr,	/*!< in: pointer where to write */
-	ib_uint64_t	val,	/*!< in: value to write */
-	mtr_t*		mtr)	/*!< in: mini-transaction handle */
-{
-	mach_write_to_8(ptr, val);
-
-	if (mtr != 0) {
-		byte*	log_ptr = mlog_open(mtr, 11 + 2 + 9);
-
-		/* If no logging is requested, we may return now */
-		if (log_ptr != 0) {
-
-			log_ptr = mlog_write_initial_log_record_fast(
-				ptr, MLOG_8BYTES, log_ptr, mtr);
-
-			mach_write_to_2(log_ptr, page_offset(ptr));
-			log_ptr += 2;
-
-			log_ptr += mach_u64_write_compressed(log_ptr, val);
-
-			mlog_close(mtr, log_ptr);
-		}
-	}
-}
-
-/********************************************************//**
-Writes a string to a file page buffered in the buffer pool. Writes the
-corresponding log record to the mini-transaction log. */
-void
-mlog_write_string(
-/*==============*/
-	byte*		ptr,	/*!< in: pointer where to write */
-	const byte*	str,	/*!< in: string to write */
-	ulint		len,	/*!< in: string length */
-	mtr_t*		mtr)	/*!< in: mini-transaction handle */
-{
-	ut_ad(ptr && mtr);
-	ut_a(len < srv_page_size);
-
-	memcpy(ptr, str, len);
-
-	mlog_log_string(ptr, len, mtr);
-}
-
-/********************************************************//**
-Logs a write of a string to a file page buffered in the buffer pool.
-Writes the corresponding log record to the mini-transaction log. */
-void
-mlog_log_string(
-/*============*/
-	byte*	ptr,	/*!< in: pointer written to */
-	ulint	len,	/*!< in: string length */
-	mtr_t*	mtr)	/*!< in: mini-transaction handle */
-{
-	byte*	log_ptr;
-
-	ut_ad(ptr && mtr);
-	ut_ad(len <= srv_page_size);
-
-	log_ptr = mlog_open(mtr, 30);
-
-	/* If no logging is requested, we may return now */
-	if (log_ptr == NULL) {
-
-		return;
-	}
-
-	log_ptr = mlog_write_initial_log_record_fast(ptr, MLOG_WRITE_STRING,
-						     log_ptr, mtr);
-	mach_write_to_2(log_ptr, page_offset(ptr));
-	log_ptr += 2;
-
-	mach_write_to_2(log_ptr, len);
-	log_ptr += 2;
-
-	mlog_close(mtr, log_ptr);
-
-	mlog_catenate_string(mtr, ptr, len);
-}
-
-/********************************************************//**
-Parses a log record written by mlog_write_string.
+Parses a log record written by mtr_t::memcpy().
 @return parsed record end, NULL if not a complete record */
 const byte*
 mlog_parse_string(
@@ -427,69 +348,34 @@ mlog_parse_string(
 }
 
 /** Initialize a string of bytes.
-@param[in,out]	b	buffer page
-@param[in]	ofs	byte offset from block->frame
-@param[in]	len	length of the data to write
-@param[in]	val	the data byte to write
-@param[in,out]	mtr	mini-transaction */
-void
-mlog_memset(buf_block_t* b, ulint ofs, ulint len, byte val, mtr_t* mtr)
+@param[in,out]  b       buffer page
+@param[in]      ofs     byte offset from block->frame
+@param[in]      len     length of the data to write
+@param[in]      val     the data byte to write */
+void mtr_t::memset(const buf_block_t* b, ulint ofs, ulint len, byte val)
 {
-	ut_ad(len);
-	ut_ad(ofs <= ulint(srv_page_size));
-	ut_ad(ofs + len <= ulint(srv_page_size));
-	memset(ofs + b->frame, val, len);
+  ut_ad(len);
+  ut_ad(ofs <= ulint(srv_page_size));
+  ut_ad(ofs + len <= ulint(srv_page_size));
+  ut_ad(ofs + len < PAGE_DATA || !b->page.zip.data ||
+        mach_read_from_2(b->frame + FIL_PAGE_TYPE) <= FIL_PAGE_TYPE_ZBLOB2);
+  ::memset(ofs + b->frame, val, len);
 
-	mtr->set_modified();
-	switch (mtr->get_log_mode()) {
-	case MTR_LOG_NONE:
-	case MTR_LOG_NO_REDO:
-		return;
-	case MTR_LOG_SHORT_INSERTS:
-		ut_ad(0);
-		/* fall through */
-	case MTR_LOG_ALL:
-		break;
-	}
+  set_modified();
+  if (get_log_mode() != MTR_LOG_ALL)
+  {
+    ut_ad(get_log_mode() == MTR_LOG_NONE ||
+          get_log_mode() == MTR_LOG_NO_REDO);
+    return;
+  }
 
-	byte* l = mtr->get_log()->open(11 + 2 + 2 + 1);
-	l = mlog_write_initial_log_record_low(
-		MLOG_MEMSET, b->page.id.space(), b->page.id.page_no(), l, mtr);
-	mach_write_to_2(l, ofs);
-	mach_write_to_2(l + 2, len);
-	l[4] = val;
-	mlog_close(mtr, l + 5);
-}
-
-/** Initialize a string of bytes.
-@param[in,out]	byte	byte address
-@param[in]	len	length of the data to write
-@param[in]	val	the data byte to write
-@param[in,out]	mtr	mini-transaction */
-void mlog_memset(byte* b, ulint len, byte val, mtr_t* mtr)
-{
-	ut_ad(len);
-	ut_ad(page_offset(b) + len <= ulint(srv_page_size));
-	memset(b, val, len);
-
-	mtr->set_modified();
-	switch (mtr->get_log_mode()) {
-	case MTR_LOG_NONE:
-	case MTR_LOG_NO_REDO:
-		return;
-	case MTR_LOG_SHORT_INSERTS:
-		ut_ad(0);
-		/* fall through */
-	case MTR_LOG_ALL:
-		break;
-	}
-
-	byte* l = mtr->get_log()->open(11 + 2 + 2 + 1);
-	l = mlog_write_initial_log_record_fast(b, MLOG_MEMSET, l, mtr);
-	mach_write_to_2(l, page_offset(b));
-	mach_write_to_2(l + 2, len);
-	l[4] = val;
-	mlog_close(mtr, l + 5);
+  byte *l= get_log()->open(11 + 2 + 2 + 1);
+  l= mlog_write_initial_log_record_low(MLOG_MEMSET, b->page.id.space(),
+                                       b->page.id.page_no(), l, this);
+  mach_write_to_2(l, ofs);
+  mach_write_to_2(l + 2, len);
+  l[4]= val;
+  mlog_close(this, l + 5);
 }
 
 /********************************************************//**
