@@ -1,7 +1,7 @@
 /*****************************************************************************
 
 Copyright (c) 1995, 2017, Oracle and/or its affiliates. All Rights Reserved.
-Copyright (c) 2017, 2019, MariaDB Corporation.
+Copyright (c) 2017, 2020, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -297,11 +297,10 @@ struct DebugCheck {
 /** Release a resource acquired by the mini-transaction. */
 struct ReleaseBlocks {
 	/** Release specific object */
-	ReleaseBlocks(lsn_t start_lsn, lsn_t end_lsn, FlushObserver* observer)
+	ReleaseBlocks(lsn_t start_lsn, lsn_t end_lsn)
 		:
 		m_end_lsn(end_lsn),
-		m_start_lsn(start_lsn),
-		m_flush_observer(observer)
+		m_start_lsn(start_lsn)
 	{
 		/* Do nothing */
 	}
@@ -316,8 +315,7 @@ struct ReleaseBlocks {
 
 		block = reinterpret_cast<buf_block_t*>(slot->object);
 
-		buf_flush_note_modification(block, m_start_lsn,
-					    m_end_lsn, m_flush_observer);
+		buf_flush_note_modification(block, m_start_lsn, m_end_lsn);
 	}
 
 	/** @return true always. */
@@ -340,9 +338,6 @@ struct ReleaseBlocks {
 
 	/** Mini-transaction REDO end LSN */
 	lsn_t		m_start_lsn;
-
-	/** Flush observer */
-	FlushObserver*	m_flush_observer;
 };
 
 /** Write the block contents to the REDO log */
@@ -380,6 +375,9 @@ void mtr_t::start()
 {
   UNIV_MEM_INVALID(this, sizeof *this);
 
+  ut_d(m_start= true);
+  ut_d(m_commit= false);
+
   new(&m_memo) mtr_buf_t();
   new(&m_log) mtr_buf_t();
 
@@ -389,24 +387,22 @@ void mtr_t::start()
   m_n_log_recs= 0;
   m_log_mode= MTR_LOG_ALL;
   ut_d(m_user_space_id= TRX_SYS_SPACE);
-  m_user_space= NULL;
-  m_state= MTR_STATE_ACTIVE;
-  m_flush_observer= NULL;
+  m_user_space= nullptr;
   m_commit_lsn= 0;
 }
 
 /** Release the resources */
 inline void mtr_t::release_resources()
 {
+  ut_ad(is_active());
   ut_d(m_memo.for_each_block_in_reverse(CIterate<DebugCheck>()));
   m_log.erase();
   m_memo.erase();
-  m_state= MTR_STATE_COMMITTED;
+  ut_d(m_commit= true);
 }
 
 /** Commit a mini-transaction. */
-void
-mtr_t::commit()
+void mtr_t::commit()
 {
   ut_ad(is_active());
   ut_ad(!is_inside_ibuf());
@@ -415,8 +411,7 @@ mtr_t::commit()
   ut_ad(!m_modifications || !recv_no_log_write);
   ut_ad(!m_modifications || m_log_mode != MTR_LOG_NONE);
 
-  if (m_modifications
-      && (m_n_log_recs || m_log_mode == MTR_LOG_NO_REDO))
+  if (m_modifications && (m_n_log_recs || m_log_mode == MTR_LOG_NO_REDO))
   {
     ut_ad(!srv_read_only_mode || m_log_mode == MTR_LOG_NO_REDO);
 
@@ -436,8 +431,7 @@ mtr_t::commit()
     log_mutex_exit();
 
     m_memo.for_each_block_in_reverse(CIterate<const ReleaseBlocks>
-                                     (ReleaseBlocks(start_lsn, m_commit_lsn,
-                                                    m_flush_observer)));
+                                     (ReleaseBlocks(start_lsn, m_commit_lsn)));
     if (m_made_dirty)
       log_flush_order_mutex_exit();
 
@@ -451,16 +445,11 @@ mtr_t::commit()
 
 /** Commit a mini-transaction that did not modify any pages,
 but generated some redo log on a higher level, such as
-MLOG_FILE_NAME records and a MLOG_CHECKPOINT marker.
+MLOG_FILE_NAME records and an optional MLOG_CHECKPOINT marker.
 The caller must invoke log_mutex_enter() and log_mutex_exit().
 This is to be used at log_checkpoint().
-@param[in]	checkpoint_lsn		the LSN of the log checkpoint
-@param[in]	write_mlog_checkpoint	Write MLOG_CHECKPOINT marker
-					if it is enabled. */
-void
-mtr_t::commit_checkpoint(
-	lsn_t	checkpoint_lsn,
-	bool	write_mlog_checkpoint)
+@param[in]	checkpoint_lsn		log checkpoint LSN, or 0 */
+void mtr_t::commit_files(lsn_t checkpoint_lsn)
 {
 	ut_ad(log_mutex_own());
 	ut_ad(is_active());
@@ -469,7 +458,7 @@ mtr_t::commit_checkpoint(
 	ut_ad(!m_made_dirty);
 	ut_ad(m_memo.size() == 0);
 	ut_ad(!srv_read_only_mode);
-	ut_ad(write_mlog_checkpoint || m_n_log_recs > 1);
+	ut_ad(checkpoint_lsn || m_n_log_recs > 1);
 
 	switch (m_n_log_recs) {
 	case 0:
@@ -481,7 +470,7 @@ mtr_t::commit_checkpoint(
 		mlog_catenate_ulint(&m_log, MLOG_MULTI_REC_END, MLOG_1BYTE);
 	}
 
-	if (write_mlog_checkpoint) {
+	if (checkpoint_lsn) {
 		byte*	ptr = m_log.push<byte*>(SIZE_OF_MLOG_CHECKPOINT);
 		compile_time_assert(SIZE_OF_MLOG_CHECKPOINT == 1 + 8);
 		*ptr = MLOG_CHECKPOINT;
@@ -491,7 +480,7 @@ mtr_t::commit_checkpoint(
 	finish_write(m_log.size());
 	release_resources();
 
-	if (write_mlog_checkpoint) {
+	if (checkpoint_lsn) {
 		DBUG_PRINT("ib_log",
 			   ("MLOG_CHECKPOINT(" LSN_PF ") written at " LSN_PF,
 			    checkpoint_lsn, log_sys.lsn));
@@ -563,8 +552,7 @@ mtr_t::x_lock_space(ulint space_id, const char* file, unsigned line)
 		space = fil_space_get(space_id);
 		ut_ad(get_log_mode() != MTR_LOG_NO_REDO
 		      || space->purpose == FIL_TYPE_TEMPORARY
-		      || space->purpose == FIL_TYPE_IMPORT
-		      || space->redo_skipped_count > 0);
+		      || space->purpose == FIL_TYPE_IMPORT);
 	}
 
 	ut_ad(space);
