@@ -118,7 +118,7 @@ row_undo_ins_remove_clust_rec(
 
 	if (online && dict_index_is_online_ddl(index)) {
 		mem_heap_t*	heap	= NULL;
-		const ulint*	offsets	= rec_get_offsets(
+		const offset_t*	offsets	= rec_get_offsets(
 			rec, index, NULL, true, ULINT_UNDEFINED, &heap);
 		row_log_table_delete(rec, index, offsets, NULL);
 		mem_heap_free(heap);
@@ -130,8 +130,7 @@ row_undo_ins_remove_clust_rec(
 			      == RW_X_LATCH);
 			ut_ad(node->rec_type == TRX_UNDO_INSERT_REC);
 
-			dict_drop_index_tree(rec, &node->pcur, node->trx,
-					     &mtr);
+			dict_drop_index_tree(&node->pcur, node->trx, &mtr);
 			mtr.commit();
 
 			mtr.start();
@@ -208,25 +207,29 @@ func_exit:
 		/* When rolling back the very first instant ADD COLUMN
 		operation, reset the root page to the basic state. */
 		ut_ad(!index->table->is_temporary());
-		if (page_t* root = btr_root_get(index, &mtr)) {
-			byte* page_type = root + FIL_PAGE_TYPE;
+		if (buf_block_t* root = btr_root_block_get(index, RW_SX_LATCH,
+							   &mtr)) {
+			byte* page_type = root->frame + FIL_PAGE_TYPE;
 			ut_ad(mach_read_from_2(page_type)
 			      == FIL_PAGE_TYPE_INSTANT
 			      || mach_read_from_2(page_type)
 			      == FIL_PAGE_INDEX);
-			mlog_write_ulint(page_type, FIL_PAGE_INDEX,
-					 MLOG_2BYTES, &mtr);
-			byte* instant = PAGE_INSTANT + PAGE_HEADER + root;
-			mlog_write_ulint(instant,
-					 page_ptr_get_direction(instant + 1),
-					 MLOG_2BYTES, &mtr);
-			rec_t* infimum = page_get_infimum_rec(root);
-			rec_t* supremum = page_get_supremum_rec(root);
+			mtr.write<2,mtr_t::OPT>(*root, page_type,
+						FIL_PAGE_INDEX);
+			byte* instant = PAGE_INSTANT + PAGE_HEADER
+				+ root->frame;
+			mtr.write<2,mtr_t::OPT>(
+				*root, instant,
+				page_ptr_get_direction(instant + 1));
+			rec_t* infimum = page_get_infimum_rec(root->frame);
+			rec_t* supremum = page_get_supremum_rec(root->frame);
 			static const byte str[8 + 8] = "supremuminfimum";
 			if (memcmp(infimum, str + 8, 8)
 			    || memcmp(supremum, str, 8)) {
-				mlog_write_string(infimum, str + 8, 8, &mtr);
-				mlog_write_string(supremum, str, 8, &mtr);
+				mtr.memcpy(root, page_offset(infimum),
+					   str + 8, 8);
+				mtr.memcpy(root, page_offset(supremum),
+					   str, 8);
 			}
 		}
 	}
