@@ -3,7 +3,7 @@
 Copyright (c) 1995, 2017, Oracle and/or its affiliates. All Rights Reserved.
 Copyright (c) 2008, 2009 Google Inc.
 Copyright (c) 2009, Percona Inc.
-Copyright (c) 2013, 2019, MariaDB Corporation.
+Copyright (c) 2013, 2020, MariaDB Corporation.
 
 Portions of this file contain modifications contributed and copyrighted by
 Google, Inc. Those modifications are gratefully acknowledged and are described
@@ -125,10 +125,6 @@ my_bool	srv_undo_log_truncate;
 /** Maximum size of undo tablespace. */
 unsigned long long	srv_max_undo_log_size;
 
-/** Default undo tablespace size in UNIV_PAGEs count (10MB). */
-const ulint SRV_UNDO_TABLESPACE_SIZE_IN_PAGES =
-	((1024 * 1024) * 10) / UNIV_PAGE_SIZE_DEF;
-
 /** Set if InnoDB must operate in read-only mode. We don't do any
 recovery and open all tables in RO mode instead of RW mode. We don't
 sync the max trx id to disk either. */
@@ -212,13 +208,8 @@ ulint	srv_buf_pool_size;
 const ulint	srv_buf_pool_min_size	= 5 * 1024 * 1024;
 /** Default pool size in bytes */
 const ulint	srv_buf_pool_def_size	= 128 * 1024 * 1024;
-/** Requested buffer pool chunk size. Each buffer pool instance consists
-of one or more chunks. */
+/** Requested buffer pool chunk size */
 ulong	srv_buf_pool_chunk_unit;
-/** innodb_buffer_pool_instances (0 is interpreted as 1) */
-ulong	srv_buf_pool_instances;
-/** Default value of innodb_buffer_pool_instances */
-const ulong	srv_buf_pool_instances_default = 0;
 /** innodb_page_hash_locks (a debug-only parameter);
 number of locks to protect buf_pool->page_hash */
 ulong	srv_n_page_hash_locks = 16;
@@ -271,9 +262,6 @@ ulong	srv_io_capacity;
 /** innodb_io_capacity_max */
 ulong	srv_max_io_capacity;
 
-/** innodb_page_cleaners; the number of page cleaner threads */
-ulong	srv_n_page_cleaners;
-
 /* The InnoDB main thread tries to keep the ratio of modified pages
 in the buffer pool to all database pages in the buffer pool smaller than
 the following number. But it is not guaranteed that the value stays below
@@ -472,7 +460,9 @@ current_time % 60 == 0 and no tasks will be performed when
 current_time % 5 != 0. */
 
 # define	SRV_MASTER_CHECKPOINT_INTERVAL		(7)
-# define	SRV_MASTER_PURGE_INTERVAL		(10)
+#ifdef MEM_PERIODIC_CHECK
+# define	SRV_MASTER_MEM_VALIDATE_INTERVAL	(13)
+#endif /* MEM_PERIODIC_CHECK */
 # define	SRV_MASTER_DICT_LRU_INTERVAL		(47)
 
 /** Simulate compression failures. */
@@ -593,10 +583,6 @@ struct purge_coordinator_state
 };
 
 static purge_coordinator_state purge_state;
-extern tpool::waitable_task purge_coordinator_task;
-
-/** @return whether the purge coordinator thread is active */
-bool purge_sys_t::running() { return purge_coordinator_task.is_running(); }
 
 /** threadpool timer for srv_error_monitor_task(). */
 std::unique_ptr<tpool::timer> srv_error_monitor_timer;
@@ -627,11 +613,6 @@ everything after flush log_make_checkpoint(). */
 PSI_stage_info	srv_stage_alter_table_end
 	= {0, "alter table (end)", PSI_FLAG_STAGE_PROGRESS};
 
-/** Performance schema stage event for monitoring ALTER TABLE progress
-log_make_checkpoint(). */
-PSI_stage_info	srv_stage_alter_table_flush
-	= {0, "alter table (flush)", PSI_FLAG_STAGE_PROGRESS};
-
 /** Performance schema stage event for monitoring ALTER TABLE progress
 row_merge_insert_index_tuples(). */
 PSI_stage_info	srv_stage_alter_table_insert
@@ -718,6 +699,13 @@ static void thread_pool_thread_end()
 }
 
 
+#ifndef DBUG_OFF
+static void dbug_after_task_callback()
+{
+  ut_ad(!sync_check_iterate(sync_check()));
+}
+#endif
+
 void srv_thread_pool_init()
 {
   DBUG_ASSERT(!srv_thread_pool);
@@ -729,6 +717,9 @@ void srv_thread_pool_init()
 #endif
   srv_thread_pool->set_thread_callbacks(thread_pool_thread_init,
                                         thread_pool_thread_end);
+#ifndef DBUG_OFF
+  tpool::set_after_task_callback(dbug_after_task_callback);
+#endif
 }
 
 
@@ -859,7 +850,7 @@ srv_refresh_innodb_monitor_stats(void)
 
 	log_refresh_stats();
 
-	buf_refresh_io_stats_all();
+	buf_refresh_io_stats();
 
 	srv_n_rows_inserted_old = srv_stats.n_rows_inserted;
 	srv_n_rows_updated_old = srv_stats.n_rows_updated;
@@ -1130,17 +1121,9 @@ void
 srv_export_innodb_status(void)
 /*==========================*/
 {
-	buf_pool_stat_t		stat;
-	buf_pools_list_size_t	buf_pools_list_size;
-	ulint			LRU_len;
-	ulint			free_len;
-	ulint			flush_list_len;
 	fil_crypt_stat_t	crypt_stat;
 	btr_scrub_stat_t	scrub_stat;
 
-	buf_get_total_stat(&stat);
-	buf_get_total_list_len(&LRU_len, &free_len, &flush_list_len);
-	buf_get_total_list_size_in_bytes(&buf_pools_list_size);
 	if (!srv_read_only_mode) {
 		fil_crypt_total_stat(&crypt_stat);
 		btr_scrub_total_stat(&scrub_stat);
@@ -1150,6 +1133,7 @@ srv_export_innodb_status(void)
 	ulint mem_adaptive_hash = 0;
 	ut_ad(btr_search_sys->hash_tables);
 	for (ulong i = 0; i < btr_ahi_parts; i++) {
+		rw_lock_s_lock(btr_search_latches[i]);
 		hash_table_t*	ht = btr_search_sys->hash_tables[i];
 
 		ut_ad(ht);
@@ -1161,6 +1145,7 @@ srv_export_innodb_status(void)
 
 		mem_adaptive_hash += mem_heap_get_size(ht->heap)
 			+ ht->n_cells * sizeof(hash_cell_t);
+		rw_lock_s_unlock(btr_search_latches[i]);
 	}
 	export_vars.innodb_mem_adaptive_hash = mem_adaptive_hash;
 #endif
@@ -1176,7 +1161,7 @@ srv_export_innodb_status(void)
 		ulint(MONITOR_VALUE(MONITOR_OS_PENDING_WRITES));
 
 	export_vars.innodb_data_pending_fsyncs =
-		fil_n_pending_log_flushes
+		log_sys.get_pending_flushes()
 		+ fil_n_pending_tablespace_flushes;
 
 	export_vars.innodb_data_fsyncs = os_n_fsyncs;
@@ -1189,7 +1174,8 @@ srv_export_innodb_status(void)
 
 	export_vars.innodb_data_written = srv_stats.data_written;
 
-	export_vars.innodb_buffer_pool_read_requests = stat.n_page_gets;
+	export_vars.innodb_buffer_pool_read_requests
+		= buf_pool->stat.n_page_gets;
 
 	export_vars.innodb_buffer_pool_write_requests =
 		srv_stats.buf_pool_write_requests;
@@ -1203,38 +1189,37 @@ srv_export_innodb_status(void)
 	export_vars.innodb_buffer_pool_reads = srv_stats.buf_pool_reads;
 
 	export_vars.innodb_buffer_pool_read_ahead_rnd =
-		stat.n_ra_pages_read_rnd;
+		buf_pool->stat.n_ra_pages_read_rnd;
 
 	export_vars.innodb_buffer_pool_read_ahead =
-		stat.n_ra_pages_read;
+		buf_pool->stat.n_ra_pages_read;
 
 	export_vars.innodb_buffer_pool_read_ahead_evicted =
-		stat.n_ra_pages_evicted;
+		buf_pool->stat.n_ra_pages_evicted;
 
-	export_vars.innodb_buffer_pool_pages_data = LRU_len;
+	export_vars.innodb_buffer_pool_pages_data =
+		UT_LIST_GET_LEN(buf_pool->LRU);
 
 	export_vars.innodb_buffer_pool_bytes_data =
-		buf_pools_list_size.LRU_bytes
-		+ buf_pools_list_size.unzip_LRU_bytes;
+		buf_pool->stat.LRU_bytes
+		+ (UT_LIST_GET_LEN(buf_pool->unzip_LRU)
+		   << srv_page_size_shift);
 
-	export_vars.innodb_buffer_pool_pages_dirty = flush_list_len;
+	export_vars.innodb_buffer_pool_pages_dirty =
+		UT_LIST_GET_LEN(buf_pool->flush_list);
 
 	export_vars.innodb_buffer_pool_pages_made_young
-		= stat.n_pages_made_young;
+		= buf_pool->stat.n_pages_made_young;
 	export_vars.innodb_buffer_pool_pages_made_not_young
-		= stat.n_pages_not_made_young;
+		= buf_pool->stat.n_pages_not_made_young;
 
-	export_vars.innodb_buffer_pool_pages_old = 0;
-
-	for (ulong i = 0; i < srv_buf_pool_instances; i++) {
-		export_vars.innodb_buffer_pool_pages_old +=
-			buf_pool_from_array(i)->LRU_old_len;
-	}
+	export_vars.innodb_buffer_pool_pages_old = buf_pool->LRU_old_len;
 
 	export_vars.innodb_buffer_pool_bytes_dirty =
-		buf_pools_list_size.flush_list_bytes;
+		buf_pool->stat.flush_list_bytes;
 
-	export_vars.innodb_buffer_pool_pages_free = free_len;
+	export_vars.innodb_buffer_pool_pages_free =
+		UT_LIST_GET_LEN(buf_pool->free);
 
 #ifdef UNIV_DEBUG
 	export_vars.innodb_buffer_pool_pages_latched =
@@ -1243,7 +1228,9 @@ srv_export_innodb_status(void)
 	export_vars.innodb_buffer_pool_pages_total = buf_pool_get_n_pages();
 
 	export_vars.innodb_buffer_pool_pages_misc =
-		buf_pool_get_n_pages() - LRU_len - free_len;
+		buf_pool_get_n_pages()
+		- UT_LIST_GET_LEN(buf_pool->LRU)
+		- UT_LIST_GET_LEN(buf_pool->free);
 
 	export_vars.innodb_max_trx_id = trx_sys.get_max_trx_id();
 	export_vars.innodb_history_list_length = trx_sys.rseg_history_len;
@@ -1252,9 +1239,10 @@ srv_export_innodb_status(void)
 
 	export_vars.innodb_os_log_written = srv_stats.os_log_written;
 
-	export_vars.innodb_os_log_fsyncs = fil_n_log_flushes;
+	export_vars.innodb_os_log_fsyncs = log_sys.get_flushes();
 
-	export_vars.innodb_os_log_pending_fsyncs = fil_n_pending_log_flushes;
+	export_vars.innodb_os_log_pending_fsyncs
+		= log_sys.get_pending_flushes();
 
 	export_vars.innodb_os_log_pending_writes =
 		srv_stats.os_log_pending_writes;
@@ -1268,11 +1256,11 @@ srv_export_innodb_status(void)
 
 	export_vars.innodb_dblwr_writes = srv_stats.dblwr_writes;
 
-	export_vars.innodb_pages_created = stat.n_pages_created;
+	export_vars.innodb_pages_created = buf_pool->stat.n_pages_created;
 
-	export_vars.innodb_pages_read = stat.n_pages_read;
+	export_vars.innodb_pages_read = buf_pool->stat.n_pages_read;
 
-	export_vars.innodb_pages_written = stat.n_pages_written;
+	export_vars.innodb_pages_written = buf_pool->stat.n_pages_written;
 
 	export_vars.innodb_row_lock_waits = srv_stats.n_lock_wait_count;
 
@@ -1581,9 +1569,8 @@ static tpool::task_group purge_task_group;
 tpool::waitable_task purge_worker_task(purge_worker_callback, nullptr,
                                        &purge_task_group);
 static tpool::task_group purge_coordinator_task_group(1);
-tpool::waitable_task purge_coordinator_task(purge_coordinator_callback,
-                                            nullptr,
-                                            &purge_coordinator_task_group);
+static tpool::waitable_task purge_coordinator_task
+  (purge_coordinator_callback, nullptr, &purge_coordinator_task_group);
 
 static tpool::timer *purge_coordinator_timer;
 
@@ -1602,6 +1589,66 @@ srv_wake_purge_thread_if_not_active()
 	}
 }
 
+/** @return whether the purge tasks are active */
+bool purge_sys_t::running() const
+{
+  return purge_coordinator_task.is_running();
+}
+
+/** Stop purge during FLUSH TABLES FOR EXPORT */
+void purge_sys_t::stop()
+{
+  rw_lock_x_lock(&latch);
+
+  if (!enabled())
+  {
+    /* Shutdown must have been initiated during FLUSH TABLES FOR EXPORT. */
+    ut_ad(!srv_undo_sources);
+    rw_lock_x_unlock(&latch);
+    return;
+  }
+
+  ut_ad(srv_n_purge_threads > 0);
+
+  const auto paused= m_paused++;
+
+  rw_lock_x_unlock(&latch);
+
+  if (!paused)
+  {
+    ib::info() << "Stopping purge";
+    MONITOR_ATOMIC_INC(MONITOR_PURGE_STOP_COUNT);
+    purge_coordinator_task.disable();
+  }
+}
+
+/** Resume purge at UNLOCK TABLES after FLUSH TABLES FOR EXPORT */
+void purge_sys_t::resume()
+{
+   if (!enabled())
+   {
+     /* Shutdown must have been initiated during FLUSH TABLES FOR EXPORT. */
+     ut_ad(!srv_undo_sources);
+     return;
+   }
+   ut_ad(!srv_read_only_mode);
+   ut_ad(srv_force_recovery < SRV_FORCE_NO_BACKGROUND);
+   ut_ad(!sync_check_iterate(sync_check()));
+   purge_coordinator_task.enable();
+   rw_lock_x_lock(&latch);
+   int32_t paused= m_paused--;
+   ut_a(paused);
+
+   if (paused == 1)
+   {
+     ib::info() << "Resuming purge";
+     purge_state.m_running = 0;
+     srv_wake_purge_thread_if_not_active();
+     MONITOR_ATOMIC_INC(MONITOR_PURGE_RESUME_COUNT);
+   }
+   rw_lock_x_unlock(&latch);
+}
+
 /** Wake up the master thread if it is suspended or being suspended. */
 void
 srv_wake_master_thread()
@@ -2033,7 +2080,6 @@ static bool srv_task_execute()
 }
 
 
-
 /** Do the actual purge operation.
 @param[in,out]	n_total_purged	total number of purged pages
 @return length of history list before the last purge batch. */
@@ -2174,7 +2220,8 @@ static void purge_worker_callback(void*)
   ut_ad(srv_force_recovery < SRV_FORCE_NO_BACKGROUND);
   void *ctx;
   THD *thd= acquire_thd(&ctx);
-  while (srv_task_execute()) {}
+  while (srv_task_execute())
+    ut_ad(purge_sys.running());
   release_thd(thd,ctx);
 }
 
@@ -2279,21 +2326,6 @@ ulint srv_get_task_queue_length()
 }
 #endif
 
-/** Wake up the purge coordinator. */
-void
-srv_purge_wakeup()
-{
-	ut_ad(!srv_read_only_mode);
-	ut_ad(!sync_check_iterate(sync_check()));
-
-	if (srv_force_recovery >= SRV_FORCE_NO_BACKGROUND) {
-		return;
-	}
-	ut_a(purge_sys.enabled() && !purge_sys.paused());
-	purge_state.m_running = 0;
-	srv_wake_purge_thread_if_not_active();
-}
-
 /** Shut down the purge threads. */
 void srv_purge_shutdown()
 {
