@@ -1,7 +1,7 @@
 /*****************************************************************************
 
 Copyright (c) 1996, 2017, Oracle and/or its affiliates. All Rights Reserved.
-Copyright (c) 2016, 2020, MariaDB Corporation.
+Copyright (c) 2016, 2019, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -44,6 +44,10 @@ Created 3/26/1996 Heikki Tuuri
 #include "trx0trx.h"
 #include "trx0undo.h"
 
+#ifdef UNIV_PFS_THREAD
+mysql_pfs_key_t	trx_rollback_clean_thread_key;
+#endif
+
 /** true if trx_rollback_all_recovered() thread is active */
 bool			trx_rollback_is_active;
 
@@ -59,10 +63,10 @@ static bool trx_rollback_finish(trx_t* trx)
 	trx->mod_tables.clear();
 	bool finished = trx->error_state == DB_SUCCESS;
 	if (UNIV_LIKELY(finished)) {
-		trx->commit();
+		trx_commit(trx);
 	} else {
 		ut_a(trx->error_state == DB_INTERRUPTED);
-		ut_ad(srv_shutdown_state != SRV_SHUTDOWN_NONE);
+		ut_ad(!srv_is_being_started);
 		ut_a(!srv_undo_sources);
 		ut_ad(srv_fast_shutdown);
 		ut_d(trx->in_rollback = false);
@@ -84,7 +88,7 @@ static bool trx_rollback_finish(trx_t* trx)
 			ut_free(undo);
 			undo = NULL;
 		}
-		trx->commit_low();
+		trx_commit_low(trx, NULL);
 	}
 
 	trx->lock.que_state = TRX_QUE_RUNNING;
@@ -178,7 +182,7 @@ trx_rollback_to_savepoint(
 				complete rollback */
 {
 #ifdef WITH_WSREP
-	if (!savept && trx->is_wsrep() && wsrep_thd_is_SR(trx->mysql_thd)) {
+	if (savept == NULL && wsrep_on(trx->mysql_thd)) {
 		wsrep_handle_SR_rollback(NULL, trx->mysql_thd);
 	}
 #endif /* WITH_WSREP */
@@ -228,10 +232,6 @@ dberr_t trx_rollback_for_mysql(trx_t* trx)
 	case TRX_STATE_NOT_STARTED:
 		trx->will_lock = 0;
 		ut_ad(trx->mysql_thd);
-#ifdef WITH_WSREP
-		trx->wsrep= false;
-		trx->lock.was_chosen_as_wsrep_victim= false;
-#endif
 		return(DB_SUCCESS);
 
 	case TRX_STATE_ACTIVE:
@@ -243,21 +243,10 @@ dberr_t trx_rollback_for_mysql(trx_t* trx)
 	case TRX_STATE_PREPARED_RECOVERED:
 		ut_ad(!trx_is_autocommit_non_locking(trx));
 		if (trx->rsegs.m_redo.undo || trx->rsegs.m_redo.old_insert) {
-			/* The XA ROLLBACK of a XA PREPARE transaction
-			will consist of multiple mini-transactions.
-
-			As the very first step of XA ROLLBACK, we must
-			change the undo log state back from
-			TRX_UNDO_PREPARED to TRX_UNDO_ACTIVE, in order
-			to ensure that recovery will complete the
-			rollback.
-
-			Failure to perform this step could cause a
-			situation where we would roll back part of
-			a XA PREPARE transaction, the server would be
-			killed, and finally, the transaction would be
-			recovered in XA PREPARE state, with some of
-			the actions already having been rolled back. */
+			/* Change the undo log state back from
+			TRX_UNDO_PREPARED to TRX_UNDO_ACTIVE
+			so that if the system gets killed,
+			recovery will perform the rollback. */
 			ut_ad(!trx->rsegs.m_redo.undo
 			      || trx->rsegs.m_redo.undo->rseg
 			      == trx->rsegs.m_redo.rseg);
@@ -276,15 +265,29 @@ dberr_t trx_rollback_for_mysql(trx_t* trx)
 							      &mtr);
 			}
 			mutex_exit(&trx->rsegs.m_redo.rseg->mutex);
-			/* Write the redo log for the XA ROLLBACK
-			state change to the global buffer. It is
-			not necessary to flush the redo log. If
-			a durable log write of a later mini-transaction
-			takes place for whatever reason, then this state
-			change will be durable as well. */
+			/* Persist the XA ROLLBACK, so that crash
+			recovery will replay the rollback in case
+			the redo log gets applied past this point. */
 			mtr.commit();
 			ut_ad(mtr.commit_lsn() > 0);
 		}
+#ifdef ENABLED_DEBUG_SYNC
+		if (trx->mysql_thd == NULL) {
+			/* We could be executing XA ROLLBACK after
+			XA PREPARE and a server restart. */
+		} else if (!trx->has_logged_persistent()) {
+			/* innobase_close_connection() may roll back a
+			transaction that did not generate any
+			persistent undo log. The DEBUG_SYNC
+			would cause an assertion failure for a
+			disconnected thread.
+
+			NOTE: InnoDB will not know about the XID
+			if no persistent undo log was generated. */
+		} else {
+			DEBUG_SYNC_C("trx_xa_rollback");
+		}
+#endif /* ENABLED_DEBUG_SYNC */
 		return(trx_rollback_for_mysql_low(trx));
 
 	case TRX_STATE_COMMITTED_IN_MEMORY:
@@ -363,8 +366,7 @@ trx_savepoint_find(
 	for (savep = UT_LIST_GET_FIRST(trx->trx_savepoints);
 	     savep != NULL;
 	     savep = UT_LIST_GET_NEXT(trx_savepoints, savep)) {
-
-		if (0 == ut_strcmp(savep->name, name)) {
+		if (!strcmp(savep->name, name)) {
 			return(savep);
 		}
 	}
@@ -454,7 +456,7 @@ trx_rollback_to_savepoint_for_mysql_low(
 
 	trx->op_info = "";
 #ifdef WITH_WSREP
-	trx->lock.was_chosen_as_wsrep_victim = false;
+	trx->lock.was_chosen_as_wsrep_victim = FALSE;
 #endif
 	return(err);
 }
@@ -591,6 +593,19 @@ trx_release_savepoint_for_mysql(
 	return(savep != NULL ? DB_SUCCESS : DB_NO_SAVEPOINT);
 }
 
+/*******************************************************************//**
+Determines if this transaction is rolling back an incomplete transaction
+in crash recovery.
+@return TRUE if trx is an incomplete transaction that is being rolled
+back in crash recovery */
+ibool
+trx_is_recv(
+/*========*/
+	const trx_t*	trx)	/*!< in: transaction */
+{
+	return(trx == trx_roll_crash_recv_trx);
+}
+
 /*******************************************************************//**
 Returns a transaction savepoint taken at this point in time.
 @return savepoint */
@@ -800,8 +815,7 @@ void trx_rollback_recovered(bool all)
     ut_ad(trx_state_eq(trx, TRX_STATE_ACTIVE));
     ut_d(trx_mutex_exit(trx));
 
-    if (srv_shutdown_state != SRV_SHUTDOWN_NONE && !srv_undo_sources &&
-        srv_fast_shutdown)
+    if (!srv_is_being_started && !srv_undo_sources && srv_fast_shutdown)
       goto discard;
 
     if (all || trx_get_dict_operation(trx) != TRX_DICT_OP_NONE)
@@ -834,12 +848,11 @@ discard:
         trx_free_at_shutdown(trx);
       }
       else
-        trx->free();
+        trx_free(trx);
     }
   }
 }
 
-
 /*******************************************************************//**
 Rollback or clean up any incomplete transactions which were
 encountered in crash recovery.  If the transaction already was
