@@ -1,4 +1,5 @@
 /* Copyright (C) 2006 MySQL AB & MySQL Finland AB & TCX DataKonsult AB
+   Copyright (c) 2020, MariaDB Corporation.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -675,8 +676,7 @@ static void update_record(uchar *record)
     ptr=blob_key;
     memcpy(pos+4,&ptr,sizeof(char*));	/* Store pointer to new key */
     if (keyinfo[0].seg[0].type != HA_KEYTYPE_NUM)
-      default_charset_info->cset->casedn(default_charset_info,
-                                         (char*) blob_key, length,
+      my_ci_casedn(default_charset_info, (char*) blob_key, length,
                                          (char*) blob_key, length);
     pos+=recinfo[0].length;
   }
@@ -684,16 +684,14 @@ static void update_record(uchar *record)
   {
     uint pack_length= HA_VARCHAR_PACKLENGTH(recinfo[0].length-1);
     uint length= pack_length == 1 ? (uint) *(uchar*) pos : uint2korr(pos);
-    default_charset_info->cset->casedn(default_charset_info,
-                                       (char*) pos + pack_length, length,
+    my_ci_casedn(default_charset_info, (char*) pos + pack_length, length,
                                        (char*) pos + pack_length, length);
     pos+=recinfo[0].length;
   }
   else
   {
     if (keyinfo[0].seg[0].type != HA_KEYTYPE_NUM)
-      default_charset_info->cset->casedn(default_charset_info,
-                                         (char*) pos, keyinfo[0].seg[0].length,
+      my_ci_casedn(default_charset_info, (char*) pos, keyinfo[0].seg[0].length,
                                          (char*) pos, keyinfo[0].seg[0].length);
     pos+=recinfo[0].length;
   }
