@@ -189,11 +189,11 @@ static void usage(void)
 
 
 static my_bool
-get_one_option(int optid __attribute__((unused)),
-               const struct my_option *opt __attribute__((unused)),
-               char *argument __attribute__((unused)))
+get_one_option(const struct my_option *opt,
+               char *argument __attribute__((unused)),
+               const char *filename __attribute__((unused)))
 {
-  switch (optid) {
+  switch (opt->id) {
   case '?':
     usage();
     exit(0);
