@@ -1,13 +1,20 @@
 /* Copyright (c) 2008, 2015, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; version 2 of the License.
+  it under the terms of the GNU General Public License, version 2.0,
+  as published by the Free Software Foundation.
+
+  This program is also distributed with certain software (including
+  but not limited to OpenSSL) that is licensed under separate terms,
+  as designated in a particular file or component or in included license
+  documentation.  The authors of MySQL hereby grant you an additional
+  permission to link the program and your derivative works with the
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
+  GNU General Public License, version 2.0, for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
@@ -444,22 +451,22 @@ public:
   ~PFS_internal_schema_access()
   {}
 
-  ACL_internal_access_result check(ulong want_access,
-                                   ulong *save_priv) const;
+  ACL_internal_access_result check(privilege_t want_access,
+                                   privilege_t *save_priv) const;
 
   const ACL_internal_table_access *lookup(const char *name) const;
 };
 
 ACL_internal_access_result
-PFS_internal_schema_access::check(ulong want_access,
-                                  ulong *save_priv)  const
+PFS_internal_schema_access::check(privilege_t want_access,
+                                  privilege_t *save_priv)  const
 {
-  const ulong always_forbidden= /* CREATE_ACL | */ REFERENCES_ACL
+  const privilege_t always_forbidden= /* CREATE_ACL | */ REFERENCES_ACL
     | INDEX_ACL | ALTER_ACL | CREATE_TMP_ACL | EXECUTE_ACL
     | CREATE_VIEW_ACL | SHOW_VIEW_ACL | CREATE_PROC_ACL | ALTER_PROC_ACL
     | EVENT_ACL | TRIGGER_ACL ;
 
-  if (unlikely(want_access & always_forbidden))
+  if (unlikely((want_access & always_forbidden) != NO_ACL))
     return ACL_INTERNAL_ACCESS_DENIED;
 
   /*
@@ -504,13 +511,13 @@ void initialize_performance_schema_acl(bool bootstrap)
 PFS_readonly_acl pfs_readonly_acl;
 
 ACL_internal_access_result
-PFS_readonly_acl::check(ulong want_access, ulong *save_priv) const
+PFS_readonly_acl::check(privilege_t want_access, privilege_t *save_priv) const
 {
-  const ulong always_forbidden= INSERT_ACL | UPDATE_ACL | DELETE_ACL
+  const privilege_t always_forbidden= INSERT_ACL | UPDATE_ACL | DELETE_ACL
     | /* CREATE_ACL | */ REFERENCES_ACL | INDEX_ACL | ALTER_ACL
     | CREATE_VIEW_ACL | SHOW_VIEW_ACL | TRIGGER_ACL | LOCK_TABLES_ACL;
 
-  if (unlikely(want_access & always_forbidden))
+  if (unlikely((want_access & always_forbidden) != NO_ACL))
     return ACL_INTERNAL_ACCESS_DENIED;
 
   return ACL_INTERNAL_ACCESS_CHECK_GRANT;
@@ -519,13 +526,13 @@ PFS_readonly_acl::check(ulong want_access, ulong *save_priv) const
 PFS_truncatable_acl pfs_truncatable_acl;
 
 ACL_internal_access_result
-PFS_truncatable_acl::check(ulong want_access, ulong *save_priv) const
+PFS_truncatable_acl::check(privilege_t want_access, privilege_t *save_priv) const
 {
-  const ulong always_forbidden= INSERT_ACL | UPDATE_ACL | DELETE_ACL
+  const privilege_t always_forbidden= INSERT_ACL | UPDATE_ACL | DELETE_ACL
     | /* CREATE_ACL | */ REFERENCES_ACL | INDEX_ACL | ALTER_ACL
     | CREATE_VIEW_ACL | SHOW_VIEW_ACL | TRIGGER_ACL | LOCK_TABLES_ACL;
 
-  if (unlikely(want_access & always_forbidden))
+  if (unlikely((want_access & always_forbidden) != NO_ACL))
     return ACL_INTERNAL_ACCESS_DENIED;
 
   return ACL_INTERNAL_ACCESS_CHECK_GRANT;
@@ -534,13 +541,13 @@ PFS_truncatable_acl::check(ulong want_access, ulong *save_priv) const
 PFS_updatable_acl pfs_updatable_acl;
 
 ACL_internal_access_result
-PFS_updatable_acl::check(ulong want_access, ulong *save_priv) const
+PFS_updatable_acl::check(privilege_t want_access, privilege_t *save_priv) const
 {
-  const ulong always_forbidden= INSERT_ACL | DELETE_ACL
+  const privilege_t always_forbidden= INSERT_ACL | DELETE_ACL
     | /* CREATE_ACL | */ REFERENCES_ACL | INDEX_ACL | ALTER_ACL
     | CREATE_VIEW_ACL | SHOW_VIEW_ACL | TRIGGER_ACL;
 
-  if (unlikely(want_access & always_forbidden))
+  if (unlikely((want_access & always_forbidden) != NO_ACL))
     return ACL_INTERNAL_ACCESS_DENIED;
 
   return ACL_INTERNAL_ACCESS_CHECK_GRANT;
@@ -549,12 +556,12 @@ PFS_updatable_acl::check(ulong want_access, ulong *save_priv) const
 PFS_editable_acl pfs_editable_acl;
 
 ACL_internal_access_result
-PFS_editable_acl::check(ulong want_access, ulong *save_priv) const
+PFS_editable_acl::check(privilege_t want_access, privilege_t *save_priv) const
 {
-  const ulong always_forbidden= /* CREATE_ACL | */ REFERENCES_ACL
+  const privilege_t always_forbidden= /* CREATE_ACL | */ REFERENCES_ACL
     | INDEX_ACL | ALTER_ACL | CREATE_VIEW_ACL | SHOW_VIEW_ACL | TRIGGER_ACL;
 
-  if (unlikely(want_access & always_forbidden))
+  if (unlikely((want_access & always_forbidden) != NO_ACL))
     return ACL_INTERNAL_ACCESS_DENIED;
 
   return ACL_INTERNAL_ACCESS_CHECK_GRANT;
@@ -563,13 +570,13 @@ PFS_editable_acl::check(ulong want_access, ulong *save_priv) const
 PFS_unknown_acl pfs_unknown_acl;
 
 ACL_internal_access_result
-PFS_unknown_acl::check(ulong want_access, ulong *save_priv) const
+PFS_unknown_acl::check(privilege_t want_access, privilege_t *save_priv) const
 {
-  const ulong always_forbidden= CREATE_ACL
+  const privilege_t always_forbidden= CREATE_ACL
     | REFERENCES_ACL | INDEX_ACL | ALTER_ACL
     | CREATE_VIEW_ACL | TRIGGER_ACL;
 
-  if (unlikely(want_access & always_forbidden))
+  if (unlikely((want_access & always_forbidden) != NO_ACL))
     return ACL_INTERNAL_ACCESS_DENIED;
 
   /*
