@@ -1,5 +1,5 @@
 /* Copyright (c) 2008, 2015, Oracle and/or its affiliates. All rights reserved.
-  Copyright (c) 2020, MariaDB Corporation.
+   Copyright (c) 2020, MariaDB Corporation.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -48,13 +48,6 @@
   @{
 */
 
-/**
-  Global performance schema flag.
-  Indicate if the performance schema is enabled.
-  This flag is set at startup, and never changes.
-*/
-my_bool pfs_enabled= TRUE;
-
 /**
   PFS_INSTRUMENT option settings array and associated state variable to
   serialize access during shutdown.
@@ -620,7 +613,6 @@ static void init_instr_class(PFS_instr_class *klass,
   DBUG_ASSERT(name_length <= PFS_MAX_INFO_NAME_LENGTH);
   memset(klass, 0, sizeof(PFS_instr_class));
   strncpy(klass->m_name, name, name_length);
-  klass->m_name[PFS_MAX_INFO_NAME_LENGTH - 1]= '\0';
   klass->m_name_length= name_length;
   klass->m_flags= flags;
   klass->m_enabled= true;
@@ -649,7 +641,7 @@ static void configure_instr_class(PFS_instr_class *entry)
 
       Consecutive wildcards affect the count.
     */
-    if (!my_wildcmp(&my_charset_latin1,
+    if (!my_charset_latin1.wildcmp(
                     entry->m_name, entry->m_name+entry->m_name_length,
                     e->m_name, e->m_name+e->m_name_length,
                     '\\', '?','%'))
@@ -1194,7 +1186,7 @@ PFS_instr_class *sanitize_idle_class(PFS_instr_class *unsafe)
 
 static void set_keys(PFS_table_share *pfs, const TABLE_SHARE *share)
 {
-  int len;
+  uint len;
   KEY *key_info= share->key_info;
   PFS_table_key *pfs_key= pfs->m_keys;
   PFS_table_key *pfs_key_last= pfs->m_keys + share->keys;
@@ -1202,8 +1194,8 @@ static void set_keys(PFS_table_share *pfs, const TABLE_SHARE *share)
 
   for ( ; pfs_key < pfs_key_last; pfs_key++, key_info++)
   {
-    len= strlen(key_info->name);
-    memcpy(pfs_key->m_name, key_info->name, len);
+    len= (uint)key_info->name.length;
+    memcpy(pfs_key->m_name, key_info->name.str, len);
     pfs_key->m_name_length= len;
   }
 
@@ -1224,11 +1216,11 @@ static int compare_keys(PFS_table_share *pfs, const TABLE_SHARE *share)
 
   for ( ; pfs_key < pfs_key_last; pfs_key++, key_info++)
   {
-    len= strlen(key_info->name);
+    len= (uint)key_info->name.length;
     if (len != pfs_key->m_name_length)
       return 1;
 
-    if (memcmp(pfs_key->m_name, key_info->name, len) != 0)
+    if (memcmp(pfs_key->m_name, key_info->name.str, len) != 0)
       return 1;
   }
 
@@ -1257,9 +1249,9 @@ PFS_table_share* find_or_create_table_share(PFS_thread *thread,
   }
 
   const char *schema_name= share->db.str;
-  uint schema_name_length= share->db.length;
+  uint schema_name_length= (uint)share->db.length;
   const char *table_name= share->table_name.str;
-  uint table_name_length= share->table_name.length;
+  uint table_name_length= (uint)share->table_name.length;
 
   set_table_share_key(&key, temporary,
                       schema_name, schema_name_length,
