@@ -1,5 +1,5 @@
 /* Copyright (c) 2003, 2013, Oracle and/or its affiliates
-   Copyright (c) 2009, 2016, MariaDB
+   Copyright (c) 2009, 2020, MariaDB
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -143,12 +143,12 @@ my_copy_incomplete_char(CHARSET_INFO *cs,
     
     Make sure we didn't pad to an incorrect character.
   */
-  if (cs->cset->charlen(cs, (uchar *) dst, (uchar *) dst + cs->mbminlen) ==
+  if (my_ci_charlen(cs, (uchar *) dst, (uchar *) dst + cs->mbminlen) ==
       (int) cs->mbminlen)
     return MY_CHAR_COPY_OK;
 
   if (fix &&
-      cs->cset->wc_mb(cs, '?', (uchar *) dst, (uchar *) dst + cs->mbminlen) ==
+      my_ci_wc_mb(cs, '?', (uchar *) dst, (uchar *) dst + cs->mbminlen) ==
       (int) cs->mbminlen)
     return MY_CHAR_COPY_FIXED;
 
@@ -733,7 +733,7 @@ my_l10tostr_mb2_or_mb4(CHARSET_INFO *cs,
   
   for ( db= dst, de= dst + len ; (dst < de) && *p ; p++)
   {
-    int cnvres= cs->cset->wc_mb(cs,(my_wc_t)p[0],(uchar*) dst, (uchar*) de);
+    int cnvres= my_ci_wc_mb(cs, (my_wc_t) p[0], (uchar*) dst, (uchar*) de);
     if (cnvres > 0)
       dst+= cnvres;
     else
@@ -796,7 +796,7 @@ cnv:
   
   for ( db= dst, de= dst + len ; (dst < de) && *p ; p++)
   {
-    int cnvres= cs->cset->wc_mb(cs, (my_wc_t) p[0], (uchar*) dst, (uchar*) de);
+    int cnvres= my_ci_wc_mb(cs, (my_wc_t) p[0], (uchar*) dst, (uchar*) de);
     if (cnvres > 0)
       dst+= cnvres;
     else
@@ -1057,7 +1057,7 @@ my_fill_mb2(CHARSET_INFO *cs, char *s, size_t slen, int fill)
 
   DBUG_ASSERT((slen % 2) == 0);
 
-  buflen= cs->cset->wc_mb(cs, (my_wc_t) fill, (uchar*) buf,
+  buflen= my_ci_wc_mb(cs, (my_wc_t) fill, (uchar*) buf,
                           (uchar*) buf + sizeof(buf));
 
   DBUG_ASSERT(buflen > 0);
@@ -1367,7 +1367,7 @@ static void
 my_hash_sort_utf16(CHARSET_INFO *cs, const uchar *s, size_t slen,
                    ulong *nr1, ulong *nr2)
 {
-  size_t lengthsp= cs->cset->lengthsp(cs, (const char *) s, slen);
+  size_t lengthsp= my_ci_lengthsp(cs, (const char *) s, slen);
   my_hash_sort_utf16_nopad(cs, s, lengthsp, nr1, nr2);
 }
 
@@ -1402,7 +1402,7 @@ static int
 my_charlen_utf16(CHARSET_INFO *cs, const uchar *str, const uchar *end)
 {
   my_wc_t wc;
-  return cs->cset->mb_wc(cs, &wc, str, end);
+  return my_ci_mb_wc(cs, &wc, str, end);
 }
 
 
@@ -1492,7 +1492,7 @@ static void
 my_hash_sort_utf16_bin(CHARSET_INFO *cs,
                        const uchar *pos, size_t len, ulong *nr1, ulong *nr2)
 {
-  size_t lengthsp= cs->cset->lengthsp(cs, (const char *) pos, len);
+  size_t lengthsp= my_ci_lengthsp(cs, (const char *) pos, len);
   my_hash_sort_utf16_nopad_bin(cs, pos, lengthsp, nr1, nr2);
 }
 
@@ -1733,7 +1733,7 @@ struct charset_info_st my_charset_utf16_nopad_bin=
 #define MY_FUNCTION_NAME(x)      my_ ## x ## _utf16le_general_ci
 #define DEFINE_STRNXFRM_UNICODE
 #define DEFINE_STRNXFRM_UNICODE_NOPAD
-#define MY_MB_WC(cs, pwc, s, e)  (cs->cset->mb_wc(cs, pwc, s, e))
+#define MY_MB_WC(cs, pwc, s, e)  (my_ci_mb_wc(cs, pwc, s, e))
 #define OPTIMIZE_ASCII           0
 #define UNICASE_MAXCHAR          MY_UNICASE_INFO_DEFAULT_MAXCHAR
 #define UNICASE_PAGE0            my_unicase_default_page00
@@ -2599,8 +2599,7 @@ void my_fill_utf32(CHARSET_INFO *cs,
 #ifdef DBUG_ASSERT_EXISTS
   buflen=
 #endif
-    cs->cset->wc_mb(cs, (my_wc_t) fill, (uchar*) buf,
-                    (uchar*) buf + sizeof(buf));
+    my_ci_wc_mb(cs, (my_wc_t) fill, (uchar*) buf, (uchar*) buf + sizeof(buf));
   DBUG_ASSERT(buflen == 4);
   while (s < e)
   {
