@@ -32,6 +32,14 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA */
 #include <shellapi.h>
 #include <stdlib.h>
 #include <winservice.h>
+#include <string>
+#include <iostream>
+#include <fstream>
+#include <sstream>
+#include <vector>
+#include <map>
+
+using namespace std;
 
 #define ONE_MB 1048576
 UINT ExecRemoveDataDirectory(wchar_t *dir)
@@ -56,7 +64,7 @@ UINT ExecRemoveDataDirectory(wchar_t *dir)
 }
 
 
-extern "C" UINT __stdcall RemoveDataDirectory(MSIHANDLE hInstall) 
+extern "C" UINT __stdcall RemoveDataDirectory(MSIHANDLE hInstall)
 {
   HRESULT hr = S_OK;
   UINT er = ERROR_SUCCESS;
@@ -72,13 +80,13 @@ extern "C" UINT __stdcall RemoveDataDirectory(MSIHANDLE hInstall)
   er= ExecRemoveDataDirectory(dir);
   WcaLog(LOGMSG_STANDARD, "SHFileOperation returned %d", er);
 LExit:
-  return WcaFinalize(er); 
+  return WcaFinalize(er);
 }
 
 /*
   Escape command line parameter fpr pass to CreateProcess().
 
-  We assume out has enough space to include encoded string 
+  We assume out has enough space to include encoded string
   2*wcslen(in) is enough.
 
   It is assumed that called will add double quotation marks before and after
@@ -106,21 +114,21 @@ static void EscapeCommandLine(const wchar_t *in, wchar_t *out, size_t buflen)
   }
 
   pos= 0;
-  for(int i = 0 ; ; i++) 
+  for(int i = 0 ; ; i++)
   {
     size_t n_backslashes = 0;
     wchar_t c;
-    while (in[i] == L'\\') 
+    while (in[i] == L'\\')
     {
       i++;
       n_backslashes++;
     }
 
     c= in[i];
-    if (c == 0) 
+    if (c == 0)
     {
       /*
-        Escape all backslashes, but let the terminating double quotation mark 
+        Escape all backslashes, but let the terminating double quotation mark
         that caller adds be interpreted as a metacharacter.
       */
       for(size_t j= 0; j < 2*n_backslashes;j++)
@@ -129,7 +137,7 @@ static void EscapeCommandLine(const wchar_t *in, wchar_t *out, size_t buflen)
       }
       break;
     }
-    else if (c == L'"') 
+    else if (c == L'"')
     {
       /*
         Escape all backslashes and the following double quotation mark.
@@ -140,7 +148,7 @@ static void EscapeCommandLine(const wchar_t *in, wchar_t *out, size_t buflen)
       }
       out[pos++]= L'"';
     }
-    else 
+    else
     {
       /* Backslashes aren't special here. */
       for (size_t j=0; j < n_backslashes; j++)
@@ -151,11 +159,11 @@ static void EscapeCommandLine(const wchar_t *in, wchar_t *out, size_t buflen)
   }
   out[pos++]= 0;
 }
-/* 
-  Check for if directory is empty during install, 
+/*
+  Check for if directory is empty during install,
   sets "<PROPERTY>_NOT_EMPTY" otherise
 */
-extern "C" UINT __stdcall CheckDirectoryEmpty(MSIHANDLE hInstall, 
+extern "C" UINT __stdcall CheckDirectoryEmpty(MSIHANDLE hInstall,
   const wchar_t *PropertyName)
 {
   HRESULT hr = S_OK;
@@ -165,14 +173,14 @@ extern "C" UINT __stdcall CheckDirectoryEmpty(MSIHANDLE hInstall,
   WIN32_FIND_DATAW data;
   HANDLE h;
   bool empty;
-  
+
   hr = WcaInitialize(hInstall, __FUNCTION__);
   ExitOnFailure(hr, "Failed to initialize");
   WcaLog(LOGMSG_STANDARD, "Initialized.");
 
   MsiGetPropertyW(hInstall, PropertyName, buf, &len);
   wcscat_s(buf, MAX_PATH, L"*.*");
-  
+
   WcaLog(LOGMSG_STANDARD, "Checking files in %S", buf);
 
   h= FindFirstFile(buf, &data);
@@ -198,7 +206,7 @@ extern "C" UINT __stdcall CheckDirectoryEmpty(MSIHANDLE hInstall,
   }
 
   if(empty)
-    WcaLog(LOGMSG_STANDARD, "Directory %S is empty or non-existent", 
+    WcaLog(LOGMSG_STANDARD, "Directory %S is empty or non-existent",
     PropertyName);
   else
     WcaLog(LOGMSG_STANDARD, "Directory %S is NOT empty", PropertyName);
@@ -208,7 +216,7 @@ extern "C" UINT __stdcall CheckDirectoryEmpty(MSIHANDLE hInstall,
   WcaSetProperty(buf, empty? L"":L"1");
 
 LExit:
-  return WcaFinalize(er); 
+  return WcaFinalize(er);
 }
 
 extern "C" UINT __stdcall CheckDataDirectoryEmpty(MSIHANDLE hInstall)
@@ -220,12 +228,12 @@ bool CheckServiceExists(const wchar_t *name)
 {
    SC_HANDLE manager =0, service=0;
    manager = OpenSCManager( NULL, NULL, SC_MANAGER_CONNECT);
-   if (!manager) 
+   if (!manager)
    {
      return false;
    }
 
-   service = OpenService(manager, name, SC_MANAGER_CONNECT); 
+   service = OpenService(manager, name, SC_MANAGER_CONNECT);
    if(service)
      CloseServiceHandle(service);
    CloseServiceHandle(manager);
@@ -239,11 +247,11 @@ bool ExecRemoveService(const wchar_t *name)
    SC_HANDLE manager =0, service=0;
    manager = OpenSCManager( NULL, NULL, SC_MANAGER_ALL_ACCESS);
    bool ret;
-   if (!manager) 
+   if (!manager)
    {
      return false;
    }
-   service = OpenService(manager, name, DELETE); 
+   service = OpenService(manager, name, DELETE);
    if(service)
    {
      ret= DeleteService(service);
@@ -289,7 +297,7 @@ bool IsPortFree(short port)
 }
 
 
-/* 
+/*
    Helper function used in filename normalization.
    Removes leading quote and terminates string at the position of the next one
    (if applicable, does not change string otherwise). Returns modified string
@@ -310,23 +318,23 @@ wchar_t *strip_quotes(wchar_t *s)
 /*
   Checks  for consistency of service configuration.
 
-  It can happen that SERVICENAME or DATADIR 
-  MSI properties are in inconsistent state after somebody upgraded database 
-  We catch this case during uninstall. In particular, either service is not 
+  It can happen that SERVICENAME or DATADIR
+  MSI properties are in inconsistent state after somebody upgraded database
+  We catch this case during uninstall. In particular, either service is not
   removed even if SERVICENAME was set (but this name is reused by someone else)
   or data directory is not removed (if it is used by someone else). To find out
-  whether service name and datadirectory are in use For every service, 
+  whether service name and datadirectory are in use For every service,
   configuration is read and checked as follows:
 
   - look if a service has to do something with mysql
-  - If so, check its name against SERVICENAME. if match, check binary path 
+  - If so, check its name against SERVICENAME. if match, check binary path
     against INSTALLDIR\bin.  If binary path does not match, then service runs
     under different  installation and won't be removed.
   - Check options file for datadir and look if this is inside this
     installation's datadir don't remove datadir if this is the case.
 
-  "Don't remove" in this context means that custom action is removing 
-  SERVICENAME property or CLEANUPDATA property, which later on in course of 
+  "Don't remove" in this context means that custom action is removing
+  SERVICENAME property or CLEANUPDATA property, which later on in course of
   installation mean, that either datadir or service is kept.
 */
 
@@ -367,15 +375,15 @@ void CheckServiceConfig(
   if(!is_my_service)
   {
     WcaLog(LOGMSG_STANDARD, "service does not match current service");
-    /* 
+    /*
       TODO probably the best thing possible would be to add temporary
       row to MSI ServiceConfig table with remove on uninstall
     */
   }
   else if (!same_bindir)
   {
-    WcaLog(LOGMSG_STANDARD, 
-      "Service name matches, but not the executable path directory, mine is %S", 
+    WcaLog(LOGMSG_STANDARD,
+      "Service name matches, but not the executable path directory, mine is %S",
       bindir);
     WcaSetProperty(L"SERVICENAME", L"");
   }
@@ -392,10 +400,10 @@ void CheckServiceConfig(
 
   WcaLog(LOGMSG_STANDARD, "parsed defaults file is %S", defaults_file);
 
-  if (GetPrivateProfileStringW(L"mysqld", L"datadir", NULL, current_datadir, 
+  if (GetPrivateProfileStringW(L"mysqld", L"datadir", NULL, current_datadir,
     MAX_PATH, defaults_file) == 0)
   {
-    WcaLog(LOGMSG_STANDARD, 
+    WcaLog(LOGMSG_STANDARD,
       "Cannot find datadir in ini file '%S'", defaults_file);
     goto end;
   }
@@ -404,18 +412,18 @@ void CheckServiceConfig(
   strip_quotes(current_datadir);
 
   /* Convert to Windows path */
-  if (GetFullPathNameW(current_datadir, MAX_PATH, normalized_current_datadir, 
+  if (GetFullPathNameW(current_datadir, MAX_PATH, normalized_current_datadir,
     NULL))
   {
     /* Add backslash to be compatible with directory formats in MSI */
     wcsncat(normalized_current_datadir, L"\\", MAX_PATH+1);
-    WcaLog(LOGMSG_STANDARD, "normalized current datadir is '%S'", 
+    WcaLog(LOGMSG_STANDARD, "normalized current datadir is '%S'",
       normalized_current_datadir);
   }
 
   if (_wcsicmp(datadir, normalized_current_datadir) == 0 && !same_bindir)
   {
-    WcaLog(LOGMSG_STANDARD, 
+    WcaLog(LOGMSG_STANDARD,
       "database directory from current installation, but different mysqld.exe");
     WcaSetProperty(L"CLEANUPDATA", L"");
   }
@@ -427,13 +435,13 @@ end:
 /*
   Checks if database directory or service are modified by user
   For example, service may point to different mysqld.exe that it was originally
-  installed, or some different service might use this database directory. This 
-  would normally mean user has done an upgrade of the database and in this case 
+  installed, or some different service might use this database directory. This
+  would normally mean user has done an upgrade of the database and in this case
   uninstall should neither delete service nor database directory.
 
-  If this function find that service is modified by user (mysqld.exe used by 
+  If this function find that service is modified by user (mysqld.exe used by
   service does not point to the installation bin directory), MSI public variable
-  SERVICENAME is removed, if DATADIR is used by some other service, variables 
+  SERVICENAME is removed, if DATADIR is used by some other service, variables
   DATADIR and CLEANUPDATA are removed.
 
   The effect of variable removal is that service does not get uninstalled and
@@ -453,11 +461,11 @@ extern "C" UINT CheckDBInUse(MSIHANDLE hInstall)
   wchar_t *datadir= NULL;
   wchar_t *bindir=NULL;
 
-  SC_HANDLE scm    = NULL; 
-  ULONG  bufsize   = sizeof(buf); 
-  ULONG  bufneed   = 0x00; 
-  ULONG  num_services = 0x00; 
-  LPENUM_SERVICE_STATUS_PROCESS info = NULL;  
+  SC_HANDLE scm    = NULL;
+  ULONG  bufsize   = sizeof(buf);
+  ULONG  bufneed   = 0x00;
+  ULONG  num_services = 0x00;
+  LPENUM_SERVICE_STATUS_PROCESS info = NULL;
   BOOL ok;
 
   hr = WcaInitialize(hInstall, __FUNCTION__);
@@ -467,48 +475,49 @@ extern "C" UINT CheckDBInUse(MSIHANDLE hInstall)
   WcaGetProperty(L"SERVICENAME", &servicename);
   WcaGetProperty(L"DATADIR", &datadir);
   WcaGetFormattedString(L"[INSTALLDIR]bin\\", &bindir);
+
   WcaLog(LOGMSG_STANDARD,"SERVICENAME=%S, DATADIR=%S, bindir=%S",
     servicename, datadir, bindir);
 
-  scm = OpenSCManager(NULL, NULL, 
-    SC_MANAGER_ENUMERATE_SERVICE | SC_MANAGER_CONNECT);  
-  if (scm == NULL) 
-  { 
+  scm = OpenSCManager(NULL, NULL,
+    SC_MANAGER_ENUMERATE_SERVICE | SC_MANAGER_CONNECT);
+  if (scm == NULL)
+  {
     ExitOnFailure(E_FAIL, "OpenSCManager failed");
   }
 
   ok = EnumServicesStatusExW(  scm,
-    SC_ENUM_PROCESS_INFO, 
+    SC_ENUM_PROCESS_INFO,
     SERVICE_WIN32,
-    SERVICE_STATE_ALL,  
-    buf, 
-    bufsize,  
-    &bufneed,  
-    &num_services,  
-    NULL, 
+    SERVICE_STATE_ALL,
+    buf,
+    bufsize,
+    &bufneed,
+    &num_services,
+    NULL,
     NULL);
-  if(!ok) 
+  if(!ok)
   {
     WcaLog(LOGMSG_STANDARD, "last error %d", GetLastError());
     ExitOnFailure(E_FAIL, "EnumServicesStatusExW failed");
   }
-  info = (LPENUM_SERVICE_STATUS_PROCESS)buf; 
+  info = (LPENUM_SERVICE_STATUS_PROCESS)buf;
   for (ULONG i=0; i < num_services; i++)
   {
-    SC_HANDLE service= OpenServiceW(scm, info[i].lpServiceName, 
+    SC_HANDLE service= OpenServiceW(scm, info[i].lpServiceName,
       SERVICE_QUERY_CONFIG);
     if (!service)
       continue;
     WcaLog(LOGMSG_VERBOSE, "Checking Service %S", info[i].lpServiceName);
-    QUERY_SERVICE_CONFIGW *config= 
+    QUERY_SERVICE_CONFIGW *config=
       (QUERY_SERVICE_CONFIGW *)(void *)config_buffer;
     DWORD needed;
-    BOOL ok= QueryServiceConfigW(service, config,sizeof(config_buffer), 
+    BOOL ok= QueryServiceConfigW(service, config,sizeof(config_buffer),
       &needed);
     CloseServiceHandle(service);
     if (ok)
     {
-       CheckServiceConfig(servicename, datadir, bindir, info[i].lpServiceName, 
+       CheckServiceConfig(servicename, datadir, bindir, info[i].lpServiceName,
          config);
     }
   }
@@ -520,18 +529,18 @@ LExit:
   ReleaseStr(servicename);
   ReleaseStr(datadir);
   ReleaseStr(bindir);
-  return WcaFinalize(er); 
+  return WcaFinalize(er);
 }
 
-/* 
+/*
   Get maximum size of the buffer process can allocate.
   this is calculated as min(RAM,virtualmemorylimit)
   For 32bit processes, virtual address memory is 2GB (x86 OS)
   or 4GB(x64 OS).
-  
-  Fragmentation due to loaded modules, heap  and stack 
+
+  Fragmentation due to loaded modules, heap  and stack
   limit maximum size of continuous memory block further,
-  so that limit for 32 bit process is about 1200 on 32 bit OS 
+  so that limit for 32 bit process is about 1200 on 32 bit OS
   or 2000 MB on 64 bit OS(found experimentally).
 */
 unsigned long long GetMaxBufferSize(unsigned long long totalPhys)
@@ -549,9 +558,9 @@ unsigned long long GetMaxBufferSize(unsigned long long totalPhys)
 
 
 /*
-  Checks SERVICENAME, PORT and BUFFERSIZE parameters 
+  Checks SERVICENAME, PORT and BUFFERSIZE parameters
 */
-extern "C" UINT  __stdcall CheckDatabaseProperties (MSIHANDLE hInstall) 
+extern "C" UINT  __stdcall CheckDatabaseProperties (MSIHANDLE hInstall)
 {
   wchar_t ServiceName[MAX_PATH]={0};
   wchar_t SkipNetworking[MAX_PATH]={0};
@@ -574,7 +583,7 @@ extern "C" UINT  __stdcall CheckDatabaseProperties (MSIHANDLE hInstall)
   ExitOnFailure(hr, "Failed to initialize");
   WcaLog(LOGMSG_STANDARD, "Initialized.");
 
- 
+
   MsiGetPropertyW (hInstall, L"SERVICENAME", ServiceName, &ServiceNameLen);
   if(ServiceName[0])
   {
@@ -585,10 +594,10 @@ extern "C" UINT  __stdcall CheckDatabaseProperties (MSIHANDLE hInstall)
     }
     for(DWORD i=0; i< ServiceNameLen;i++)
     {
-      if(ServiceName[i] == L'\\' || ServiceName[i] == L'/' 
+      if(ServiceName[i] == L'\\' || ServiceName[i] == L'/'
         || ServiceName[i]=='\'' || ServiceName[i] ==L'"')
       {
-        ErrorMsg = 
+        ErrorMsg =
           L"Invalid service name. Forward slash and back slash are forbidden."
           L"Single and double quotes are also not permitted.";
         goto LExit;
@@ -608,10 +617,10 @@ extern "C" UINT  __stdcall CheckDatabaseProperties (MSIHANDLE hInstall)
     sizeof(EscapedPassword)/sizeof(EscapedPassword[0]));
   MsiSetPropertyW(hInstall,L"ESCAPEDPASSWORD",EscapedPassword);
 
-  MsiGetPropertyW(hInstall, L"SKIPNETWORKING", SkipNetworking, 
+  MsiGetPropertyW(hInstall, L"SKIPNETWORKING", SkipNetworking,
     &SkipNetworkingLen);
   MsiGetPropertyW(hInstall, L"PORT", Port, &PortLen);
-  
+
   if(SkipNetworking[0]==0 && Port[0] != 0)
   {
     /* Strip spaces */
@@ -644,13 +653,13 @@ extern "C" UINT  __stdcall CheckDatabaseProperties (MSIHANDLE hInstall)
     short port = (short)_wtoi(Port);
     if (!IsPortFree(port))
     {
-      ErrorMsg = 
+      ErrorMsg =
         L"The TCP Port you selected is already in use. "
         L"Please choose a different port.";
       goto LExit;
     }
   }
-  
+
   MsiGetPropertyW (hInstall, L"STDCONFIG", QuickConfig, &QuickConfigLen);
   if(QuickConfig[0] !=0)
   {
@@ -660,7 +669,7 @@ extern "C" UINT  __stdcall CheckDatabaseProperties (MSIHANDLE hInstall)
 
      if (!GlobalMemoryStatusEx(&memstatus))
      {
-        WcaLog(LOGMSG_STANDARD, "Error %u from GlobalMemoryStatusEx", 
+        WcaLog(LOGMSG_STANDARD, "Error %u from GlobalMemoryStatusEx",
           GetLastError());
         er= ERROR_INSTALL_FAILURE;
         goto LExit;
@@ -714,7 +723,7 @@ LExit:
   return WcaFinalize(er);
 }
 
-/* 
+/*
   Sets Innodb buffer pool size (1/8 of RAM by default), if not already specified
   via command line.
   Calculates innodb log file size as min(100, innodb buffer pool size/4)
@@ -737,7 +746,7 @@ extern "C" UINT __stdcall PresetDatabaseProperties(MSIHANDLE hInstall)
 
   if (BufferPoolsizeParamLen && buff[0])
   {
-    WcaLog(LOGMSG_STANDARD, "BUFFERPOOLSIZE=%s, len=%u",buff, BufferPoolsizeParamLen);
+    WcaLog(LOGMSG_STANDARD, "BUFFERPOOLSIZE=%S, len=%u",buff, BufferPoolsizeParamLen);
     InnodbBufferPoolSize= _wtoi64(buff);
   }
   else
@@ -745,7 +754,7 @@ extern "C" UINT __stdcall PresetDatabaseProperties(MSIHANDLE hInstall)
     memstatus.dwLength = sizeof(memstatus);
     if (!GlobalMemoryStatusEx(&memstatus))
     {
-       WcaLog(LOGMSG_STANDARD, "Error %u from GlobalMemoryStatusEx", 
+       WcaLog(LOGMSG_STANDARD, "Error %u from GlobalMemoryStatusEx",
          GetLastError());
        er= ERROR_INSTALL_FAILURE;
        goto LExit;
@@ -754,14 +763,14 @@ extern "C" UINT __stdcall PresetDatabaseProperties(MSIHANDLE hInstall)
     /* Give innodb 12.5% of available physical memory. */
     InnodbBufferPoolSize= totalPhys/ONE_MB/8;
  #ifdef _M_IX86
-    /* 
+    /*
       For 32 bit processes, take virtual address space limitation into account.
       Do not try to use more than 3/4 of virtual address space, even if there
       is plenty of physical memory.
     */
     InnodbBufferPoolSize= min(GetMaxBufferSize(totalPhys)/ONE_MB*3/4,
       InnodbBufferPoolSize);
- #endif 
+ #endif
     swprintf_s(buff, L"%llu",InnodbBufferPoolSize);
     MsiSetPropertyW(hInstall, L"BUFFERPOOLSIZE", buff);
   }
@@ -816,7 +825,7 @@ static void DumpErrorLog(const wchar_t *dir)
 }
 
 /* Remove service and data directory created by CreateDatabase operation */
-extern "C" UINT __stdcall CreateDatabaseRollback(MSIHANDLE hInstall) 
+extern "C" UINT __stdcall CreateDatabaseRollback(MSIHANDLE hInstall)
 {
   HRESULT hr = S_OK;
   UINT er = ERROR_SUCCESS;
@@ -857,17 +866,17 @@ extern "C" UINT __stdcall CreateDatabaseRollback(MSIHANDLE hInstall)
     ExecRemoveDataDirectory(dir);
   }
 LExit:
-  return WcaFinalize(er); 
+  return WcaFinalize(er);
 }
 
 
 /*
-  Enables/disables optional "Launch upgrade wizard" checkbox at the end of 
+  Enables/disables optional "Launch upgrade wizard" checkbox at the end of
   installation
 */
 #define MAX_VERSION_PROPERTY_SIZE 64
 
-extern "C" UINT __stdcall CheckServiceUpgrades(MSIHANDLE hInstall) 
+extern "C" UINT __stdcall CheckServiceUpgrades(MSIHANDLE hInstall)
 {
   HRESULT hr = S_OK;
   UINT er = ERROR_SUCCESS;
@@ -884,7 +893,7 @@ extern "C" UINT __stdcall CheckServiceUpgrades(MSIHANDLE hInstall)
 
   hr = WcaInitialize(hInstall, __FUNCTION__);
    WcaLog(LOGMSG_STANDARD, "Initialized.");
-  if (MsiGetPropertyW(hInstall, L"ProductVersion", installerVersion, &size) 
+  if (MsiGetPropertyW(hInstall, L"ProductVersion", installerVersion, &size)
     != ERROR_SUCCESS)
   {
     hr = HRESULT_FROM_WIN32(GetLastError());
@@ -903,12 +912,12 @@ extern "C" UINT __stdcall CheckServiceUpgrades(MSIHANDLE hInstall)
     hr = HRESULT_FROM_WIN32(GetLastError());
     ExitOnFailure(hr, "MsiGetPropertyW failed");
   }
- 
 
-  scm = OpenSCManager(NULL, NULL, 
-    SC_MANAGER_ENUMERATE_SERVICE | SC_MANAGER_CONNECT);  
-  if (scm == NULL) 
-  { 
+
+  scm = OpenSCManager(NULL, NULL,
+    SC_MANAGER_ENUMERATE_SERVICE | SC_MANAGER_CONNECT);
+  if (scm == NULL)
+  {
     hr = HRESULT_FROM_WIN32(GetLastError());
     ExitOnFailure(hr,"OpenSCManager failed");
   }
@@ -921,7 +930,7 @@ extern "C" UINT __stdcall CheckServiceUpgrades(MSIHANDLE hInstall)
   DWORD num_services;
   ok= EnumServicesStatusExW(scm, SC_ENUM_PROCESS_INFO,  SERVICE_WIN32,
     SERVICE_STATE_ALL,  buf, bufsize,  &bufneed, &num_services, NULL, NULL);
-  if(!ok) 
+  if(!ok)
   {
     hr = HRESULT_FROM_WIN32(GetLastError());
     ExitOnFailure(hr,"EnumServicesStatusEx failed");
@@ -931,11 +940,11 @@ extern "C" UINT __stdcall CheckServiceUpgrades(MSIHANDLE hInstall)
   index=-1;
   for (ULONG i=0; i < num_services; i++)
   {
-    SC_HANDLE service= OpenServiceW(scm, info[i].lpServiceName, 
+    SC_HANDLE service= OpenServiceW(scm, info[i].lpServiceName,
       SERVICE_QUERY_CONFIG);
     if (!service)
       continue;
-    QUERY_SERVICE_CONFIGW *config= 
+    QUERY_SERVICE_CONFIGW *config=
       (QUERY_SERVICE_CONFIGW*)(void *)config_buffer;
     DWORD needed;
     ok= QueryServiceConfigW(service, config,sizeof(config_buffer),
@@ -946,7 +955,7 @@ extern "C" UINT __stdcall CheckServiceUpgrades(MSIHANDLE hInstall)
        mysqld_service_properties props;
        if (get_mysql_service_properties(config->lpBinaryPathName, &props))
                   continue;
-        /* 
+        /*
           Only look for services that have mysqld.exe outside of the current
           installation directory.
         */
@@ -955,7 +964,7 @@ extern "C" UINT __stdcall CheckServiceUpgrades(MSIHANDLE hInstall)
            WcaLog(LOGMSG_STANDARD, "found service %S, major=%d, minor=%d",
             info[i].lpServiceName, props.version_major, props.version_minor);
           if(props.version_major < installerMajorVersion
-            || (props.version_major == installerMajorVersion && 
+            || (props.version_major == installerMajorVersion &&
                 props.version_minor <= installerMinorVersion))
           {
             upgradableServiceFound= true;
@@ -979,7 +988,7 @@ extern "C" UINT __stdcall CheckServiceUpgrades(MSIHANDLE hInstall)
 LExit:
   if(scm)
     CloseServiceHandle(scm);
-  return WcaFinalize(er); 
+  return WcaFinalize(er);
 }
 
 
@@ -1003,3 +1012,269 @@ extern "C" BOOL WINAPI DllMain(
 
   return TRUE;
 }
+
+
+static wstring MakeExePath(const wchar_t *installDir, const wchar_t *basename)
+{
+  wstring path(installDir);
+  if (path.back() != L'\\')
+    path.append(L"\\");
+  path.append(L"bin\\");
+  path.append(basename);
+  path.append(L".exe");
+  return path;
+}
+
+static wstring MakeExePath(const wchar_t *installDir, string basename)
+{
+  wstring path(installDir);
+  if (path.back() != L'\\')
+    path.append(L"\\");
+  path.append(L"bin\\");
+  for (auto c : basename)
+    path+= c;
+  path.append(L".exe");
+  return path;
+}
+static wstring MakeFixSymlinksLogPath()
+{
+  wchar_t buf[MAX_PATH];
+  if (GetTempPathW(MAX_PATH, buf))
+    return wstring(buf) + L"\\mariadb_msi_fix_symlinks.log";
+  return L"";
+}
+
+static string w2s(wstring w)
+{
+  string s;
+  for (auto wc : w)
+    s += (char)wc;
+  return s;
+}
+
+/*
+  Remove symlinks if target file does not exist.
+  Create symlink if target exists, but symlink is not.
+*/
+static void fix_symlink(const wchar_t *file, const wchar_t *link,
+                        const wchar_t *installdir,
+                        vector<string> &rollback_actions)
+{
+  WcaLog(LOGMSG_STANDARD, "fix_symlink %S=>%S", link, file);
+
+  auto tgt= MakeExePath(installdir, file);
+  auto lnk= MakeExePath(installdir, link);
+  auto target_path= tgt.c_str();
+  auto link_path= lnk.c_str();
+
+  auto target_attr= GetFileAttributesW(target_path);
+  auto link_attr= GetFileAttributesW(link_path);
+  WcaLog(LOGMSG_STANDARD, "%S %s", target_path,
+         target_attr == INVALID_FILE_ATTRIBUTES ? "does not exist" : "exists");
+  WcaLog(LOGMSG_STANDARD, "%S %s", link_path,
+         link_attr == INVALID_FILE_ATTRIBUTES ? "does not exist" : "exists");
+
+  if (link_attr != INVALID_FILE_ATTRIBUTES &&
+      ((link_attr & FILE_ATTRIBUTE_REPARSE_POINT) == 0))
+  {
+    WcaLog(LOGMSG_STANDARD, "%S is not a symlink!", link_path);
+    return;
+  }
+
+  if (target_attr == INVALID_FILE_ATTRIBUTES)
+  {
+    if (link_attr == INVALID_FILE_ATTRIBUTES)
+    {
+      return;
+    }
+    auto ok= DeleteFileW(link_path);
+    WcaLog(LOGMSG_STANDARD, "DeleteFileW(L\"%S\") returned %s, last error %lu",
+           link_path, ok ? "success" : "error", GetLastError());
+    if (ok)
+    {
+      rollback_actions.push_back(string("create_symlink ") + w2s(link) + " " +
+                                 w2s(file));
+    }
+    return;
+  }
+
+  if (link_attr != INVALID_FILE_ATTRIBUTES)
+    return;
+
+  auto ok= CreateSymbolicLinkW(link_path, target_path, 0);
+  WcaLog(LOGMSG_STANDARD,
+         "CreateSymbolicLinkW(\"%S\",\"%S\", 0) returned %s, last error %d",
+         link_path, target_path, ok ? "success" : "error",
+         (int) GetLastError());
+  if (ok)
+    rollback_actions.push_back(string("delete ") + w2s(link));
+}
+
+static string rollback_info_path()
+{
+  char tmppath[MAX_PATH];
+  if (!GetTempPathA(MAX_PATH, tmppath))
+    return "";
+  string filepath(tmppath);
+  filepath.append("\\mariadb_symlink_rollback_info.tmp");
+  return filepath;
+}
+
+
+static void save_symlink_rollback_info(vector<string> rollback_info)
+{
+  std::ofstream ofs;
+  string path = rollback_info_path();
+  ofs.open(path, std::ofstream::out | std::ofstream::trunc);
+  if (!ofs.good())
+    return;
+  WcaLog(LOGMSG_STANDARD,
+    "Storing this rollback script for custom action in %s, in case installation rolls back",
+    path.c_str());
+  for (auto line : rollback_info)
+  {
+    WcaLog(LOGMSG_STANDARD, "%s", line.c_str());
+    ofs << line << "\n";
+  }
+  WcaLog(LOGMSG_STANDARD, "End of rollback script");
+}
+
+
+#include <symlinks.h>
+/* MDEV-19781 MariaDB symlinks on Windows */
+extern "C" UINT __stdcall FixSymlinksRollback(MSIHANDLE hInstall)
+{
+  HRESULT hr= S_OK;
+  UINT er= ERROR_SUCCESS;
+  wchar_t targetdir[MAX_PATH];
+  DWORD len= MAX_PATH;
+  hr= WcaInitialize(hInstall, __FUNCTION__);
+  WcaLog(LOGMSG_STANDARD, "Initialized.");
+  std::ifstream ifs;
+  std::string command;
+
+  if (MsiGetPropertyW(hInstall, L"CustomActionData", targetdir, &len) !=
+      ERROR_SUCCESS)
+  {
+    hr= HRESULT_FROM_WIN32(GetLastError());
+    ExitOnFailure(hr, "MsiGetPropertyW failed");
+  }
+
+  ifs.open(rollback_info_path(), std::ofstream::in);
+  if (!ifs.good())
+    goto LExit;
+
+  while (ifs >> command)
+  {
+    if (command == "create_symlink")
+    {
+      string link;
+      ifs >> link;
+      auto link_path= MakeExePath(targetdir, link);
+      string file;
+      ifs >> file;
+      auto target_path= MakeExePath(targetdir, file);
+      bool ok= CreateSymbolicLinkW(link_path.c_str(), target_path.c_str(), 0);
+      WcaLog(LOGMSG_STANDARD, "CreateSymbolicLinkW(%S,%S) %s, last error %lu",
+             link_path.c_str(), target_path.c_str(), ok ? "success" : "error",
+             GetLastError());
+    }
+    else if (command == "delete")
+    {
+      string link;
+      ifs >> link;
+      auto link_path= MakeExePath(targetdir, link);
+      auto link_attr= GetFileAttributesW(link_path.c_str());
+      if (link_attr != INVALID_FILE_ATTRIBUTES &&
+          (link_attr & FILE_ATTRIBUTE_REPARSE_POINT))
+      {
+        auto ok= DeleteFileW(link_path.c_str());
+        WcaLog(LOGMSG_STANDARD, "DeleteFile(%S) %s, last error %lu",
+               link_path.c_str(), ok ? "success" : "error", GetLastError());
+      }
+    }
+    else
+    {
+      WcaLog(LOGMSG_STANDARD, "Unknown command '%s' in rollback script ",
+             command.c_str());
+      goto LExit;
+    }
+  }
+
+LExit:
+  return WcaFinalize(er);
+}
+
+
+extern "C" UINT __stdcall SymlinksUninstall(MSIHANDLE hInstall)
+{
+  HRESULT hr = S_OK;
+  UINT er = ERROR_SUCCESS;
+  wchar_t targetdir[MAX_PATH];
+  DWORD len = MAX_PATH;
+  int i;
+  hr = WcaInitialize(hInstall, __FUNCTION__);
+  WcaLog(LOGMSG_STANDARD, "Initialized.");
+
+  if (MsiGetPropertyW(hInstall, L"CustomActionData", targetdir, &len) !=
+    ERROR_SUCCESS)
+  {
+    hr = HRESULT_FROM_WIN32(GetLastError());
+    ExitOnFailure(hr, "MsiGetPropertyW failed");
+  }
+
+  for (i = 0; all_symlinks[i].file; i++)
+  {
+    auto link_path = MakeExePath(targetdir, all_symlinks[i].link);
+    auto link_attr = GetFileAttributesW(link_path.c_str());
+    auto filepath = link_path.c_str();
+
+    if (link_attr == INVALID_FILE_ATTRIBUTES)
+    {
+      WcaLog(LOGMSG_STANDARD, " %S does not exist",filepath);
+      continue;
+    }
+
+    if(!(link_attr & FILE_ATTRIBUTE_REPARSE_POINT))
+    {
+      WcaLog(LOGMSG_STANDARD, " %S is not a symbolic link!",filepath);
+      continue;
+    }
+    BOOL ok = DeleteFileW(filepath);
+    WcaLog(LOGMSG_STANDARD, "DeleteFile(%S) %s, last error %lu",
+      filepath, ok? "succeeded":"failed", GetLastError());
+  }
+
+LExit:
+  return WcaFinalize(er);
+}
+
+/* MDEV-19781 MariaDB symlinks on Windows */
+extern "C" UINT __stdcall FixSymlinks(MSIHANDLE hInstall)
+{
+  HRESULT hr= S_OK;
+  UINT er= ERROR_SUCCESS;
+  wchar_t targetdir[MAX_PATH];
+  vector<string> rollback_actions;
+  DWORD len= MAX_PATH;
+  int i;
+  hr= WcaInitialize(hInstall, __FUNCTION__);
+  WcaLog(LOGMSG_STANDARD, "Initialized.");
+
+  if (MsiGetPropertyW(hInstall, L"CustomActionData", targetdir, &len) !=
+      ERROR_SUCCESS)
+  {
+    hr= HRESULT_FROM_WIN32(GetLastError());
+    ExitOnFailure(hr, "MsiGetPropertyW failed");
+  }
+
+  for (i= 0; all_symlinks[i].file; i++)
+    fix_symlink(all_symlinks[i].file, all_symlinks[i].link, targetdir,
+                rollback_actions);
+
+  save_symlink_rollback_info(rollback_actions);
+
+LExit:
+  return WcaFinalize(er);
+}
+
